@@ -1,7 +1,6 @@
-# -*- coding: utf-8 -*-
-#
+# COPYRIGHT (C) 2020 Nicotine+ Team
 # COPYRIGHT (C) 2016-2017 Michael Labouebe <gfarmerfr@free.fr>
-# COPYRIGHT (C) 2016 Mutnick <muhing@yahoo.com>
+# COPYRIGHT (C) 2016-2018 Mutnick <mutnick@techie.com>
 # COPYRIGHT (C) 2008-2011 Quinox <quinox@users.sf.net>
 # COPYRIGHT (C) 2009 Hedonist <ak@sensi.org>
 # COPYRIGHT (C) 2007 Gallows <g4ll0ws@gmail.com>
@@ -29,20 +28,28 @@
 This module contains configuration classes for Nicotine.
 """
 
-import ConfigParser
-import string
+import configparser
 import os
-import time
-import cPickle
-import bz2
+import pickle
 import shelve
 import sys
-import thread
-
+import time
+from ast import literal_eval
+from gettext import gettext as _
 from os.path import exists
 
-from logfacility import log
-from utils import findBestEncoding
+import _thread
+from pynicotine.logfacility import log
+
+
+class RestrictedUnpickler(pickle.Unpickler):
+    """
+    Don't allow code execution from pickles
+    """
+    def find_class(self, module, name):
+        # Forbid all globals
+        raise pickle.UnpicklingError("global '%s.%s' is forbidden" %
+                                     (module, name))
 
 
 class Config:
@@ -51,10 +58,10 @@ class Config:
     following methods:
 
     needConfig() - returns true if configuration information is incomplete
-    readConfig() - reads configuration information from ~/.nicotine/config
+    readConfig() - reads configuration information from file
     setConfig(config_info_dict) - sets configuration information
-    writeConfiguration - writes configuration information to ~/.nicotine/config
-    writeDownloadQueue - writes download queue to ~/.nicotine/config.transfers.pickle
+    writeConfiguration - writes configuration information to file
+    writeDownloadQueue - writes download queue to file
     writeConfig - calls writeConfiguration followed by writeDownloadQueue
 
     The actual configuration information is stored as a two-level dictionary.
@@ -62,16 +69,17 @@ class Config:
     parameters.
     """
 
-    def __init__(self, filename):
+    def __init__(self, filename, data_dir):
 
-        self.config_lock = thread.allocate_lock()
+        self.config_lock = _thread.allocate_lock()
         self.config_lock.acquire()
         self.frame = None
         self.filename = filename
-        self.parser = ConfigParser.ConfigParser()
+        self.data_dir = data_dir
+        self.parser = configparser.RawConfigParser()
         self.parser.read([self.filename])
 
-        LOGDIR = os.path.join(self.filename.rsplit(os.sep, 1)[0], "logs")
+        LOGDIR = os.path.join(data_dir, "logs")
 
         self.sections = {
             "server": {
@@ -82,12 +90,8 @@ class Config:
                 "ctcpmsgs": 0,
                 "autosearch": [],
                 "autoreply": "",
-                "roomencoding": {},
-                "fallbackencodings": ['utf-8', 'cp1252'],  # Put the multi-byte encodings up front - they are the most likely to err
-                "userencoding": {},
                 "portrange": (2234, 2239),
-                "upnp": False,
-                "enc": "utf-8",
+                "upnp": True,
                 "userlist": [],
                 "banlist": [],
                 "ignorelist": [],
@@ -99,7 +103,7 @@ class Config:
             },
 
             "transfers": {
-                "incompletedir": os.path.join(os.path.expanduser("~"), '.nicotine', 'incompletefiles'),
+                "incompletedir": os.path.join(data_dir, 'incompletefiles'),
                 "downloaddir": os.path.join(os.path.expanduser("~"), 'nicotine-downloads'),
                 "uploaddir": os.path.join(os.path.expanduser("~"), 'nicotine-uploads'),
                 "sharedownloaddir": 0,
@@ -123,21 +127,21 @@ class Config:
                 "usecustomban": 0,
                 "limitby": 1,
                 "customban": "Banned, don't bother retrying",
-                "queuelimit": 100,
+                "queuelimit": 10000,
                 "filelimit": 1000,
                 "friendsonly": 0,
                 "friendsnolimits": 0,
                 "enablebuddyshares": 0,
                 "enabletransferbuttons": 1,
-                "groupdownloads": 0,
-                "groupuploads": 1,
+                "groupdownloads": True,
+                "groupuploads": True,
                 "geoblock": 0,
                 "geopanic": 0,
                 "geoblockcc": [""],
                 "remotedownloads": 1,
                 "uploadallowed": 2,
+                "autoclear_downloads": 0,
                 "autoclear_uploads": 0,
-                "autoretry_downloads": 0,
                 "downloads": [],
                 "sharedfiles": {},
                 "sharedfilesstreams": {},
@@ -158,7 +162,7 @@ class Config:
                     ["folder.jpg", 1],
                     ["*.url", 1],
                     ["thumbs.db", 1],
-                    ["albumart(_{........-....-....-....-............}_)?(_?(large|small))?\.jpg", 0]
+                    ["albumart(_{........-....-....-....-............}_)?(_?(large|small))?\\.jpg", 0]
                 ],
                 "download_doubleclick": 1,
                 "upload_doubleclick": 1,
@@ -201,7 +205,7 @@ class Config:
                 "debug": False,
                 "debugmodes": [0, 1],
                 "logcollapsed": 0,
-                "logsdir": os.path.expanduser(LOGDIR),
+                "logsdir": LOGDIR,
                 "rooms_timestamp": "%H:%M:%S",
                 "private_timestamp": "%Y-%m-%d %H:%M:%S",
                 "log_timestamp": "%Y-%m-%d %H:%M:%S",
@@ -209,8 +213,8 @@ class Config:
                 "privatechat": 0,
                 "chatrooms": 0,
                 "transfers": 0,
-                "roomlogsdir": os.path.expanduser(os.path.join(LOGDIR, "rooms")),
-                "privatelogsdir": os.path.expanduser(os.path.join(LOGDIR, "private")),
+                "roomlogsdir": os.path.join(LOGDIR, "rooms"),
+                "privatelogsdir": os.path.join(LOGDIR, "private"),
                 "readroomlogs": 1,
                 "readroomlines": 15,
                 "readprivatelines": 15,
@@ -223,15 +227,24 @@ class Config:
             },
 
             "columns": {
+                "userbrowse": [1, 1, 1, 1],
+                "userbrowse_widths": [600, 100, 70, 0],
                 "userlist": [1, 1, 1, 1, 1, 1, 1, 1, 1, 1],
+                "userlist_widths": [0, 25, 120, 0, 0, 0, 0, 0, 160, 0],
                 "chatrooms": {},
-                "downloads": [1, 1, 1, 1, 1, 1, 1, 1, 1],
-                "uploads": [1, 1, 1, 1, 1, 1, 1, 1, 1],
-                "search": [1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1],
+                "chatrooms_widths": {},
+                "download_columns": [1, 1, 1, 1, 1, 1, 1, 1, 1, 1],
+                "download_widths": [200, 250, 250, 140, 50, 70, 170, 90, 140, 0],
+                "upload_columns": [1, 1, 1, 1, 1, 1, 1, 1, 1, 1],
+                "upload_widths": [200, 250, 250, 140, 50, 70, 170, 90, 140, 0],
+                "filesearch_columns": [1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1],
+                "filesearch_widths": [50, 200, 25, 20, 90, 50, 400, 400, 100, 50, 0],
                 "hideflags": False
             },
 
             "searches": {
+                "expand_searches": True,
+                "group_searches": True,
                 "maxresults": 50,
                 "re_filter": 0,
                 "history": [],
@@ -246,8 +259,9 @@ class Config:
                 "distrib_timer": 0,
                 "distrib_ignore": 60,
                 "search_results": 1,
-                "max_displayed_results": 500,
-                "max_stored_results": 1500
+                "max_displayed_results": 1000,
+                "max_stored_results": 1500,
+                "remove_special_chars": True
             },
 
             "ui": {
@@ -273,6 +287,7 @@ class Config:
                 "tab_default": "",
                 "tab_hilite": "red",
                 "tab_changed": "#0000ff",
+                "tab_select_previous": 1,
                 "tab_reorderable": 1,
                 "tabmain": "Top",
                 "tabrooms": "Top",
@@ -297,6 +312,7 @@ class Config:
                 "listfont": "",
                 "browserfont": "",
                 "transfersfont": "",
+                "last_tab_id": 0,
                 "modes_visible": {
                     "chatrooms": 1,
                     "private": 1,
@@ -324,16 +340,16 @@ class Config:
                 "trayicon": 1,
                 "soundenabled": 1,
                 "soundtheme": "",
-                "soundcommand": "play -q",
                 "filemanager": "xdg-open $",
                 "speechenabled": 0,
                 "speechprivate": "%(user)s told you.. %(message)s",
                 "speechrooms": "In %(room)s, %(user)s said %(message)s",
                 "speechcommand": "flite -t $",
-                "width": 800,
+                "width": 1000,
                 "height": 600,
                 "xposition": -1,
                 "yposition": -1,
+                "maximized": "True",
                 "urgencyhint": True
             },
 
@@ -344,7 +360,7 @@ class Config:
 
             "urls": {
                 "urlcatching": 1,
-                "protocols": {"http": "", "https": ""},
+                "protocols": {},
                 "humanizeurls": 1
             },
 
@@ -373,43 +389,26 @@ class Config:
             }
         }
 
-        # URls handling for Windows
-        if sys.platform.startswith('win'):
-            self.sections["urls"]["protocols"] = {
-                "http": "python -m webbrowser -t $",
-                "https": "python -m webbrowser -t $"
-            }
-
-        # URls handling for Linux
-        if sys.platform.startswith('linux'):
-            self.sections["urls"]["protocols"] = {
-                "http": "xdg-open $",
-                "https": "xdg-open $"
-            }
-
         # Windows specific stuff
         if sys.platform.startswith('win'):
             self.sections['ui']['filemanager'] = 'explorer $'
-            self.sections['transfers']['incompletedir'] = os.path.join(os.environ['APPDATA'], 'nicotine', 'incompletefiles')
-            self.sections['transfers']['downloaddir'] = os.path.join(os.environ['APPDATA'], 'nicotine', 'uploads')
-            self.sections['transfers']['uploaddir'] = os.path.join(os.environ['APPDATA'], 'nicotine', 'uploads')
 
         self.defaults = {}
-        for key, value in self.sections.items():
+        for key, value in list(self.sections.items()):
             if type(value) is dict:
                 if key not in self.defaults:
                     self.defaults[key] = {}
 
-                for key2, value2 in value.items():
+                for key2, value2 in list(value.items()):
                     self.defaults[key][key2] = value2
             else:
                 self.defaults[key] = value
 
         try:
-            f = open(filename+".alias")
-            self.aliases = cPickle.load(f)
+            f = open(filename + ".alias", 'rb')
+            self.aliases = RestrictedUnpickler(f).load()
             f.close()
-        except:
+        except Exception:
             self.aliases = {}
 
         self.config_lock.release()
@@ -419,8 +418,8 @@ class Config:
         errorlevel = 0
 
         try:
-            for i in self.sections.keys():
-                for j in self.sections[i].keys():
+            for i in list(self.sections.keys()):
+                for j in list(self.sections[i].keys()):
 
                     if type(self.sections[i][j]) not in [type(None), type("")]:
                         continue
@@ -451,16 +450,16 @@ class Config:
                                 )
                                 errorlevel = 2
 
-                            self.frame.logMessage(_("Config option unset: Section: %(section)s, Option: %(option)s") % {'section': i, 'option': j})
-                            self.frame.settingswindow.InvalidSettings(i, j)
+                            if self.frame.settingswindow is not None:
+                                self.frame.settingswindow.InvalidSettings(i, j)
 
-        except Exception, error:
+        except Exception as error:
             message = _("Config error: %s") % error
             self.frame.logMessage(message)
             if errorlevel < 3:
                 errorlevel = 3
 
-        if errorlevel > 1:
+        if errorlevel > 1 and self.frame.settingswindow is not None:
             self.frame.settingswindow.SetSettings(self.sections)
 
         return errorlevel
@@ -471,27 +470,33 @@ class Config:
 
         self.sections['transfers']['downloads'] = []
 
-        if exists(self.filename+'.transfers.pickle'):
+        if exists(os.path.join(self.data_dir, 'transfers.pickle')):
             # <1.2.13 stored transfers inside the main config
             try:
-                handle = open(self.filename+'.transfers.pickle')
-            except IOError, inst:
+                handle = open(os.path.join(self.data_dir, 'transfers.pickle'), 'rb')
+            except IOError as inst:
                 log.addwarning(_("Something went wrong while opening your transfer list: %(error)s") % {'error': str(inst)})
             else:
                 try:
-                    self.sections['transfers']['downloads'] = cPickle.load(handle)
-                except (IOError, EOFError, ValueError), inst:
+                    self.sections['transfers']['downloads'] = RestrictedUnpickler(handle).load()
+                except (IOError, EOFError, ValueError) as inst:
                     log.addwarning(_("Something went wrong while reading your transfer list: %(error)s") % {'error': str(inst)})
             try:
                 handle.close()
-            except:
+            except Exception:
                 pass
 
         path, fn = os.path.split(self.filename)
         try:
             if not os.path.isdir(path):
                 os.makedirs(path)
-        except OSError, msg:
+        except OSError as msg:
+            log.addwarning("Can't create directory '%s', reported error: %s" % (path, msg))
+
+        try:
+            if not os.path.isdir(self.data_dir):
+                os.makedirs(self.data_dir)
+        except OSError as msg:
             log.addwarning("Can't create directory '%s', reported error: %s" % (path, msg))
 
         # Transition from 1.2.16 -> 1.4.0
@@ -512,6 +517,30 @@ class Config:
         self.removeOldOption("language", "setlanguage")
         self.removeOldSection("language")
 
+        # Transition from 1.4.1 -> 1.4.2
+        self.removeOldOption("columns", "downloads")
+        self.removeOldOption("columns", "uploads")
+
+        # Remove old encoding settings (1.4.3)
+        self.removeOldOption("server", "enc")
+        self.removeOldOption("server", "fallbackencodings")
+        self.removeOldOption("server", "roomencoding")
+        self.removeOldOption("server", "userencoding")
+
+        # Remove soundcommand config, replaced by GSound (1.4.3)
+        self.removeOldOption("ui", "soundcommand")
+
+        # Remove old column widths in preparation for "group by folder"-feature
+        self.removeOldOption("columns", "search")
+        self.removeOldOption("columns", "search_widths")
+        self.removeOldOption("columns", "downloads_columns")
+        self.removeOldOption("columns", "downloads_widths")
+        self.removeOldOption("columns", "uploads_columns")
+        self.removeOldOption("columns", "uploads_widths")
+
+        # Remove auto-retry failed downloads-option, this is now default behavior
+        self.removeOldOption("transfers", "autoretry_downloads")
+
         # Checking for unknown section/options
         unknown1 = [
             'login', 'passw', 'enc', 'downloaddir', 'uploaddir', 'customban',
@@ -523,8 +552,8 @@ class Config:
 
         unknown2 = {
             'ui': [
-                "roomlistcollapsed", "tabclosers", "tab_colors",
-                "tab_reorderable", "buddylistinchatrooms", "trayicon",
+                "roomlistcollapsed", "tab_select_previous", "tabclosers",
+                "tab_colors", "tab_reorderable", "buddylistinchatrooms", "trayicon",
                 "showaway", "usernamehotspots", "exitdialog",
                 "tab_icons", "spellcheck", "modes_order", "modes_visible",
                 "chat_hidebuttons", "tab_status_icons", "notexists",
@@ -554,8 +583,8 @@ class Config:
                         self.sections[i][j] = None
                 else:
                     try:
-                        self.sections[i][j] = eval(val, {})
-                    except:
+                        self.sections[i][j] = literal_eval(val)
+                    except Exception:
                         self.sections[i][j] = None
                         log.addwarning("CONFIG ERROR: Couldn't decode '%s' section '%s' value '%s'" % (str(j), str(i), str(val)))
 
@@ -582,30 +611,30 @@ class Config:
         bsharedmtimes = None
 
         shelves = [
-            self.filename + ".files.db",
-            self.filename + ".buddyfiles.db",
-            self.filename + ".streams.db",
-            self.filename + ".buddystreams.db",
-            self.filename + ".wordindex.db",
-            self.filename + ".buddywordindex.db",
-            self.filename + ".fileindex.db",
-            self.filename + ".buddyfileindex.db",
-            self.filename + ".mtimes.db",
-            self.filename + ".buddymtimes.db"
+            os.path.join(self.data_dir, "files.db"),
+            os.path.join(self.data_dir, "buddyfiles.db"),
+            os.path.join(self.data_dir, "streams.db"),
+            os.path.join(self.data_dir, "buddystreams.db"),
+            os.path.join(self.data_dir, "wordindex.db"),
+            os.path.join(self.data_dir, "buddywordindex.db"),
+            os.path.join(self.data_dir, "fileindex.db"),
+            os.path.join(self.data_dir, "buddyfileindex.db"),
+            os.path.join(self.data_dir, "mtimes.db"),
+            os.path.join(self.data_dir, "buddymtimes.db")
         ]
 
         _opened_shelves = []
         _errors = []
         for shelvefile in shelves:
             try:
-                _opened_shelves.append(shelve.open(shelvefile))
-            except:
+                _opened_shelves.append(shelve.open(shelvefile, protocol=pickle.HIGHEST_PROTOCOL))
+            except Exception:
                 _errors.append(shelvefile)
                 try:
                     os.unlink(shelvefile)
-                    _opened_shelves.append(shelve.open(shelvefile, flag='n'))
-                except Exception, ex:
-                    print("Failed to unlink %s: %s" % (shelvefile, ex))
+                    _opened_shelves.append(shelve.open(shelvefile, flag='n', protocol=pickle.HIGHEST_PROTOCOL))
+                except Exception as ex:
+                    print(("Failed to unlink %s: %s" % (shelvefile, ex)))
 
         sharedfiles = _opened_shelves.pop(0)
         bsharedfiles = _opened_shelves.pop(0)
@@ -666,83 +695,83 @@ class Config:
             if sharedfiles:
                 sharedfiles.close()
             try:
-                os.unlink(self.filename + '.files.db')
-            except:
+                os.unlink(os.path.join(self.data_dir, 'files.db'))
+            except Exception:
                 pass
-            sharedfiles = shelve.open(self.filename + ".files.db", flag='n')
+            sharedfiles = shelve.open(os.path.join(self.data_dir, "files.db"), flag='n', protocol=pickle.HIGHEST_PROTOCOL)
 
             if bsharedfiles:
                 bsharedfiles.close()
             try:
-                os.unlink(self.filename + '.buddyfiles.db')
-            except:
+                os.unlink(os.path.join(self.data_dir, 'buddyfiles.db'))
+            except Exception:
                 pass
-            bsharedfiles = shelve.open(self.filename + ".buddyfiles.db", flag='n')
+            bsharedfiles = shelve.open(os.path.join(self.data_dir, "buddyfiles.db"), flag='n', protocol=pickle.HIGHEST_PROTOCOL)
 
             if sharedfilesstreams:
                 sharedfilesstreams.close()
             try:
-                os.unlink(self.filename + '.streams.db')
-            except:
+                os.unlink(os.path.join(self.data_dir, 'streams.db'))
+            except Exception:
                 pass
-            sharedfilesstreams = shelve.open(self.filename + ".streams.db", flag='n')
+            sharedfilesstreams = shelve.open(os.path.join(self.data_dir, "streams.db"), flag='n', protocol=pickle.HIGHEST_PROTOCOL)
 
             if bsharedfilesstreams:
                 bsharedfilesstreams.close()
             try:
-                os.unlink(self.filename + '.buddystreams.db')
-            except:
+                os.unlink(os.path.join(self.data_dir, 'buddystreams.db'))
+            except Exception:
                 pass
-            bsharedfilesstreams = shelve.open(self.filename + ".buddystreams.db", flag='n')
+            bsharedfilesstreams = shelve.open(os.path.join(self.data_dir, "buddystreams.db"), flag='n', protocol=pickle.HIGHEST_PROTOCOL)
 
             if wordindex:
                 wordindex.close()
             try:
-                os.unlink(self.filename + '.wordindex.db')
-            except:
+                os.unlink(os.path.join(self.data_dir, 'wordindex.db'))
+            except Exception:
                 pass
-            wordindex = shelve.open(self.filename + ".wordindex.db", flag='n')
+            wordindex = shelve.open(os.path.join(self.data_dir, "wordindex.db"), flag='n', protocol=pickle.HIGHEST_PROTOCOL)
 
             if bwordindex:
                 bwordindex.close()
             try:
-                os.unlink(self.filename + '.buddywordindex.db')
-            except:
+                os.unlink(os.path.join(self.data_dir, 'buddywordindex.db'))
+            except Exception:
                 pass
-            bwordindex = shelve.open(self.filename + ".buddywordindex.db", flag='n')
+            bwordindex = shelve.open(os.path.join(self.data_dir, "buddywordindex.db"), flag='n', protocol=pickle.HIGHEST_PROTOCOL)
 
             if fileindex:
                 fileindex.close()
             try:
-                os.unlink(self.filename + '.fileindex.db')
-            except:
+                os.unlink(os.path.join(self.data_dir, 'fileindex.db'))
+            except Exception:
                 pass
-            fileindex = shelve.open(self.filename + ".fileindex.db", flag='n')
+            fileindex = shelve.open(os.path.join(self.data_dir, "fileindex.db"), flag='n', protocol=pickle.HIGHEST_PROTOCOL)
 
             if bfileindex:
                 bfileindex.close()
             try:
-                os.unlink(self.filename + '.buddyfileindex.db')
-            except:
+                os.unlink(os.path.join(self.data_dir, 'buddyfileindex.db'))
+            except Exception:
                 pass
-            bfileindex = shelve.open(self.filename + ".buddyfileindex.db", flag='n')
+            bfileindex = shelve.open(os.path.join(self.data_dir, "buddyfileindex.db"), flag='n', protocol=pickle.HIGHEST_PROTOCOL)
 
             if sharedmtimes:
                 sharedmtimes.close()
             try:
-                os.unlink(self.filename + '.mtimes.db')
-            except:
+                os.unlink(os.path.join(self.data_dir, 'mtimes.db'))
+            except Exception:
                 pass
-            sharedmtimes = shelve.open(self.filename + ".mtimes.db", flag='n')
+            sharedmtimes = shelve.open(os.path.join(self.data_dir, "mtimes.db"), flag='n', protocol=pickle.HIGHEST_PROTOCOL)
 
             if bsharedmtimes:
                 bsharedmtimes.close()
             try:
-                os.unlink(self.filename + '.buddymtimes.db')
-            except:
+                os.unlink(os.path.join(self.data_dir, 'buddymtimes.db'))
+            except Exception:
                 pass
-            bsharedmtimes = shelve.open(self.filename + ".buddymtimes.db", flag='n')
-        except Exception, error:
+            bsharedmtimes = shelve.open(os.path.join(self.data_dir, "buddymtimes.db"), flag='n', protocol=pickle.HIGHEST_PROTOCOL)
+        except Exception as error:
             log.addwarning(_("Error while writing database files: %s") % error)
             return None
         return sharedfiles, bsharedfiles, sharedfilesstreams, bsharedfilesstreams, wordindex, bwordindex, fileindex, bfileindex, sharedmtimes, bsharedmtimes
@@ -753,34 +782,34 @@ class Config:
 
     def writeDownloadQueue(self):
         self.config_lock.acquire()
-        realfile = self.filename + '.transfers.pickle'
+        realfile = os.path.join(self.data_dir, 'transfers.pickle')
         tmpfile = realfile + '.tmp'
         backupfile = realfile + ' .backup'
         try:
-            handle = open(tmpfile, 'w')
-        except Exception, inst:
+            handle = open(tmpfile, 'wb')
+        except Exception as inst:
             log.addwarning(_("Something went wrong while opening your transfer list: %(error)s") % {'error': str(inst)})
         else:
             try:
-                cPickle.dump(self.sections['transfers']['downloads'], handle)
+                pickle.dump(self.sections['transfers']['downloads'], handle, protocol=pickle.HIGHEST_PROTOCOL)
                 handle.close()
                 try:
                     # Please let it be atomic...
                     os.rename(tmpfile, realfile)
-                except Exception, inst:
+                except Exception as inst:  # noqa: F841
                     # ...ugh. Okay, how about...
                     try:
                         os.unlink(backupfile)
-                    except:
+                    except Exception:
                         pass
                     os.rename(realfile, backupfile)
                     os.rename(tmpfile, realfile)
-            except Exception, inst:
+            except Exception as inst:
                 log.addwarning(_("Something went wrong while writing your transfer list: %(error)s") % {'error': str(inst)})
         finally:
             try:
                 handle.close()
-            except:
+            except Exception:
                 pass
         self.config_lock.release()
 
@@ -794,10 +823,10 @@ class Config:
             "bwordindex", "bfileindex", "bsharedmtimes", "downloads"
         ]
 
-        for i in self.sections.keys():
+        for i in list(self.sections.keys()):
             if not self.parser.has_section(i):
                 self.parser.add_section(i)
-            for j in self.sections[i].keys():
+            for j in list(self.sections[i].keys()):
                 if j not in external_sections:
                     self.parser.set(i, j, self.sections[i][j])
                 else:
@@ -807,21 +836,21 @@ class Config:
         try:
             if not os.path.isdir(path):
                 os.makedirs(path)
-        except OSError, msg:
+        except OSError as msg:
             log.addwarning(_("Can't create directory '%(path)s', reported error: %(error)s") % {'path': path, 'error': msg})
 
-        oldumask = os.umask(0077)
+        oldumask = os.umask(0o077)
 
         try:
             f = open(self.filename + ".new", "w")
-        except IOError, e:
+        except IOError as e:
             log.addwarning(_("Can't save config file, I/O error: %s") % e)
             self.config_lock.release()
             return
         else:
             try:
                 self.parser.write(f)
-            except IOError, e:
+            except IOError as e:
                 log.addwarning(_("Can't save config file, I/O error: %s") % e)
                 self.config_lock.release()
                 return
@@ -832,8 +861,8 @@ class Config:
 
         # A paranoid precaution since config contains the password
         try:
-            os.chmod(self.filename, 0600)
-        except:
+            os.chmod(self.filename, 0o600)
+        except Exception:
             pass
 
         try:
@@ -842,18 +871,18 @@ class Config:
                 try:
                     if os.path.exists(self.filename + ".old"):
                         os.remove(self.filename + ".old")
-                except OSError, error:
+                except OSError as error:  # noqa: F841
                     log.addwarning(_("Can't remove %s" % self.filename + ".old"))
                 try:
                     os.rename(self.filename, self.filename + ".old")
-                except OSError, error:
+                except OSError as error:
                     log.addwarning(_("Can't back config file up, error: %s") % error)
         except OSError:
             pass
 
         try:
             os.rename(self.filename + ".new", self.filename)
-        except OSError, error:
+        except OSError as error:
             log.addwarning(_("Can't rename config file, error: %s") % error)
 
         self.config_lock.release()
@@ -875,12 +904,12 @@ class Config:
             if not os.path.exists(self.filename):
                 raise BaseException("Config file missing")
             tar.add(self.filename)
-            if os.path.exists(self.filename+".alias"):
-                tar.add(self.filename+".alias")
+            if os.path.exists(self.filename + ".alias"):
+                tar.add(self.filename + ".alias")
 
             tar.close()
-        except Exception, e:
-            print e
+        except Exception as e:
+            print(e)
             self.config_lock.release()
             return (1, "Cannot write backup archive: %s" % e)
         self.config_lock.release()
@@ -889,11 +918,11 @@ class Config:
     def setBuddyShares(self, files, streams, wordindex, fileindex, mtimes):
 
         storable_objects = [
-                (files,     "bsharedfiles",        ".buddyfiles.db"),
-                (streams,   "bsharedfilesstreams", ".buddystreams.db"),
-                (mtimes,    "bsharedmtimes",       ".buddymtimes.db"),
-                (wordindex, "bwordindex",          ".buddywordindex.db"),
-                (fileindex, "bfileindex",          ".buddyfileindex.db")
+            (files, "bsharedfiles", "buddyfiles.db"),
+            (streams, "bsharedfilesstreams", "buddystreams.db"),
+            (mtimes, "bsharedmtimes", "buddymtimes.db"),
+            (wordindex, "bwordindex", "buddywordindex.db"),
+            (fileindex, "bfileindex", "buddyfileindex.db")
         ]
 
         self.config_lock.acquire()
@@ -903,11 +932,11 @@ class Config:
     def setShares(self, files, streams, wordindex, fileindex, mtimes):
 
         storable_objects = [
-                (files,     "sharedfiles",        ".files.db"),
-                (streams,   "sharedfilesstreams", ".streams.db"),
-                (mtimes,    "sharedmtimes",       ".mtimes.db"),
-                (wordindex, "wordindex",          ".wordindex.db"),
-                (fileindex, "fileindex",          ".fileindex.db")
+            (files, "sharedfiles", "files.db"),
+            (streams, "sharedfilesstreams", "streams.db"),
+            (mtimes, "sharedmtimes", "mtimes.db"),
+            (wordindex, "wordindex", "wordindex.db"),
+            (fileindex, "fileindex", "fileindex.db")
         ]
 
         self.config_lock.acquire()
@@ -916,13 +945,16 @@ class Config:
 
     def _storeObjects(self, storable_objects):
 
-        for (source, destination, prefix) in storable_objects:
+        for (source, destination, filename) in storable_objects:
+            try:
+                self.sections["transfers"][destination].close()
+                self.sections["transfers"][destination] = shelve.open(os.path.join(self.data_dir, filename), flag='n', protocol=pickle.HIGHEST_PROTOCOL)
 
-            self.sections["transfers"][destination].close()
-            self.sections["transfers"][destination] = shelve.open(self.filename + prefix, flag='n')
-
-            for (key, value) in source.iteritems():
-                self.sections["transfers"][destination][key] = value
+                for (key, value) in source.items():
+                    self.sections["transfers"][destination][key] = value
+            except Exception as e:
+                log.addwarning(_("Can't save %s: %s") % (filename, e))
+                return
 
     def writeShares(self):
 
@@ -952,9 +984,21 @@ class Config:
 
     def writeAliases(self):
         self.config_lock.acquire()
-        f = open(self.filename+".alias", "w")
-        cPickle.dump(self.aliases, f, 1)
-        f.close()
+        try:
+            f = open(self.filename + ".alias", "wb")
+        except Exception as e:
+            log.addwarning(_("Something went wrong while opening your alias file: %s") % e)
+        else:
+            try:
+                pickle.dump(self.aliases, f, protocol=pickle.HIGHEST_PROTOCOL)
+                f.close()
+            except Exception as e:
+                log.addwarning(_("Something went wrong while saving your alias file: %s") % e)
+        finally:
+            try:
+                f.close()
+            except Exception:
+                pass
         self.config_lock.release()
 
     def AddAlias(self, rest):
@@ -971,9 +1015,9 @@ class Config:
                 return _("No such alias (%s)") % rest + "\n"
         else:
             m = "\n" + _("Aliases:") + "\n"
-            for (key, value) in self.aliases.iteritems():
+            for (key, value) in self.aliases.items():
                 m = m + "%s: %s\n" % (key, value)
-            return m+"\n"
+            return m + "\n"
 
     def Unalias(self, rest):
         if rest and rest in self.aliases:
