@@ -1,7 +1,6 @@
-# -*- coding: utf-8 -*-
-#
+# COPYRIGHT (C) 2020 Nicotine+ Team
 # COPYRIGHT (C) 2016-2017 Michael Labouebe <gfarmerfr@free.fr>
-# COPYRIGHT (C) 2016 Mutnick <muhing@yahoo.com>
+# COPYRIGHT (C) 2016-2018 Mutnick <mutnick@techie.com>
 # COPYRIGHT (C) 2013 eL_vErDe <gandalf@le-vert.net>
 # COPYRIGHT (C) 2008-2012 Quinox <quinox@users.sf.net>
 # COPYRIGHT (C) 2009 Hedonist <ak@sensi.org>
@@ -23,29 +22,34 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
-
-import gtk
-from thread import start_new_thread
-
-from transferlist import TransferList
-from utils import PopupMenu, PressHeader, HumanizeBytes
-from pynicotine import slskmessages
-import string
 import os
+from gettext import gettext as _
+
+from gi.repository import Gdk
+from gi.repository import Gtk as gtk
+
+from _thread import start_new_thread
+from pynicotine import slskmessages
+from pynicotine.gtkgui.entrydialog import MetaDialog
+from pynicotine.gtkgui.entrydialog import OptionDialog
+from pynicotine.gtkgui.transferlist import TransferList
+from pynicotine.gtkgui.utils import CollapseTreeview
+from pynicotine.gtkgui.utils import FillFileGroupingCombobox
+from pynicotine.gtkgui.utils import HumanSize
+from pynicotine.gtkgui.utils import PopupMenu
+from pynicotine.gtkgui.utils import SetTreeviewSelectedRow
 from pynicotine.utils import executeCommand
-from entrydialog import MetaDialog, OptionDialog
 
 
 class Downloads(TransferList):
 
     def __init__(self, frame):
 
-        TransferList.__init__(self, frame, frame.DownloadList)
+        TransferList.__init__(self, frame, frame.DownloadList, type='download')
         self.myvbox = self.frame.downloadsvbox
-        self.frame.DownloadList.set_property("rules-hint", True)
-        self.accel_group = gtk.AccelGroup()
 
-        self.popup_menu2 = popup2 = PopupMenu(frame)
+        self.popup_menu_users = PopupMenu(self.frame, False)
+        self.popup_menu_clear = popup2 = PopupMenu(self.frame, False)
         popup2.setup(
             ("#" + _("Clear finished/aborted"), self.OnClearFinishedAborted),
             ("#" + _("Clear finished"), self.OnClearFinished),
@@ -54,7 +58,6 @@ class Downloads(TransferList):
             ("#" + _("Clear filtered"), self.OnClearFiltered),
             ("#" + _("Clear queued"), self.OnClearQueued)
         )
-        self.popup_menu_users = PopupMenu(frame)
 
         self.popup_menu = popup = PopupMenu(frame)
         popup.setup(
@@ -74,22 +77,9 @@ class Downloads(TransferList):
             ("#" + _("Abort & Delete"), self.OnAbortRemoveTransfer),
             ("#" + _("_Clear"), self.OnClearTransfer),
             ("", None),
-            (1, _("Clear Groups"), self.popup_menu2, None)
+            (1, _("Clear Groups"), self.popup_menu_clear, None)
         )
 
-        frame.DownloadList.connect("button_press_event", self.OnPopupMenu, "mouse")
-        frame.DownloadList.connect("key-press-event", self.on_key_press_event)
-        cols = frame.DownloadList.get_columns()
-
-        for i in range(9):
-
-            parent = cols[i].get_widget().get_ancestor(gtk.Button)
-            if parent:
-                parent.connect("button_press_event", PressHeader)
-
-            # Read Show / Hide column settings from last session
-            cols[i].set_visible(self.frame.np.config.sections["columns"]["downloads"][i])
-
         frame.clearFinishedAbortedButton.connect("clicked", self.OnClearFinishedAborted)
         frame.clearQueuedButton.connect("clicked", self.OnTryClearQueued)
         frame.retryTransferButton.connect("clicked", self.OnRetryTransfer)
@@ -98,28 +88,18 @@ class Downloads(TransferList):
         frame.banDownloadButton.connect("clicked", self.OnBan)
         frame.DownloadList.expand_all()
 
-        self.frame.ToggleAutoRetry.set_active(self.frame.np.config.sections["transfers"]["autoretry_downloads"])
-        frame.ToggleAutoRetry.connect("toggled", self.OnToggleAutoRetry)
+        self.frame.ToggleAutoclearDownloads.set_active(self.frame.np.config.sections["transfers"]["autoclear_downloads"])
+        frame.ToggleAutoclearDownloads.connect("toggled", self.OnToggleAutoclear)
 
-        self.frame.ToggleTreeDownloads.set_active(self.frame.np.config.sections["transfers"]["groupdownloads"])
-        frame.ToggleTreeDownloads.connect("toggled", self.OnToggleTree)
+        FillFileGroupingCombobox(frame.ToggleTreeDownloads)
+        frame.ToggleTreeDownloads.set_active(self.frame.np.config.sections["transfers"]["groupdownloads"])
+        frame.ToggleTreeDownloads.connect("changed", self.OnToggleTree)
         self.OnToggleTree(None)
 
         self.frame.ExpandDownloads.set_active(self.frame.np.config.sections["transfers"]["downloadsexpanded"])
         frame.ExpandDownloads.connect("toggled", self.OnExpandDownloads)
         self.OnExpandDownloads(None)
 
-    def saveColumns(self):
-
-        columns = []
-        for column in self.frame.DownloadList.get_columns():
-            columns.append(column.get_visible())
-
-        self.frame.np.config.sections["columns"]["downloads"] = columns
-
-    def OnToggleAutoRetry(self, widget):
-        self.frame.np.config.sections["transfers"]["autoretry_downloads"] = self.frame.ToggleAutoRetry.get_active()
-
     def OnTryClearQueued(self, widget):
 
         direction = "down"
@@ -129,12 +109,11 @@ class Downloads(TransferList):
         win.set_icon(self.frame.images["n"])
         win.show()
 
-    def expandcollapse(self, path):
-
+    def expand(self, path):
         if self.frame.ExpandDownloads.get_active():
-            self.frame.DownloadList.expand_row(path, True)
+            self.frame.DownloadList.expand_to_path(path)
         else:
-            self.frame.DownloadList.collapse_row(path)
+            CollapseTreeview(self.frame.DownloadList, self.TreeUsers)
 
     def OnExpandDownloads(self, widget):
 
@@ -144,18 +123,20 @@ class Downloads(TransferList):
             self.frame.DownloadList.expand_all()
             self.frame.ExpandDownloadsImage.set_from_stock(gtk.STOCK_REMOVE, 4)
         else:
-            self.frame.DownloadList.collapse_all()
+            CollapseTreeview(self.frame.DownloadList, self.TreeUsers)
             self.frame.ExpandDownloadsImage.set_from_stock(gtk.STOCK_ADD, 4)
 
         self.frame.np.config.sections["transfers"]["downloadsexpanded"] = expanded
         self.frame.np.config.writeConfiguration()
 
-    def OnToggleTree(self, widget):
+    def OnToggleAutoclear(self, widget):
+        self.frame.np.config.sections["transfers"]["autoclear_downloads"] = self.frame.ToggleAutoclearDownloads.get_active()
 
+    def OnToggleTree(self, widget):
         self.TreeUsers = self.frame.ToggleTreeDownloads.get_active()
         self.frame.np.config.sections["transfers"]["groupdownloads"] = self.TreeUsers
 
-        if not self.TreeUsers:
+        if self.TreeUsers == 0:
             self.frame.ExpandDownloads.hide()
         else:
             self.frame.ExpandDownloads.show()
@@ -164,7 +145,7 @@ class Downloads(TransferList):
 
     def MetaBox(self, title="Meta Data", message="", data=None, modal=True, Search=False):
 
-        win = MetaDialog(self.frame, message,  data, modal, Search=Search)
+        win = MetaDialog(self.frame, message, data, modal, Search=Search)
         win.set_title(title)
         win.set_icon(self.frame.images["n"])
         win.show()
@@ -177,19 +158,19 @@ class Downloads(TransferList):
             return
 
         user = model.get_value(iter, 0)
-        filename = model.get_value(iter, 1)
+        filename = model.get_value(iter, 2)
         fullname = model.get_value(iter, 10)
         size = speed = "0"
-        length = bitrate = None
+        length = bitrate = None  # noqa: F841
         queue = immediate = num = country = bitratestr = ""
 
         for transfer in self.frame.np.transfers.downloads:
             if transfer.user == user and fullname == transfer.filename:
-                size = HumanizeBytes(transfer.size)
+                size = HumanSize(transfer.size)
                 try:
                     speed = str(int(transfer.speed))
                     speed += _(" KB/s")
-                except:
+                except Exception:
                     pass
                 bitratestr = str(transfer.bitrate)
                 length = str(transfer.length)
@@ -235,8 +216,8 @@ class Downloads(TransferList):
 
         complete_path = os.path.join(downloaddir, transfer.path)
 
-        if transfer.path is "":
-            if transfer.status is "Finished":
+        if transfer.path == "":
+            if transfer.status == "Finished":
                 executeCommand(filemanager, downloaddir)
             else:
                 executeCommand(filemanager, incompletedir)
@@ -245,51 +226,13 @@ class Downloads(TransferList):
         else:
             executeCommand(filemanager, incompletedir)
 
-    def RebuildTransfers(self):
-
-        if self.frame.np.transfers is None:
-            return
-
-        self.Clear()
-        self.update()
-
-    def select_transfers(self):
-        self.selected_transfers = []
-        self.selected_users = []
-        self.widget.get_selection().selected_foreach(self.SelectedTransfersCallback)
-
-    def OnBan(self, widgets):
-        self.select_transfers()
-        for user in self.selected_users:
-            self.frame.BanUser(user)
-
     def OnSelectAbortTransfer(self, widget):
         self.select_transfers()
         self.OnAbortTransfer(widget, False)
 
-    def OnSelectUserTransfer(self, widget):
-
-        if len(self.selected_users) == 0:
-            return
-
-        selected_user = widget.parent.user
-
-        sel = self.frame.DownloadList.get_selection()
-        fmodel = self.frame.DownloadList.get_model()
-        sel.unselect_all()
-
-        for item in self.transfers:
-            user_file, iter, transfer = item
-            user, filepath = user_file
-            if selected_user == user:
-                ix = fmodel.get_path(iter)
-                sel.select_path(ix,)
-
-        self.select_transfers()
-
     def on_key_press_event(self, widget, event):
 
-        key = gtk.gdk.keyval_name(event.keyval)
+        key = Gdk.keyval_name(event.keyval)
 
         if key in ("P", "p"):
             self.OnPopupMenu(widget, event, "keyboard")
@@ -302,6 +245,12 @@ class Downloads(TransferList):
                 self.OnRetryTransfer(widget)
             elif key == "Delete":
                 self.OnAbortTransfer(widget, True, True)
+            else:
+                # No key match, continue event
+                return False
+
+        widget.stop_emission_by_name("key_press_event")
+        return True
 
     def OnPlayFiles(self, widget, prefix=""):
         start_new_thread(self._OnPlayFiles, (widget, prefix))
@@ -325,7 +274,7 @@ class Downloads(TransferList):
             else:
                 # If this file doesn't exist anymore, it may have finished downloading and have been renamed
                 # try looking in the download directory and match the original filename.
-                basename = string.split(fn.filename, '\\')[-1]
+                basename = str.split(fn.filename, '\\')[-1]
                 path = os.sep.join([downloaddir, basename])
                 if os.path.exists(path):
                     playfile = path
@@ -333,69 +282,6 @@ class Downloads(TransferList):
             if playfile:
                 executeCommand(executable, playfile, background=False)
 
-    def OnPopupMenuUsers(self, widget):
-
-        self.selected_transfers = []
-        self.selected_users = []
-        self.widget.get_selection().selected_foreach(self.SelectedTransfersCallback)
-
-        self.popup_menu_users.clear()
-
-        if len(self.selected_users) > 0:
-
-            items = []
-            self.selected_users.sort(key=str.lower)
-
-            for user in self.selected_users:
-
-                popup = PopupMenu(self.frame)
-                popup.setup(
-                    ("#" + _("Send _message"), popup.OnSendMessage),
-                    ("#" + _("Show IP a_ddress"), popup.OnShowIPaddress),
-                    ("#" + _("Get user i_nfo"), popup.OnGetUserInfo),
-                    ("#" + _("Brow_se files"), popup.OnBrowseUser),
-                    ("#" + _("Gi_ve privileges"), popup.OnGivePrivileges),
-                    ("", None),
-                    ("$" + _("_Add user to list"), popup.OnAddToList),
-                    ("$" + _("_Ban this user"), popup.OnBanUser),
-                    ("$" + _("_Ignore this user"), popup.OnIgnoreUser),
-                    ("#" + _("Select User's Transfers"), self.OnSelectUserTransfer)
-                )
-                popup.set_user(user)
-
-                items.append((1, user, popup, self.OnPopupMenuUser, popup))
-
-            self.popup_menu_users.setup(*items)
-
-        return True
-
-    def OnPopupMenuUser(self, widget, popup=None):
-
-        if popup is None:
-            return
-
-        menu = popup
-        user = menu.user
-        items = menu.get_children()
-
-        act = False
-        if len(self.selected_users) >= 1:
-            act = True
-
-        items[0].set_sensitive(act)
-        items[1].set_sensitive(act)
-        items[2].set_sensitive(act)
-        items[3].set_sensitive(act)
-
-        items[6].set_active(user in [i[0] for i in self.frame.np.config.sections["server"]["userlist"]])
-        items[7].set_active(user in self.frame.np.config.sections["server"]["banlist"])
-        items[8].set_active(user in self.frame.np.config.sections["server"]["ignorelist"])
-
-        for i in range(4, 9):
-            items[i].set_sensitive(act)
-
-        return True
-
     def DoubleClick(self, event):
 
         self.select_transfers()
@@ -418,26 +304,24 @@ class Downloads(TransferList):
 
         if kind == "mouse":
             if event.button != 3:
-                if event.button == 1 and event.type == gtk.gdk._2BUTTON_PRESS:
+                if event.button == 1 and event.type == Gdk.EventType._2BUTTON_PRESS:
                     self.DoubleClick(event)
                 return False
 
-        self.selected_transfers = []
-        self.selected_users = []
-        self.widget.get_selection().selected_foreach(self.SelectedTransfersCallback)
+            SetTreeviewSelectedRow(widget, event)
+
+        self.select_transfers()
 
         users = len(self.selected_users) > 0
-        multi_users = len(self.selected_users) > 1
+        multi_users = len(self.selected_users) > 1  # noqa: F841
         files = len(self.selected_transfers) > 0
         multi_files = len(self.selected_transfers) > 1
 
-        self.SelectCurrentRow(event, kind)
-
         items = self.popup_menu.get_children()
         if users:
-            items[7].set_sensitive(True)  # Users Menu
+            items[8].set_sensitive(True)  # Users Menu
         else:
-            items[7].set_sensitive(False)  # Users Menu
+            items[8].set_sensitive(False)  # Users Menu
 
         if files:
             act = True
@@ -448,7 +332,7 @@ class Downloads(TransferList):
         items[4].set_sensitive(act)  # Send to player
         items[5].set_sensitive(act)  # View Meta
         items[6].set_sensitive(act)  # File manager
-        items[8].set_sensitive(act)  # Search filename
+        items[7].set_sensitive(act)  # Search filename
 
         act = False
         if not multi_files and files:
@@ -467,21 +351,15 @@ class Downloads(TransferList):
         for i in range(10, 15):
             items[i].set_sensitive(act)
 
-        self.popup_menu.popup(None, None, None, 3, event.time)
+        self.popup_menu.popup(None, None, None, None, 3, event.time)
 
         if kind == "keyboard":
-            widget.emit_stop_by_name("key_press_event")
+            widget.stop_emission_by_name("key_press_event")
         elif kind == "mouse":
-            widget.emit_stop_by_name("button_press_event")
+            widget.stop_emission_by_name("button_press_event")
 
         return True
 
-    def update(self, transfer=None, forced=False):
-
-        TransferList.update(self, transfer, forced)
-        if transfer is None and self.frame.np.transfers is not None:
-            self.frame.np.transfers.SaveDownloads()
-
     def OnGetPlaceInQueue(self, widget):
 
         self.select_transfers()
@@ -491,15 +369,6 @@ class Downloads(TransferList):
                 continue
             self.frame.np.ProcessRequestToPeer(i.user, slskmessages.PlaceInQueueRequest(None, i.filename))
 
-    def OnFileSearch(self, widget):
-
-        self.select_transfers()
-
-        for transfer in self.selected_transfers:
-            self.frame.SearchEntry.set_text(transfer.filename.rsplit("\\", 1)[1])
-            self.frame.ChangeMainPage(None, "search")
-            break
-
     def OnRetryTransfer(self, widget):
 
         self.select_transfers()
@@ -510,11 +379,8 @@ class Downloads(TransferList):
                 continue
 
             self.frame.np.transfers.AbortTransfer(transfer)
-            transfer.req = None
             self.frame.np.transfers.getFile(transfer.user, transfer.filename, transfer.path, transfer)
 
-        self.frame.np.transfers.SaveDownloads()
-
     def OnAbortRemoveTransfer(self, widget):
         self.select_transfers()
         self.OnClearTransfer(widget)
