@@ -1,5 +1,4 @@
-# -*- coding: utf-8 -*-
-#
+# COPYRIGHT (C) 2020 Nicotine+ Team
 # COPYRIGHT (C) 2016-2017 Michael Labouebe <gfarmerfr@free.fr>
 # COPYRIGHT (C) 2008-2009 Quinox <quinox@users.sf.net>
 # COPYRIGHT (C) 2006-2009 Daelstorm <daelstorm@gmail.com>
@@ -20,9 +19,11 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-import gtk
-import gobject
-import os
+from gettext import gettext as _
+
+from gi.repository import Gdk
+from gi.repository import GObject as gobject
+from gi.repository import Gtk as gtk
 
 
 class MetaDialog(gtk.Dialog):
@@ -34,6 +35,8 @@ class MetaDialog(gtk.Dialog):
         self.connect("delete-event", self.quit)
         self.nicotine = frame
 
+        self.set_transient_for(frame.MainWindow)
+
         if modal:
             self.set_modal(True)
 
@@ -42,13 +45,13 @@ class MetaDialog(gtk.Dialog):
         self.box = gtk.VBox(spacing=10)
         self.box.set_border_width(10)
         self.box.show()
-        self.vbox.pack_start(self.box)
+        self.vbox.pack_start(self.box, False, False, 0)
 
         if message:
             label = gtk.Label()
             label.set_markup(message)
             label.set_line_wrap(False)
-            self.box.pack_start(label, False, False)
+            self.box.pack_start(label, False, False, 0)
             label.show()
             label.set_alignment(0, 0.5)
 
@@ -60,8 +63,8 @@ class MetaDialog(gtk.Dialog):
 
         self.UF = gtk.Frame()
         self.UF.show()
-        self.UF.set_shadow_type(gtk.SHADOW_ETCHED_IN)
-        self.box.pack_start(self.UF, False, False)
+        self.UF.set_shadow_type(gtk.ShadowType.ETCHED_IN)
+        self.box.pack_start(self.UF, False, False, 0)
 
         vbox3 = gtk.VBox(spacing=5)
         vbox3.set_border_width(5)
@@ -82,7 +85,7 @@ class MetaDialog(gtk.Dialog):
             self.OnBrowseUser, _("Browse")
         )
 
-        hbox2.pack_start(self.BrowseUser, False, False)
+        hbox2.pack_start(self.BrowseUser, False, False, 0)
 
         self.PositionLabel, self.Position = self.MakeLabelStaticEntry(
             hbox2,
@@ -93,11 +96,11 @@ class MetaDialog(gtk.Dialog):
             xalign=1
         )
 
-        vbox3.pack_start(hbox2, False, False)
+        vbox3.pack_start(hbox2, False, False, 0)
 
         hbox3 = gtk.HBox(spacing=5)
         hbox3.show()
-        vbox3.pack_start(hbox3, False, False)
+        vbox3.pack_start(hbox3, False, False, 0)
 
         self.FilenameLabel, self.Filename = self.MakeLabelStaticEntry(
             hbox3,
@@ -108,7 +111,7 @@ class MetaDialog(gtk.Dialog):
 
         hbox5 = gtk.HBox(spacing=5)
         hbox5.show()
-        vbox3.pack_start(hbox5, False, False)
+        vbox3.pack_start(hbox5, False, False, 0)
 
         self.DirectoryLabel, self.Directory = self.MakeLabelStaticEntry(
             hbox5,
@@ -119,7 +122,7 @@ class MetaDialog(gtk.Dialog):
 
         self.Media = gtk.Frame()
         self.Media.show()
-        self.Media.set_shadow_type(gtk.SHADOW_ETCHED_IN)
+        self.Media.set_shadow_type(gtk.ShadowType.ETCHED_IN)
         hbox6 = gtk.HBox(spacing=5, homogeneous=False)
         hbox6.set_border_width(5)
         hbox6.show()
@@ -152,11 +155,11 @@ class MetaDialog(gtk.Dialog):
         )
 
         self.Media.add(hbox6)
-        self.box.pack_start(self.Media, False, False)
+        self.box.pack_start(self.Media, False, False, 0)
 
         hbox7 = gtk.HBox(spacing=5, homogeneous=False)
         hbox7.show()
-        self.box.pack_start(hbox7, False, False)
+        self.box.pack_start(hbox7, False, False, 0)
 
         self.ImmediateLabel, self.Immediate = self.MakeLabelStaticEntry(
             hbox7,
@@ -178,7 +181,7 @@ class MetaDialog(gtk.Dialog):
 
         hbox4 = gtk.HBox(spacing=5, homogeneous=False)
         hbox4.show()
-        self.box.pack_start(hbox4, False, False)
+        self.box.pack_start(hbox4, False, False, 0)
 
         self.SpeedLabel, self.Speed = self.MakeLabelStaticEntry(
             hbox4,
@@ -192,13 +195,13 @@ class MetaDialog(gtk.Dialog):
         self.Country = gtk.Label()
         self.Country.hide()
 
-        hbox4.pack_start(self.Country, False, False)
+        hbox4.pack_start(self.Country, False, False, 0)
 
         self.buttonbox = gtk.HBox(False, 2)
         self.buttonbox.show()
         self.buttonbox.set_spacing(2)
 
-        self.box.pack_start(self.buttonbox, False, False)
+        self.box.pack_start(self.buttonbox, False, False, 0)
 
         # Download Button
         self.DownloadItem = self.nicotine.CreateIconButton(
@@ -208,7 +211,7 @@ class MetaDialog(gtk.Dialog):
             _("Download")
         )
 
-        self.buttonbox.pack_start(self.DownloadItem, False, False)
+        self.buttonbox.pack_start(self.DownloadItem, False, False, 0)
 
         # Download All Button
         self.DownloadAll = self.nicotine.CreateIconButton(
@@ -218,11 +221,11 @@ class MetaDialog(gtk.Dialog):
             _("Download All")
         )
 
-        self.buttonbox.pack_start(self.DownloadAll, False, False)
+        self.buttonbox.pack_start(self.DownloadAll, False, False, 0)
 
         self.Selected = self.MakeLabel(
             self.buttonbox,
-            _("<b>%s</b> File(s) Selected") % len(self.data.keys()),
+            _("<b>%s</b> File(s) Selected") % len(self.data),
             expand=False,
             xalign=1
         )
@@ -241,8 +244,8 @@ class MetaDialog(gtk.Dialog):
             _("Next")
         )
 
-        self.buttonbox.pack_end(self.Next, False, False)
-        self.buttonbox.pack_end(self.Previous, False, False)
+        self.buttonbox.pack_end(self.Next, False, False, 0)
+        self.buttonbox.pack_end(self.Previous, False, False, 0)
 
         button = self.nicotine.CreateIconButton(
             gtk.STOCK_CLOSE,
@@ -251,8 +254,8 @@ class MetaDialog(gtk.Dialog):
             _("Close")
         )
 
-        button.set_flags(gtk.CAN_DEFAULT)
-        self.action_area.pack_start(button)
+        button.props.can_default = True
+        self.action_area.pack_start(button, False, False, 0)
 
         button.grab_default()
 
@@ -262,24 +265,24 @@ class MetaDialog(gtk.Dialog):
 
     def OnDownloadItem(self, widget):
         meta = self.data[self.current]
-        self.nicotine.np.transfers.getFile(meta["user"], meta["fn"], "")
+        self.nicotine.np.transfers.getFile(meta["user"], meta["fn"], "", checkduplicate=True)
 
     def OnBrowseUser(self, widget):
         meta = self.data[self.current]
         self.nicotine.BrowseUser(meta["user"])
 
     def OnDownloadAll(self, widget):
-        for item, meta in self.data.items():
-            self.nicotine.np.transfers.getFile(meta["user"], meta["fn"], "")
+        for item, meta in list(self.data.items()):
+            self.nicotine.np.transfers.getFile(meta["user"], meta["fn"], "", checkduplicate=True)
 
     def OnPrevious(self, widget):
 
-        if len(self.data.keys()) > 1:
+        if len(self.data) > 1:
 
-            _list = self.data.keys()
+            _list = list(self.data.keys())
 
             if self.current not in _list:
-                ix -= 1
+                ix -= 1  # noqa: F821
             else:
                 ix = _list.index(self.current)
                 ix -= 1
@@ -299,12 +302,12 @@ class MetaDialog(gtk.Dialog):
 
     def OnNext(self, widget):
 
-        if len(self.data.keys()) > 1:
+        if len(self.data) > 1:
 
-            _list = self.data.keys()
+            _list = list(self.data.keys())
 
             if self.current not in _list:
-                ix += 1
+                ix += 1  # noqa: F821
             else:
                 ix = _list.index(self.current)
                 ix += 1
@@ -356,7 +359,7 @@ class MetaDialog(gtk.Dialog):
         data = self.data[self.current]
         More = False
 
-        if len(self.data.keys()) > 1:
+        if len(self.data) > 1:
             More = True
 
         self.Next.set_sensitive(More)
@@ -400,12 +403,12 @@ class MetaDialog(gtk.Dialog):
         label = gtk.Label()
         label.set_markup(labeltitle)
         label.show()
-        parent.pack_start(label, expand, fill)
+        parent.pack_start(label, expand, fill, 0)
 
         try:
             label.set_property("xalign", xalign)
-        except Exception, e:
-            print e
+        except Exception as e:
+            print(e)
             pass
 
         return label
@@ -418,7 +421,7 @@ class MetaDialog(gtk.Dialog):
         label.set_markup(labeltitle)
         label.show()
 
-        parent.pack_start(label, False, False)
+        parent.pack_start(label, False, False, 0)
 
         entry = gtk.Entry()
         entry.set_property("editable", editable)
@@ -426,13 +429,13 @@ class MetaDialog(gtk.Dialog):
 
         try:
             entry.set_property("xalign", xalign)
-        except:
+        except Exception:
             pass
 
         entry.show()
         if entrydata is not None:
             entry.set_text(entrydata)
-        parent.pack_start(entry, expand, fill)
+        parent.pack_start(entry, expand, fill, 0)
         return label, entry
 
 
@@ -447,29 +450,31 @@ class EntryDialog(gtk.Dialog):
         self.connect("destroy", self.quit)
         self.connect("delete-event", self.quit)
 
+        self.set_transient_for(frame.MainWindow)
+
         self.gotoption = option
 
         box = gtk.VBox(spacing=10)
         box.set_border_width(10)
-        self.vbox.pack_start(box)
+        self.vbox.pack_start(box, False, False, 0)
         box.show()
 
         if message:
-            label = gtk.Label(message)
-            box.pack_start(label, False, False)
+            label = gtk.Label.new(message)
+            box.pack_start(label, False, False, 0)
             label.set_line_wrap(True)
             label.show()
 
         self.combo_List = gtk.ListStore(gobject.TYPE_STRING)
-        self.combo = gtk.combo_box_new_with_model_and_entry(model=self.combo_List)
+        self.combo = gtk.ComboBox.new_with_model_and_entry(model=self.combo_List)
         self.combo.set_entry_text_column(0)
 
         for i in droplist:
             self.combo_List.append([i])
 
-        self.combo.child.set_text(default_text)
+        self.combo.get_child().set_text(default_text)
 
-        box.pack_start(self.combo, False, False)
+        box.pack_start(self.combo, False, False, 0)
 
         self.combo.show()
         self.combo.grab_focus()
@@ -481,19 +486,19 @@ class EntryDialog(gtk.Dialog):
             self.option.set_label(optionmessage)
             self.option.show()
 
-            box.pack_start(self.option, False, False)
+            box.pack_start(self.option, False, False, 0)
 
         button = gtk.Button(_("Cancel"))
         button.connect("clicked", self.quit)
-        button.set_flags(gtk.CAN_DEFAULT)
-        self.action_area.pack_start(button)
+        button.props.can_default = True
+        self.action_area.pack_start(button, False, False, 0)
 
         button.show()
 
         button = gtk.Button(_("OK"))
         button.connect("clicked", self.click)
-        button.set_flags(gtk.CAN_DEFAULT)
-        self.action_area.pack_start(button)
+        button.props.can_default = True
+        self.action_area.pack_start(button, False, False, 0)
 
         button.show()
         button.grab_default()
@@ -511,9 +516,9 @@ class EntryDialog(gtk.Dialog):
     def click(self, button):
 
         if self.gotoption:
-            self.ret = [self.combo.child.get_text(), self.option.get_active()]
+            self.ret = [self.combo.get_child().get_text(), self.option.get_active()]
         else:
-            self.ret = self.combo.child.get_text()
+            self.ret = self.combo.get_child().get_text()
 
         self.quit()
 
@@ -530,7 +535,10 @@ def input_box(frame, title="Input Box", message="", default_text="",
 
     win.set_title(title)
     win.set_icon(frame.images["n"])
-    gtk.Window.set_geometry_hints(win, min_width=300)
+
+    hints_geometry = Gdk.Geometry()
+    hints_geometry.min_width = 300
+    win.set_geometry_hints(None, hints_geometry, Gdk.WindowHints(Gdk.WindowHints.MIN_SIZE))
     win.show()
 
     gtk.main()
@@ -551,7 +559,7 @@ class FindDialog(gtk.Dialog):
             gobject.signal_new(
                 "find-click",
                 gtk.Window,
-                gobject.SIGNAL_RUN_LAST,
+                gobject.SignalFlags.RUN_LAST,
                 gobject.TYPE_NONE,
                 (gobject.TYPE_STRING,)
             )
@@ -569,23 +577,25 @@ class FindDialog(gtk.Dialog):
         self.currentPosition = None
         self.lastdirection = "next"
 
+        self.set_transient_for(frame.MainWindow)
+
         if modal:
             self.set_modal(True)
 
         box = gtk.VBox(spacing=10)
         box.set_border_width(10)
-        self.vbox.pack_start(box)
+        self.vbox.pack_start(box, False, False, 0)
         box.show()
 
         if message:
-            label = gtk.Label(message)
-            box.pack_start(label, False, False)
+            label = gtk.Label.new(message)
+            box.pack_start(label, False, False, 0)
             label.set_line_wrap(True)
             label.show()
 
         self.entry = gtk.Entry()
 
-        box.pack_start(self.entry, False, False)
+        box.pack_start(self.entry, False, False, 0)
         self.entry.show()
         self.entry.grab_focus()
         self.entry.connect("activate", self.next)
@@ -596,8 +606,8 @@ class FindDialog(gtk.Dialog):
             self.quit,
             _("Cancel")
         )
-        Cancelbutton.set_flags(gtk.CAN_DEFAULT)
-        self.action_area.pack_start(Cancelbutton)
+        Cancelbutton.props.can_default = True
+        self.action_area.pack_start(Cancelbutton, False, False, 0)
 
         Previousbutton = self.nicotine.CreateIconButton(
             gtk.STOCK_GO_BACK,
@@ -605,8 +615,8 @@ class FindDialog(gtk.Dialog):
             self.previous,
             _("Previous")
         )
-        Previousbutton.set_flags(gtk.CAN_DEFAULT)
-        self.action_area.pack_start(Previousbutton)
+        Previousbutton.props.can_default = True
+        self.action_area.pack_start(Previousbutton, False, False, 0)
 
         Nextbutton = self.nicotine.CreateIconButton(
             gtk.STOCK_GO_FORWARD,
@@ -614,8 +624,8 @@ class FindDialog(gtk.Dialog):
             self.next,
             _("Next")
         )
-        Nextbutton.set_flags(gtk.CAN_DEFAULT)
-        self.action_area.pack_start(Nextbutton)
+        Nextbutton.props.can_default = True
+        self.action_area.pack_start(Nextbutton, False, False, 0)
         Nextbutton.grab_default()
 
     def next(self, button):
@@ -654,35 +664,37 @@ class FolderDownloadDialog(gtk.Dialog):
         self.nicotine = frame
 
         self.set_modal(modal)
+        self.set_transient_for(frame.MainWindow)
+
         box = gtk.VBox(spacing=10)
         box.set_border_width(10)
-        self.vbox.pack_start(box)
+        self.vbox.pack_start(box, False, False, 0)
         box.show()
         hbox = gtk.HBox(spacing=5)
         hbox.set_border_width(5)
         hbox.show()
-        box.pack_start(hbox)
+        box.pack_start(hbox, False, False, 0)
 
         image = gtk.Image()
         image.set_padding(0, 0)
         icon = gtk.STOCK_DIALOG_QUESTION
         image.set_from_stock(icon, 4)
         image.show()
-        hbox.pack_start(image)
+        hbox.pack_start(image, False, False, 0)
 
         if message:
-            label = gtk.Label(message)
-            hbox.pack_start(label)
+            label = gtk.Label.new(message)
+            hbox.pack_start(label, False, False, 0)
             label.set_line_wrap(True)
             label.show()
 
         hbox2 = gtk.HBox(spacing=5)
         hbox2.set_border_width(5)
         hbox2.show()
-        box.pack_start(hbox2)
+        box.pack_start(hbox2, False, False, 0)
 
-        cancel_button = self.add_button(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL)
-        ok_button = self.add_button(gtk.STOCK_OK, gtk.RESPONSE_OK)
+        cancel_button = self.add_button(gtk.STOCK_CANCEL, gtk.ResponseType.CANCEL)  # noqa: F841
+        ok_button = self.add_button(gtk.STOCK_OK, gtk.ResponseType.OK)
         ok_button.grab_default()
 
     def quit(self, *args):
@@ -714,14 +726,16 @@ class OptionDialog(gtk.Dialog):
         self.nicotine = frame
 
         self.set_modal(modal)
+        self.set_transient_for(frame.MainWindow)
+
         box = gtk.VBox(spacing=10)
         box.set_border_width(10)
-        self.vbox.pack_start(box)
+        self.vbox.pack_start(box, False, False, 0)
         box.show()
         hbox = gtk.HBox(spacing=5)
         hbox.set_border_width(5)
         hbox.show()
-        box.pack_start(hbox)
+        box.pack_start(hbox, False, False, 0)
 
         if status:
             image = gtk.Image()
@@ -732,27 +746,27 @@ class OptionDialog(gtk.Dialog):
                 icon = gtk.STOCK_DIALOG_QUESTION
             image.set_from_stock(icon, 4)
             image.show()
-            hbox.pack_start(image)
+            hbox.pack_start(image, False, False, 0)
 
         if message:
-            label = gtk.Label(message)
-            hbox.pack_start(label)
+            label = gtk.Label.new(message)
+            hbox.pack_start(label, False, False, 0)
             label.set_line_wrap(True)
             label.show()
 
         hbox2 = gtk.HBox(spacing=5)
         hbox2.set_border_width(5)
         hbox2.show()
-        box.pack_start(hbox2)
+        box.pack_start(hbox2, False, False, 0)
 
         # Storing under self. so we can find it easily later
         if rememberbox:
             self.checkbox = gtk.CheckButton("Remember choice")
-            box.pack_start(self.checkbox)
+            box.pack_start(self.checkbox, False, False, 0)
             self.checkbox.show()
 
         if option:
-            Alignment = gtk.Alignment(0.5, 0.5, 0, 0)
+            Alignment = gtk.Alignment(xalign=0.5, yalign=0.5, xscale=0, yscale=0)
             Alignment.show()
 
             Hbox = gtk.HBox(False, 2)
@@ -768,17 +782,17 @@ class OptionDialog(gtk.Dialog):
             Alignment.add(Hbox)
 
             if label:
-                Label = gtk.Label(third)
+                Label = gtk.Label.new(third)
                 Label.set_padding(0, 0)
                 Label.show()
                 Hbox.pack_start(Label, False, False, 0)
 
-            tray_button = self.add_button("", gtk.RESPONSE_REJECT)
+            tray_button = self.add_button("", gtk.ResponseType.REJECT)
             tray_button.remove(tray_button.get_child())
             tray_button.add(Alignment)
 
-        cancel_button = self.add_button(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL)
-        ok_button = self.add_button(gtk.STOCK_OK, gtk.RESPONSE_OK)
+        cancel_button = self.add_button(gtk.STOCK_CANCEL, gtk.ResponseType.CANCEL)  # noqa: F841
+        ok_button = self.add_button(gtk.STOCK_OK, gtk.ResponseType.OK)
         ok_button.grab_default()
 
     def quit(self, *args):
