@@ -1,5 +1,4 @@
-# -*- coding: utf-8 -*-
-#
+# COPYRIGHT (C) 2020 Nicotine+ Team
 # COPYRIGHT (C) 2016-2017 Michael Labouebe <gfarmerfr@free.fr>
 # COPYRIGHT (C) 2009-2011 Quinox <quinox@users.sf.net>
 #
@@ -19,16 +18,25 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-import gtk
-import gobject
 import os
-import sys
-import thread
-from os.path import exists, getsize, join
+from gettext import gettext as _
+from os.path import exists
+from os.path import getsize
+from os.path import join
 
-from dirchooser import ChooseDir
-from utils import OpenUri, InitialiseColumns, recode, HumanSize, popupWarning
-from entrydialog import input_box
+from gi.repository import Gdk
+from gi.repository import GLib
+from gi.repository import GObject as gobject
+from gi.repository import Gtk as gtk
+
+import _thread
+from pynicotine.gtkgui.dirchooser import ChooseDir
+from pynicotine.gtkgui.entrydialog import input_box
+from pynicotine.gtkgui.utils import HumanSize
+from pynicotine.gtkgui.utils import InitialiseColumns
+from pynicotine.gtkgui.utils import OpenUri
+from pynicotine.gtkgui.utils import popupWarning
+from pynicotine.gtkgui.utils import recode
 
 
 def dirstats(directory):
@@ -76,6 +84,7 @@ class FastConfigureAssistant(object):
         builder.add_from_file(os.path.join(os.path.dirname(os.path.realpath(__file__)), "ui", "fastconfigure.ui"))
 
         self.window = builder.get_object("FastConfigureAssistant")
+        self.window.set_transient_for(frame.MainWindow)
         builder.connect_signals(self)
 
         self.kids = {}
@@ -86,23 +95,6 @@ class FastConfigureAssistant(object):
             except TypeError:
                 pass
 
-        numpages = self.window.get_n_pages()
-
-        for n in xrange(numpages):
-            page = self.window.get_nth_page(n)
-            template = self.window.get_page_title(page)
-            self.window.set_page_title(
-                page,
-                template % {
-                    'page': (n + 1),
-                    'pages': numpages
-                }
-            )
-
-        self.templates = {
-            'listenport': self.kids['listenport'].get_text(),
-        }
-
         # Page specific, sharepage
         # The last column is the raw byte/unicode object
         # for the folder (not shown)
@@ -116,7 +108,7 @@ class FastConfigureAssistant(object):
             gobject.TYPE_STRING
         )
 
-        columns = InitialiseColumns(
+        columns = InitialiseColumns(  # noqa: F841
             self.kids['shareddirectoriestree'],
             [_("Virtual Directory"), 0, "text"],
             [_("Directory"), 0, "text"],
@@ -128,7 +120,7 @@ class FastConfigureAssistant(object):
 
         self.kids['shareddirectoriestree'].set_model(self.sharelist)
         self.kids['shareddirectoriestree'].get_selection().set_mode(
-            gtk.SELECTION_MULTIPLE
+            gtk.SelectionMode.MULTIPLE
         )
 
         self.initphase = False
@@ -149,11 +141,6 @@ class FastConfigureAssistant(object):
             self.config.sections["server"]["passw"]
         )
 
-        # portpage
-        self.kids['advancedports'].set_expanded(
-            self.config.sections["server"]["upnp"]
-        )
-
         if self.config.sections["server"]["firewalled"]:
             self.kids['portclosed'].set_active(True)
         else:
@@ -165,9 +152,6 @@ class FastConfigureAssistant(object):
         self.kids['upperport'].set_value(
             self.config.sections["server"]["portrange"][1]
         )
-        self.kids['useupnp'].set_active(
-            self.config.sections["server"]["upnp"]
-        )
 
         # sharepage
         if self.config.sections['transfers']['downloaddir']:
@@ -213,7 +197,6 @@ class FastConfigureAssistant(object):
             self.kids['lowerport'].get_value_as_int(),
             self.kids['upperport'].get_value_as_int()
         )
-        self.config.sections['server']['upnp'] = self.kids['useupnp'].get_active()
         self.config.sections['server']['firewalled'] = not self.kids['portopen'].get_active()
 
         # sharepage
@@ -234,28 +217,17 @@ class FastConfigureAssistant(object):
     def OnApply(self, widget):
         self.store()
         self.window.hide()
+
+        self.frame.logMessage(
+            _("Setup complete! Connecting...")
+        )
+
         if not self.frame.np.serverconn:
-            self.frame.OnFirstConnect(-1)
+            self.frame.OnConnect(-1)
 
     def OnCancel(self, widget):
         self.window.hide()
 
-    def updatepage(self, page):
-        """Updates information on the given page with.
-        Use _populate if possible."""
-
-        if not page:
-            return
-
-        name = gtk.Buildable.get_name(page)
-
-        if name == 'portpage':
-            self.kids['listenport'].set_markup(
-                _(self.templates['listenport']) % {
-                    'listenport': '<b>' + str(self.frame.np.waitport) + '</b>'
-                }
-            )
-
     def resetcompleteness(self, page=None):
         """Turns on the complete flag if everything required is filled in."""
 
@@ -278,12 +250,9 @@ class FastConfigureAssistant(object):
                 complete = True
 
         elif name == 'portpage':
-            if self.kids['useupnp'].get_active():
+            if self.kids['portopen'].get_active() or \
+               self.kids['portclosed'].get_active():
                 complete = True
-            else:
-                if self.kids['portopen'].get_active() or \
-                   self.kids['portclosed'].get_active():
-                    complete = True
 
         elif name == 'sharepage':
             if exists(self.kids['downloaddir'].get_filename()):
@@ -293,16 +262,13 @@ class FastConfigureAssistant(object):
 
             complete = True
             showcpwarning = (
-                self.kids['portclosed'].get_active() and
-                not self.kids['useupnp'].get_active()
+                self.kids['portclosed'].get_active()
             )
 
             if showcpwarning:
                 self.kids['labelclosedport'].show()
-                self.kids['warningclosedport'].show()
             else:
                 self.kids['labelclosedport'].hide()
-                self.kids['warningclosedport'].hide()
 
             shownfwarning = (
                 self.kids['onlysharewithfriends'].get_active() or
@@ -311,20 +277,17 @@ class FastConfigureAssistant(object):
 
             if shownfwarning:
                 self.kids['labelnoshare'].show()
-                self.kids['warningnoshare'].show()
             else:
                 self.kids['labelnoshare'].hide()
-                self.kids['warningnoshare'].hide()
 
         self.window.set_page_complete(page, complete)
 
     def OnPrepare(self, widget, page):
         self.window.set_page_complete(page, False)
-        self.updatepage(page)
         self.resetcompleteness(page)
 
     def OnEntryChanged(self, widget, param1=None, param2=None, param3=None):
-        name = gtk.Buildable.get_name(widget)
+        name = gtk.Buildable.get_name(widget)  # noqa: F841
         self.resetcompleteness()
 
     def OnEntryPaste(self, user_data):
@@ -381,14 +344,14 @@ class FastConfigureAssistant(object):
             directory[1]
         ])
 
-        thread.start_new_thread(self._addsharedir, (directory,))
+        _thread.start_new_thread(self._addsharedir, (directory,))
 
     def _addsharedir(self, directory):
 
         subdirs, files, size, extensions = dirstats(directory[1])
         exts = []
 
-        for ext, count in extensions.iteritems():
+        for ext, count in extensions.items():
             exts.append((count, ext))
 
         exts.sort(reverse=True)
@@ -399,7 +362,7 @@ class FastConfigureAssistant(object):
         if len(exts) > 5:
             extstring += ", ..."
 
-        gobject.idle_add(
+        GLib.idle_add(
             self._updatedirstats,
             directory,
             HumanSize(size),
@@ -416,12 +379,12 @@ class FastConfigureAssistant(object):
 
             if directory[1] == self.sharelist.get_value(iter, 6):
 
-                self.sharelist.insert_after(iter, [
+                self.sharelist.insert(0, [
                     directory[0],
                     recode(directory[1]),
                     HumanSize(size),
-                    files,
-                    subdirs,
+                    str(files),
+                    str(subdirs),
                     extensions,
                     directory[1]
                 ])
@@ -439,10 +402,12 @@ class FastConfigureAssistant(object):
         name = gtk.Buildable.get_name(widget)
 
         if name == "checkmyport":
-            OpenUri('='.join([
-                'http://tools.slsknet.org/porttest.php?port',
-                str(self.frame.np.waitport)
-                ])
+            OpenUri(
+                '='.join([
+                    'http://tools.slsknet.org/porttest.php?port',
+                    str(self.frame.np.waitport)
+                ]),
+                self.window
             )
 
         if name == "addshare":
@@ -521,27 +486,6 @@ class FastConfigureAssistant(object):
 
     def OnToggled(self, widget):
 
-        name = gtk.Buildable.get_name(widget)
-
-        if name == 'useupnp':
-
-            # Setting active state
-            if widget.get_active():
-                self.kids['portopen'].set_inconsistent(True)
-                self.kids['portclosed'].set_inconsistent(True)
-            else:
-                self.kids['portopen'].set_inconsistent(False)
-                self.kids['portclosed'].set_inconsistent(False)
-
-            # Setting sensitive state
-            inverse = not widget.get_active()
-
-            self.kids['portopen'].set_sensitive(inverse)
-            self.kids['portclosed'].set_sensitive(inverse)
-            self.kids['checkmyport'].set_sensitive(inverse)
-
-            self.resetcompleteness()
-
         if self.initphase:
             return
 
@@ -552,7 +496,7 @@ class FastConfigureAssistant(object):
         if self.initphase:
             return
 
-        name = gtk.Buildable.get_name(widget)
+        name = gtk.Buildable.get_name(widget)  # noqa: F841
 
         self.resetcompleteness()
 
@@ -575,5 +519,5 @@ class FastConfigureAssistant(object):
     def OnKeyPress(self, widget, event):
 
         # Close the window when escape is pressed
-        if event.keyval == gtk.keysyms.Escape:
+        if event.keyval == Gdk.KEY_Escape:
             self.OnCancel(widget)
