@@ -1,5 +1,4 @@
-# -*- coding: utf-8 -*-
-#
+# COPYRIGHT (C) 2020 Nicotine+ Team
 # COPYRIGHT (C) 2016-2017 Michael Labouebe <gfarmerfr@free.fr>
 # COPYRIGHT (C) 2008-2011 Quinox <quinox@users.sf.net>
 # COPYRIGHT (C) 2007 Gallows <g4ll0ws@gmail.com>
@@ -21,21 +20,32 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
-
 import os
-import sys
-import gtk
-import gobject
-import pango
+from collections import deque
+from gettext import gettext as _
+from time import altzone
+from time import daylight
 
-from time import daylight, altzone
+from gi.repository import Gdk
+from gi.repository import GLib
+from gi.repository import GObject as gobject
+from gi.repository import Gtk as gtk
+from gi.repository import Pango as pango
 
-from utils import AppendLine, IconNotebook, PopupMenu, WriteLog, expand_alias, is_alias, EncodingsMenu, SaveEncoding, fixpath
-from chatrooms import GetCompletion
 from pynicotine import slskmessages
-from pynicotine.slskmessages import ToBeEncoded
-from pynicotine.utils import version
+from pynicotine.gtkgui import nowplaying
+from pynicotine.gtkgui.chatrooms import GetCompletion
+from pynicotine.gtkgui.utils import AppendLine
+from pynicotine.gtkgui.utils import IconNotebook
+from pynicotine.gtkgui.utils import PopupMenu
+from pynicotine.gtkgui.utils import ScrollBottom
+from pynicotine.gtkgui.utils import WriteLog
+from pynicotine.gtkgui.utils import expand_alias
+from pynicotine.gtkgui.utils import fixpath
+from pynicotine.gtkgui.utils import is_alias
 from pynicotine.logfacility import log
+from pynicotine.utils import version
+
 
 CTCP_VERSION = "\x01VERSION\x01"
 
@@ -82,11 +92,11 @@ class PrivateChats(IconNotebook):
 
         page = notebook.get_nth_page(page_num)
 
-        for user, tab in self.users.items():
+        for user, tab in list(self.users.items()):
             if tab.Main == page:
-                gobject.idle_add(tab.ChatLine.grab_focus)
+                GLib.idle_add(tab.ChatLine.grab_focus)
                 # Remove hilite if selected tab belongs to a user in the hilite list
-                if user in self.frame.TrayApp.tray_status["hilites"]["private"]:
+                if user in self.frame.hilites["private"]:
                     self.frame.Notifications.Clear("private", tab.user)
 
     def ClearNotifications(self):
@@ -96,10 +106,10 @@ class PrivateChats(IconNotebook):
 
         page = self.get_nth_page(self.get_current_page())
 
-        for user, tab in self.users.items():
+        for user, tab in list(self.users.items()):
             if tab.Main == page:
                 # Remove hilite
-                if user in self.frame.TrayApp.tray_status["hilites"]["private"]:
+                if user in self.frame.hilites["private"]:
                     self.frame.Notifications.Clear("private", tab.user)
 
     def Focused(self, page, focused):
@@ -107,9 +117,9 @@ class PrivateChats(IconNotebook):
         if not focused:
             return
 
-        for user, tab in self.users.items():
+        for user, tab in list(self.users.items()):
             if tab.Main == page:
-                if user in self.frame.TrayApp.tray_status["hilites"]["private"]:
+                if user in self.frame.hilites["private"]:
                     self.frame.Notifications.Clear("private", tab.user)
 
     def GetUserStatus(self, msg):
@@ -182,11 +192,11 @@ class PrivateChats(IconNotebook):
 
     def on_tab_click(self, widget, event, child):
 
-        if event.type == gtk.gdk.BUTTON_PRESS:
+        if event.type == Gdk.EventType.BUTTON_PRESS:
 
             n = self.page_num(child)
             page = self.get_nth_page(n)
-            username = [user for user, tab in self.users.items() if tab.Main is page][0]
+            username = [user for user, tab in list(self.users.items()) if tab.Main is page][0]
 
             if event.button == 2:
                 self.users[username].OnClose(widget)
@@ -194,7 +204,7 @@ class PrivateChats(IconNotebook):
 
             if event.button == 3:
                 menu = self.TabPopup(username)
-                menu.popup(None, None, None, event.button, event.time)
+                menu.popup(None, None, None, None, event.button, event.time)
                 return True
 
             return False
@@ -238,7 +248,7 @@ class PrivateChats(IconNotebook):
             # the tab is not selected, or the main window isn't mapped
             if self.get_current_page() != self.page_num(chat.Main) or \
                self.frame.MainNotebook.get_current_page() != self.frame.MainNotebook.page_num(self.frame.privatevbox) or \
-               not self.frame.is_mapped:
+               not self.frame.MainWindow.get_property("visible"):
                 self.frame.Notifications.Add("private", msg.user)
 
         # SEND CLIENT VERSION to user if the following string is sent
@@ -250,17 +260,17 @@ class PrivateChats(IconNotebook):
         self.users[msg.user].ShowMessage(text, status, msg.timestamp)
 
         if ctcpversion and self.frame.np.config.sections["server"]["ctcpmsgs"] == 0:
-            self.SendMessage(msg.user, u"Nicotine+ %s" % version)
+            self.SendMessage(msg.user, "Nicotine+ %s" % version)
 
         self.frame.pluginhandler.IncomingPrivateChatNotification(msg.user, text)
 
     def UpdateColours(self):
-        for chat in self.users.values():
+        for chat in list(self.users.values()):
             chat.ChangeColours()
 
     def RemoveTab(self, tab):
 
-        if tab.user in self.frame.TrayApp.tray_status["hilites"]["private"]:
+        if tab.user in self.frame.hilites["private"]:
             self.frame.Notifications.Clear("private", tab.user)
 
         del self.users[tab.user]
@@ -302,20 +312,20 @@ class PrivateChats(IconNotebook):
     def UpdateCompletions(self):
 
         config = self.frame.np.config.sections["words"]
-        clist = [self.frame.np.config.sections["server"]["login"], "nicotine"] + self.users.keys()
+        clist = [self.frame.np.config.sections["server"]["login"], "nicotine"] + list(self.users.keys())
 
         if config["buddies"]:
             clist += [i[0] for i in self.frame.userlist.userlist]
 
         if config["aliases"]:
-            clist += ["/"+k for k in self.frame.np.config.aliases.keys()]
+            clist += ["/" + k for k in list(self.frame.np.config.aliases.keys())]
 
         if config["commands"]:
             clist += self.CMDS
 
         self.clist = clist
 
-        for user in self.users.values():
+        for user in list(self.users.values()):
             user.GetCompletionList(clist=list(self.clist))
 
 
@@ -353,35 +363,12 @@ class PrivateChat:
         self.status = -1
         self.clist = []
 
-        # Encoding Combobox
-        self.Elist = {}
-        self.encoding, m = EncodingsMenu(self.frame.np, "userencoding", user)
-        self.EncodingStore = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_STRING)
-        self.Encoding.set_model(self.EncodingStore)
-
-        cell = gtk.CellRendererText()
-        self.Encoding.pack_start(cell, True)
-        self.Encoding.add_attribute(cell, 'text', 0)
-
-        cell2 = gtk.CellRendererText()
-        self.Encoding.pack_start(cell2, False)
-        self.Encoding.add_attribute(cell2, 'text', 1)
-
-        for item in m:
-            self.Elist[item[1]] = self.EncodingStore.append([item[1], item[0]])
-            if self.encoding == item[1]:
-                self.Encoding.set_active_iter(self.Elist[self.encoding])
-
-        if self.frame.SEXY and self.frame.np.config.sections["ui"]["spellcheck"]:
-            import sexy
-            self.hbox5.remove(self.ChatLine)
-            self.ChatLine.destroy()
-            self.ChatLine = sexy.SpellEntry()
-            self.ChatLine.show()
-            self.ChatLine.connect("activate", self.OnEnter)
-            self.ChatLine.connect("key_press_event", self.OnKeyPress)
-            self.hbox5.pack_start(self.ChatLine)
-            self.hbox5.reorder_child(self.ChatLine, 0)
+        if self.frame.gspell and self.frame.np.config.sections["ui"]["spellcheck"]:
+            from gi.repository import Gspell
+            spell_buffer = Gspell.EntryBuffer.get_from_gtk_entry_buffer(self.ChatLine.get_buffer())
+            spell_buffer.set_spell_checker(Gspell.Checker.new())
+            spell_view = Gspell.Entry.get_from_gtk_entry(self.ChatLine)
+            spell_view.set_inline_spell_checking(True)
 
         completion = gtk.EntryCompletion()
         self.ChatLine.set_completion(completion)
@@ -394,7 +381,7 @@ class PrivateChat:
 
         self.Log.set_active(self.frame.np.config.sections["logging"]["privatechat"])
 
-        self.popup_menu_user = popup = PopupMenu(self.frame)
+        self.popup_menu_user = popup = PopupMenu(self.frame, False)
         popup.setup(
             ("#" + _("Show IP a_ddress"), popup.OnShowIPaddress),
             ("#" + _("Get user i_nfo"), popup.OnGetUserInfo),
@@ -443,21 +430,21 @@ class PrivateChat:
         log = os.path.join(config["logging"]["privatelogsdir"], fixpath(self.user.replace(os.sep, "-")) + ".log")
 
         try:
-            lines = int(config["logging"]["readprivatelines"])
-        except:
-            lines = 15
+            numlines = int(config["logging"]["readprivatelines"])
+        except Exception:
+            numlines = 15
 
         try:
-            f = open(log, "r")
-            d = f.read()
-            f.close()
-            s = d.split("\n")
-            for l in s[- lines:-1]:
-                AppendLine(self.ChatScroll, l + "\n", self.tag_hilite, timestamp_format="", username=self.user, usertag=self.tag_hilite)
-        except IOError, e:
+            with open(log, 'r') as lines:
+                # Only show as many log lines as specified in config
+                lines = deque(lines, numlines)
+
+                for line in lines:
+                    AppendLine(self.ChatScroll, line, self.tag_hilite, timestamp_format="", username=self.user, usertag=self.tag_hilite)
+        except IOError as e:  # noqa: F841
             pass
 
-        gobject.idle_add(self.frame.ScrollBottom, self.ChatScroll.get_parent())
+        GLib.idle_add(ScrollBottom, self.ChatScroll.get_parent())
 
     def Login(self):
         timestamp_format = self.frame.np.config.sections["logging"]["private_timestamp"]
@@ -473,18 +460,18 @@ class PrivateChat:
 
     def OnPopupMenu(self, widget, event):
 
-        if event.type == gtk.gdk.BUTTON_PRESS and event.button == 3:
+        if event.type == Gdk.EventType.BUTTON_PRESS and event.button == 3:
 
-            self.popup_menu.popup(None, None, None, event.button, event.time)
-            self.ChatScroll.emit_stop_by_name("button_press_event")
+            self.popup_menu.popup(None, None, None, None, event.button, event.time)
+            self.ChatScroll.stop_emission_by_name("button_press_event")
             return True
 
-        elif event.type == gtk.gdk.KEY_PRESS:
+        elif event.type == Gdk.EventType.KEY_PRESS:
 
-            if event.keyval == gtk.gdk.keyval_from_name("Menu"):
+            if event.keyval == Gdk.keyval_from_name("Menu"):
 
-                self.popup_menu.popup(None, None, None, 0, 0)
-                self.ChatScroll.emit_stop_by_name("key_press_event")
+                self.popup_menu.popup(None, None, None, None, 0, 0)
+                self.ChatScroll.stop_emission_by_name("key_press_event")
                 return True
 
         return False
@@ -512,19 +499,21 @@ class PrivateChat:
         bound = self.ChatScroll.get_buffer().get_selection_bounds()
         if bound is not None and len(bound) == 2:
             start, end = bound
-            log = self.ChatScroll.get_buffer().get_text(start, end)
-            self.frame.clip.set_text(log)
+            log = self.ChatScroll.get_buffer().get_text(start, end, True)
+            self.frame.clip.set_text(log, -1)
 
     def OnCopyAllChatLog(self, widget):
         start, end = self.ChatScroll.get_buffer().get_bounds()
-        log = self.ChatScroll.get_buffer().get_text(start, end)
-        self.frame.clip.set_text(log)
+        log = self.ChatScroll.get_buffer().get_text(start, end, True)
+        self.frame.clip.set_text(log, -1)
 
     def OnClearChatLog(self, widget):
         self.ChatScroll.get_buffer().set_text("")
 
     def ShowMessage(self, text, status=None, timestamp=None):
 
+        self.UpdateColours()
+
         if text[:4] == "/me ":
             line = "* %s %s" % (self.user, self.frame.CensorChat(text[4:]))
             speech = line[2:]
@@ -551,7 +540,7 @@ class PrivateChat:
             # The timestamps from the server are off by a lot, so we'll only use them when this is an offline message
             # Also, they are in UTC so we need to correct them
             if daylight:
-                timestamp -= (3600*daylight)
+                timestamp -= (3600 * daylight)
             else:
                 timestamp += altzone
 
@@ -564,7 +553,7 @@ class PrivateChat:
 
         autoreply = self.frame.np.config.sections["server"]["autoreply"]
         if self.frame.away and not self.autoreplied and autoreply:
-            self.SendMessage(u"[Auto-Message] %s" % autoreply)
+            self.SendMessage("[Auto-Message] %s" % autoreply)
             self.autoreplied = 1
 
         self.frame.Notifications.new_tts(
@@ -607,7 +596,7 @@ class PrivateChat:
         if bytestring:
             payload = text
         else:
-            payload = ToBeEncoded(self.frame.AutoReplace(text), self.encoding)
+            payload = self.frame.AutoReplace(text)
 
         if self.PeerPrivateMessages.get_active():
             # not in the soulseek protocol
@@ -625,25 +614,19 @@ class PrivateChat:
         if text[:2] == "//":
             text = text[1:]
 
-        self.frame.np.queue.put(slskmessages.MessageUser(self.user, ToBeEncoded(self.frame.AutoReplace(text), self.encoding)))
+        self.frame.np.queue.put(slskmessages.MessageUser(self.user, self.frame.AutoReplace(text)))
 
     def OnEnter(self, widget):
 
-        bytes = widget.get_text()
-
-        try:
-            text = unicode(bytes, "UTF-8")
-        except UnicodeDecodeError:
-            log.addwarning(_("We have a problem, PyGTK get_text does not seem to return UTF-8. Please file a bug report. Bytes: %s") % (repr(bytes)))
-            text = unicode(bytes, "UTF-8", "replace")
+        text = widget.get_text()
 
         if not text:
             widget.set_text("")
             return
 
         if is_alias(self.frame.np.config.aliases, text):
-            import thread
-            thread.start_new_thread(self.threadAlias, (text,))
+            import _thread
+            _thread.start_new_thread(self.threadAlias, (text,))
             widget.set_text("")
             return
 
@@ -803,7 +786,7 @@ class PrivateChat:
 
     def Attach(self, widget=None):
         self.chats.attach_tab(self.Main)
-        gobject.idle_add(self.frame.ScrollBottom, self.ChatScroll.get_parent())
+        GLib.idle_add(ScrollBottom, self.ChatScroll.get_parent())
 
     def Detach(self, widget=None):
         self.chats.detach_tab(
@@ -813,31 +796,28 @@ class PrivateChat:
                 'status': [_("Offline"), _("Away"), _("Online")][self.status]
             }
         )
-        gobject.idle_add(self.frame.ScrollBottom, self.ChatScroll.get_parent())
+        GLib.idle_add(ScrollBottom, self.ChatScroll.get_parent())
 
     def NowPlayingThread(self):
-        np = self.frame.now.DisplayNowPlaying(None, 0, self.SendMessage)
+        if self.frame.now is None:
+            self.frame.now = nowplaying.NowPlaying(self.frame)
+
+        np = self.frame.now.DisplayNowPlaying(None, 0, self.SendMessage)  # noqa: F841
 
     def makecolour(self, buffer, colour):
 
-        color = self.frame.np.config.sections["ui"][colour]
-        if color == "":
-            color = self.backupcolor
-        else:
-            color = gtk.gdk.color_parse(color)
-
+        colour = self.frame.np.config.sections["ui"][colour]
         font = self.frame.np.config.sections["ui"]["chatfont"]
-        tag = buffer.create_tag()
-        tag.set_property("foreground-gdk", color)
-        tag.set_property("font", font)
+
+        tag = buffer.create_tag(font=font)
+
+        if colour:
+            tag.set_property("foreground", colour)
 
         return tag
 
     def UpdateColours(self):
 
-        map = self.frame.MainWindow.get_style().copy()
-        self.backupcolor = map.text[gtk.STATE_NORMAL]
-
         buffer = self.ChatScroll.get_buffer()
         self.tag_remote = self.makecolour(buffer, "chatremote")
         self.tag_local = self.makecolour(buffer, "chatlocal")
@@ -864,25 +844,25 @@ class PrivateChat:
         usernamestyle = self.frame.np.config.sections["ui"]["usernamestyle"]
 
         if usernamestyle == "bold":
-            self.tag_username.set_property("weight",  pango.WEIGHT_BOLD)
-            self.tag_my_username.set_property("weight",  pango.WEIGHT_BOLD)
+            self.tag_username.set_property("weight", pango.Weight.BOLD)
+            self.tag_my_username.set_property("weight", pango.Weight.BOLD)
         else:
-            self.tag_username.set_property("weight",  pango.WEIGHT_NORMAL)
-            self.tag_my_username.set_property("weight",  pango.WEIGHT_NORMAL)
+            self.tag_username.set_property("weight", pango.Weight.NORMAL)
+            self.tag_my_username.set_property("weight", pango.Weight.NORMAL)
 
         if usernamestyle == "italic":
-            self.tag_username.set_property("style",  pango.STYLE_ITALIC)
-            self.tag_my_username.set_property("style",  pango.STYLE_ITALIC)
+            self.tag_username.set_property("style", pango.Style.ITALIC)
+            self.tag_my_username.set_property("style", pango.Style.ITALIC)
         else:
-            self.tag_username.set_property("style",  pango.STYLE_NORMAL)
-            self.tag_my_username.set_property("style",  pango.STYLE_NORMAL)
+            self.tag_username.set_property("style", pango.Style.NORMAL)
+            self.tag_my_username.set_property("style", pango.Style.NORMAL)
 
         if usernamestyle == "underline":
-            self.tag_username.set_property("underline", pango.UNDERLINE_SINGLE)
-            self.tag_my_username.set_property("underline", pango.UNDERLINE_SINGLE)
+            self.tag_username.set_property("underline", pango.Underline.SINGLE)
+            self.tag_my_username.set_property("underline", pango.Underline.SINGLE)
         else:
-            self.tag_username.set_property("underline", pango.UNDERLINE_NONE)
-            self.tag_my_username.set_property("underline", pango.UNDERLINE_NONE)
+            self.tag_username.set_property("underline", pango.Underline.NONE)
+            self.tag_my_username.set_property("underline", pango.Underline.NONE)
 
         self.frame.SetTextBG(self.ChatScroll)
         self.frame.SetTextBG(self.ChatLine)
@@ -891,19 +871,14 @@ class PrivateChat:
 
     def changecolour(self, tag, colour):
 
-        if colour in self.frame.np.config.sections["ui"]:
-            color = self.frame.np.config.sections["ui"][colour]
-        else:
-            color = ""
-
-        font = self.frame.np.config.sections["ui"]["chatfont"]
+        color = self.frame.np.config.sections["ui"][colour]
 
         if color == "":
-            color = self.backupcolor
-        else:
-            color = gtk.gdk.color_parse(color)
+            color = None
 
-        tag.set_property("foreground-gdk", color)
+        tag.set_property("foreground", color)
+
+        font = self.frame.np.config.sections["ui"]["chatfont"]
         tag.set_property("font", font)
 
         if colour in ["useraway", "useronline", "useroffline"]:
@@ -911,25 +886,22 @@ class PrivateChat:
             usernamestyle = self.frame.np.config.sections["ui"]["usernamestyle"]
 
             if usernamestyle == "bold":
-                tag.set_property("weight",  pango.WEIGHT_BOLD)
+                tag.set_property("weight", pango.Weight.BOLD)
             else:
-                tag.set_property("weight",  pango.WEIGHT_NORMAL)
+                tag.set_property("weight", pango.Weight.NORMAL)
 
             if usernamestyle == "italic":
-                tag.set_property("style",  pango.STYLE_ITALIC)
+                tag.set_property("style", pango.Style.ITALIC)
             else:
-                tag.set_property("style",  pango.STYLE_NORMAL)
+                tag.set_property("style", pango.Style.NORMAL)
 
             if usernamestyle == "underline":
-                tag.set_property("underline", pango.UNDERLINE_SINGLE)
+                tag.set_property("underline", pango.Underline.SINGLE)
             else:
-                tag.set_property("underline", pango.UNDERLINE_NONE)
+                tag.set_property("underline", pango.Underline.NONE)
 
     def ChangeColours(self):
 
-        map = self.ChatScroll.get_style().copy()
-        self.backupcolor = map.text[gtk.STATE_NORMAL]
-
         self.changecolour(self.tag_remote, "chatremote")
         self.changecolour(self.tag_local, "chatlocal")
         self.changecolour(self.tag_me, "chatme")
@@ -999,8 +971,8 @@ class PrivateChat:
         def _combilower(x):
             try:
                 return str.lower(x)
-            except:
-                return unicode.lower(x)
+            except Exception:
+                return str.lower(x)
 
         clist = list(set(clist))
         clist.sort(key=_combilower)
@@ -1017,13 +989,13 @@ class PrivateChat:
 
     def OnKeyPress(self, widget, event):
 
-        if event.keyval == gtk.gdk.keyval_from_name("Prior"):
+        if event.keyval == Gdk.keyval_from_name("Prior"):
 
             scrolled = self.ChatScroll.get_parent()
             adj = scrolled.get_vadjustment()
             adj.set_value(adj.value - adj.page_increment)
 
-        elif event.keyval == gtk.gdk.keyval_from_name("Next"):
+        elif event.keyval == Gdk.keyval_from_name("Next"):
 
             scrolled = self.ChatScroll.get_parent()
             adj = scrolled.get_vadjustment()
@@ -1035,7 +1007,7 @@ class PrivateChat:
 
             adj.set_value(new)
 
-        if event.keyval != gtk.gdk.keyval_from_name("Tab"):
+        if event.keyval != Gdk.keyval_from_name("Tab"):
             return False
 
         config = self.frame.np.config.sections["words"]
@@ -1053,14 +1025,6 @@ class PrivateChat:
             widget.insert_text(completion, ix)
             widget.set_position(preix + len(completion))
 
-        widget.emit_stop_by_name("key_press_event")
+        widget.stop_emission_by_name("key_press_event")
 
         return True
-
-    def OnEncodingChanged(self, widget):
-
-        encoding = self.Encoding.get_model().get(self.Encoding.get_active_iter(), 0)[0]
-
-        if encoding != self.encoding:
-            self.encoding = encoding
-            SaveEncoding(self.frame.np, "userencoding", self.user, self.encoding)
