@@ -1,5 +1,5 @@
-# -*- coding: utf-8 -*-
-#
+# COPYRIGHT (C) 2020 Nicotine+ Team
+# COPYRIGHT (C) 2016-2018 Mutnick <mutnick@techie.com>
 # COPYRIGHT (C) 2016-2017 Michael Labouebe <gfarmerfr@free.fr>
 # COPYRIGHT (C) 2009-2011 Quinox <quinox@users.sf.net>
 # COPYRIGHT (C) 2009 Hedonist <ak@sensi.org>
@@ -22,26 +22,32 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-import gtk
-from thread import start_new_thread
-
-from transferlist import TransferList
-from utils import PopupMenu, PressHeader
-import string
 import os
-from pynicotine.utils import executeCommand
+from gettext import gettext as _
+
+from gi.repository import Gdk
+from gi.repository import Gtk as gtk
+
+from _thread import start_new_thread
 from pynicotine import slskmessages
-from entrydialog import OptionDialog
+from pynicotine.gtkgui.entrydialog import OptionDialog
+from pynicotine.gtkgui.transferlist import TransferList
+from pynicotine.gtkgui.utils import CollapseTreeview
+from pynicotine.gtkgui.utils import FillFileGroupingCombobox
+from pynicotine.gtkgui.utils import PopupMenu
+from pynicotine.gtkgui.utils import SetTreeviewSelectedRow
+from pynicotine.utils import executeCommand
 
 
 class Uploads(TransferList):
 
     def __init__(self, frame):
 
-        TransferList.__init__(self, frame, frame.UploadList)
+        TransferList.__init__(self, frame, frame.UploadList, type='upload')
         self.myvbox = self.frame.uploadsvbox
-        self.frame.UploadList.set_property("rules-hint", True)
-        self.popup_menu2 = popup2 = PopupMenu(frame)
+
+        self.popup_menu_users = PopupMenu(self.frame, False)
+        self.popup_menu_clear = popup2 = PopupMenu(self.frame, False)
         popup2.setup(
             ("#" + _("Clear finished/erred"), self.OnClearFinishedErred),
             ("#" + _("Clear finished/aborted"), self.OnClearFinishedAborted),
@@ -51,8 +57,6 @@ class Uploads(TransferList):
             ("#" + _("Clear failed"), self.OnClearFailed)
         )
 
-        self.popup_menu_users = PopupMenu(frame)
-
         self.popup_menu = popup = PopupMenu(frame)
         popup.setup(
             ("#" + _("Copy _URL"), self.OnCopyURL),
@@ -66,23 +70,9 @@ class Uploads(TransferList):
             ("#" + _("_Clear"), self.OnClearTransfer),
             ("#" + _("_Retry"), self.OnUploadTransfer),
             ("", None),
-            (1, _("Clear Groups"), self.popup_menu2, None)
+            (1, _("Clear Groups"), self.popup_menu_clear, None)
         )
 
-        frame.UploadList.connect("button_press_event", self.OnPopupMenu, "mouse")
-        frame.UploadList.connect("key-press-event", self.on_key_press_event)
-
-        cols = frame.UploadList.get_columns()
-
-        for i in range(9):
-
-            parent = cols[i].get_widget().get_ancestor(gtk.Button)
-            if parent:
-                parent.connect("button_press_event", PressHeader)
-
-            # Read Show / Hide column settings from last session
-            cols[i].set_visible(self.frame.np.config.sections["columns"]["uploads"][i])
-
         frame.clearUploadFinishedErredButton.connect("clicked", self.OnClearFinishedErred)
         frame.clearUploadQueueButton.connect("clicked", self.OnTryClearQueued)
         frame.abortUploadButton.connect("clicked", self.OnAbortTransfer)
@@ -90,11 +80,12 @@ class Uploads(TransferList):
         frame.banUploadButton.connect("clicked", self.OnBan)
         frame.UploadList.expand_all()
 
-        self.frame.ToggleAutoclear.set_active(self.frame.np.config.sections["transfers"]["autoclear_uploads"])
-        frame.ToggleAutoclear.connect("toggled", self.OnToggleAutoclear)
-        self.frame.ToggleTreeUploads.set_active(self.frame.np.config.sections["transfers"]["groupuploads"])
-        frame.ToggleTreeUploads.connect("toggled", self.OnToggleTree)
+        self.frame.ToggleAutoclearUploads.set_active(self.frame.np.config.sections["transfers"]["autoclear_uploads"])
+        frame.ToggleAutoclearUploads.connect("toggled", self.OnToggleAutoclear)
 
+        FillFileGroupingCombobox(frame.ToggleTreeUploads)
+        frame.ToggleTreeUploads.set_active(self.frame.np.config.sections["transfers"]["groupuploads"])
+        frame.ToggleTreeUploads.connect("changed", self.OnToggleTree)
         self.OnToggleTree(None)
 
         self.frame.ExpandUploads.set_active(self.frame.np.config.sections["transfers"]["uploadsexpanded"])
@@ -102,21 +93,13 @@ class Uploads(TransferList):
 
         self.OnExpandUploads(None)
 
-    def saveColumns(self):
-
-        columns = []
-        for column in self.frame.UploadList.get_columns():
-            columns.append(column.get_visible())
-
-        self.frame.np.config.sections["columns"]["uploads"] = columns
-
     def OnTryClearQueued(self, widget):
 
         direction = "up"
 
         win = OptionDialog(self.frame, _("Clear All Queued Uploads?"), modal=True, status=None, option=False, third="")
         win.connect("response", self.frame.on_clear_response, direction)
-        win.set_title(_("Nicotine+")+": "+_("Clear Queued Transfers"))
+        win.set_title(_("Nicotine+") + ": " + _("Clear Queued Transfers"))
         win.set_icon(self.frame.images["n"])
 
         win.show()
@@ -137,21 +120,11 @@ class Uploads(TransferList):
         else:
             executeCommand(filemanager, incompletedir)
 
-    def OnFileSearch(self, widget):
-
-        self.select_transfers()
-
-        for transfer in self.selected_transfers:
-            self.frame.SearchEntry.set_text(transfer.filename.rsplit("\\", 1)[1])
-            self.frame.ChangeMainPage(None, "search")
-            break
-
-    def expandcollapse(self, path):
-
-        if self.frame.ExpandUploads.get_active():
-            self.frame.UploadList.expand_row(path, True)
+    def expand(self, path):
+        if self.frame.ExpandDownloads.get_active():
+            self.frame.UploadList.expand_to_path(path)
         else:
-            self.frame.UploadList.collapse_row(path)
+            CollapseTreeview(self.frame.UploadList, self.TreeUsers)
 
     def OnExpandUploads(self, widget):
 
@@ -161,14 +134,14 @@ class Uploads(TransferList):
             self.frame.UploadList.expand_all()
             self.frame.ExpandUploadsImage.set_from_stock(gtk.STOCK_REMOVE, 4)
         else:
-            self.frame.UploadList.collapse_all()
+            CollapseTreeview(self.frame.UploadList, self.TreeUsers)
             self.frame.ExpandUploadsImage.set_from_stock(gtk.STOCK_ADD, 4)
 
         self.frame.np.config.sections["transfers"]["uploadsexpanded"] = expanded
         self.frame.np.config.writeConfiguration()
 
     def OnToggleAutoclear(self, widget):
-        self.frame.np.config.sections["transfers"]["autoclear_uploads"] = self.frame.ToggleAutoclear.get_active()
+        self.frame.np.config.sections["transfers"]["autoclear_uploads"] = self.frame.ToggleAutoclearUploads.get_active()
 
     def OnToggleTree(self, widget):
 
@@ -177,33 +150,11 @@ class Uploads(TransferList):
 
         self.RebuildTransfers()
 
-        if not self.TreeUsers:
+        if self.TreeUsers == 0:
             self.frame.ExpandUploads.hide()
         else:
             self.frame.ExpandUploads.show()
 
-    def RebuildTransfers(self):
-
-        if self.frame.np.transfers is None:
-            return
-
-        self.Clear()
-        self.update()
-
-    def select_transfers(self):
-
-        self.selected_transfers = []
-        self.selected_users = []
-
-        self.widget.get_selection().selected_foreach(self.SelectedTransfersCallback)
-
-    def OnBan(self, widget):
-
-        self.select_transfers()
-
-        for user in self.selected_users:
-            self.frame.BanUser(user)
-
     def OnAbortUser(self, widget):
 
         self.select_transfers()
@@ -235,41 +186,25 @@ class Uploads(TransferList):
 
         self.frame.np.transfers.checkUploadQueue()
 
-    def OnSelectUserTransfer(self, widet):
-
-        if len(self.selected_users) != 1:
-            return
-
-        selected_user = self.selected_users[0]
-
-        sel = self.frame.UploadList.get_selection()
-        fmodel = self.frame.UploadList.get_model()
-        sel.unselect_all()
-
-        for item in self.transfers:
-            user_file, iter, transfer = item
-            user, filepath = user_file
-            if selected_user == user:
-                ix = fmodel.get_path(iter)
-                sel.select_path(ix,)
-
-        self.select_transfers()
-
     def on_key_press_event(self, widget, event):
 
-        key = gtk.gdk.keyval_name(event.keyval)
+        key = Gdk.keyval_name(event.keyval)
 
         if key in ("P", "p"):
             self.OnPopupMenu(widget, event, "keyboard")
         else:
-            self.selected_transfers = []
-            self.selected_users = []
-            self.widget.get_selection().selected_foreach(self.SelectedTransfersCallback)
+            self.select_transfers()
 
             if key in ("T", "t"):
                 self.OnAbortTransfer(widget)
             elif key == "Delete":
                 self.OnAbortTransfer(widget, False, True)
+            else:
+                # No key match, continue event
+                return False
+
+        widget.stop_emission_by_name("key_press_event")
+        return True
 
     def OnPlayFiles(self, widget, prefix=""):
         start_new_thread(self._OnPlayFiles, (widget, prefix))
@@ -286,84 +221,20 @@ class Uploads(TransferList):
             if os.path.exists(file):
                 executeCommand(executable, file, background=False)
 
-    def OnPopupMenuUsers(self, widget):
-
-        self.selected_transfers = []
-        self.selected_users = []
-        self.widget.get_selection().selected_foreach(self.SelectedTransfersCallback)
-
-        self.popup_menu_users.clear()
-
-        if len(self.selected_users) > 0:
-
-            items = []
-            self.selected_users.sort(key=str.lower)
-
-            for user in self.selected_users:
-                popup = PopupMenu(self.frame)
-                popup.setup(
-                    ("#" + _("Send _message"), popup.OnSendMessage),
-                    ("#" + _("Show IP a_ddress"), popup.OnShowIPaddress),
-                    ("#" + _("Get user i_nfo"), popup.OnGetUserInfo),
-                    ("#" + _("Brow_se files"), popup.OnBrowseUser),
-                    ("#" + _("Gi_ve privileges"), popup.OnGivePrivileges),
-                    ("", None),
-                    ("$" + _("_Add user to list"), popup.OnAddToList),
-                    ("$" + _("_Ban this user"), popup.OnBanUser),
-                    ("$" + _("_Ignore this user"), popup.OnIgnoreUser),
-                    ("#" + _("Select User's Transfers"), self.OnSelectUserTransfer)
-                )
-                popup.set_user(user)
-
-                items.append((1, user, popup, self.OnPopupMenuUser, popup))
-
-            self.popup_menu_users.setup(*items)
-
-        return True
-
-    def OnPopupMenuUser(self, widget, popup=None):
-
-        if popup is None:
-            return
-
-        menu = popup
-        user = menu.user
-        items = menu.get_children()
-
-        act = False
-        if len(self.selected_users) >= 1:
-            act = True
-
-        items[0].set_sensitive(act)
-        items[1].set_sensitive(act)
-        items[2].set_sensitive(act)
-        items[3].set_sensitive(act)
-
-        items[6].set_active(user in [i[0] for i in self.frame.np.config.sections["server"]["userlist"]])
-        items[7].set_active(user in self.frame.np.config.sections["server"]["banlist"])
-        items[8].set_active(user in self.frame.np.config.sections["server"]["ignorelist"])
-
-        for i in range(4, 9):
-            items[i].set_sensitive(act)
-
-        return True
-
     def OnPopupMenu(self, widget, event, kind):
 
         if kind == "mouse":
             if event.button != 3:
-                if event.button == 1 and event.type == gtk.gdk._2BUTTON_PRESS:
+                if event.button == 1 and event.type == Gdk.EventType._2BUTTON_PRESS:
                     self.DoubleClick(event)
                 return False
 
-        self.selected_transfers = []
-        self.selected_users = []
-        self.widget.get_selection().selected_foreach(self.SelectedTransfersCallback)
+            SetTreeviewSelectedRow(widget, event)
 
-        self.SelectCurrentRow(event, kind)
+        self.select_transfers()
 
         users = len(self.selected_users) > 0
-        multi_users = len(self.selected_users) > 1
+        multi_users = len(self.selected_users) > 1  # noqa: F841
         files = len(self.selected_transfers) > 0
         multi_files = len(self.selected_transfers) > 1
 
@@ -386,16 +257,17 @@ class Uploads(TransferList):
         else:
             act = False
 
-        for i in range(3, 5) + range(6, 10):
+        for i in list(range(3, 5)) + list(range(6, 10)):
             items[i].set_sensitive(act)
 
         items[2].set_sensitive(act)  # send to player
 
-        self.popup_menu.popup(None, None, None, 3, event.time)
+        self.popup_menu.popup(None, None, None, None, 3, event.time)
+
         if kind == "keyboard":
-            widget.emit_stop_by_name("key_press_event")
+            widget.stop_emission_by_name("key_press_event")
         elif kind == "mouse":
-            widget.emit_stop_by_name("button_press_event")
+            widget.stop_emission_by_name("button_press_event")
 
         return True
 
@@ -405,13 +277,11 @@ class Uploads(TransferList):
             if i.user == user:
                 if i.transfertimer is not None:
                     i.transfertimer.cancel()
-                self.list.remove(i)
+                self.remove_specific(i)
 
         self.frame.np.transfers.calcUploadQueueSizes()
         self.frame.np.transfers.checkUploadQueue()
 
-        self.update()
-
     def DoubleClick(self, event):
 
         self.select_transfers()
@@ -436,8 +306,6 @@ class Uploads(TransferList):
         self.frame.np.transfers.calcUploadQueueSizes()
         self.frame.np.transfers.checkUploadQueue()
 
-        self.update()
-
     def OnClearQueued(self, widget):
 
         self.select_transfers()
@@ -446,12 +314,8 @@ class Uploads(TransferList):
         self.frame.np.transfers.calcUploadQueueSizes()
         self.frame.np.transfers.checkUploadQueue()
 
-        self.update()
-
     def OnClearFailed(self, widget):
 
         TransferList.OnClearFailed(self, widget)
         self.frame.np.transfers.calcUploadQueueSizes()
         self.frame.np.transfers.checkUploadQueue()
-
-        self.update()
