@@ -1,5 +1,4 @@
-# -*- coding: utf-8 -*-
-#
+# COPYRIGHT (C) 2020 Nicotine+ Team
 # COPYRIGHT (C) 2016-2017 Michael Labouebe <gfarmerfr@free.fr>
 # COPYRIGHT (C) 2013 SeeSchloss <see@seos.fr>
 # COPYRIGHT (C) 2009-2010 Quinox <quinox@users.sf.net>
@@ -22,25 +21,33 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-from __future__ import division
-
-import gtk
 import os
-import sys
-import gobject
-from userinfo import UserTabs
+from gettext import gettext as _
 
-from utils import InitialiseColumns, PopupMenu, EncodingsMenu, SaveEncoding, Humanize, HumanizeBytes, PressHeader
-from dirchooser import ChooseDir
-from entrydialog import input_box
+from gi.repository import Gdk
+from gi.repository import GObject as gobject
+from gi.repository import Gtk as gtk
+
+from _thread import start_new_thread
 from pynicotine import slskmessages
-from thread import start_new_thread
-from pynicotine.utils import displayTraceback, executeCommand, CleanFile
+from pynicotine.gtkgui.dirchooser import ChooseDir
+from pynicotine.gtkgui.entrydialog import input_box
+from pynicotine.gtkgui.utils import HumanSize
+from pynicotine.gtkgui.utils import InitialiseColumns
+from pynicotine.gtkgui.utils import PopupMenu
+from pynicotine.gtkgui.utils import PressHeader
+from pynicotine.gtkgui.utils import SetTreeviewSelectedRow
+from pynicotine.utils import CleanFile
+from pynicotine.utils import displayTraceback
+from pynicotine.utils import executeCommand
+from pynicotine.utils import GetUserDirectories
+from pynicotine.utils import GetResultBitrateLength
 
 
 class UserBrowse:
 
     def __init__(self, userbrowses, user, conn):
+        _config_dir, self.data_dir = GetUserDirectories()
 
         # Build the window
         builder = gtk.Builder()
@@ -82,25 +89,6 @@ class UserBrowse:
         # Iters for current FileStore
         self.files = {}
         self.totalsize = 0
-        self.encoding, m = EncodingsMenu(self.frame.np, "userencoding", user)
-
-        # Encoding Combobox
-        self.Elist = {}
-        self.EncodingStore = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_STRING)
-        self.Encoding.set_model(self.EncodingStore)
-
-        cell = gtk.CellRendererText()
-        self.Encoding.pack_start(cell, True)
-        self.Encoding.add_attribute(cell, 'text', 0)
-
-        cell2 = gtk.CellRendererText()
-        self.Encoding.pack_start(cell2, False)
-        self.Encoding.add_attribute(cell2, 'text', 1)
-
-        for item in m:
-            self.Elist[item[1]] = self.EncodingStore.append([item[1], item[0]])
-            if self.encoding == item[1]:
-                self.Encoding.set_active_iter(self.Elist[self.encoding])
 
         self.DirStore = gtk.TreeStore(str, str)
 
@@ -113,8 +101,8 @@ class UserBrowse:
         )
         cols[0].set_sort_column_id(0)
 
-        self.popup_menu_users = PopupMenu(self.frame)
-        self.popup_menu_users2 = PopupMenu(self.frame)
+        self.popup_menu_users = PopupMenu(self.frame, False)
+        self.popup_menu_users2 = PopupMenu(self.frame, False)
         for menu in [self.popup_menu_users, self.popup_menu_users2]:
             menu.setup(
                 ("#" + _("Send _message"), menu.OnSendMessage),
@@ -122,12 +110,12 @@ class UserBrowse:
                 ("#" + _("Get user i_nfo"), menu.OnGetUserInfo),
                 ("#" + _("Gi_ve privileges"), menu.OnGivePrivileges),
                 ("", None),
-                ("$" + _("_Add user to list"),  menu.OnAddToList),
+                ("$" + _("_Add user to list"), menu.OnAddToList),
                 ("$" + _("_Ban this user"), menu.OnBanUser),
                 ("$" + _("_Ignore this user"), menu.OnIgnoreUser)
             )
 
-        self.popup_menu_downloads_folders = PopupMenu(self.frame)
+        self.popup_menu_downloads_folders = PopupMenu(self.frame, False)
         self.popup_menu_downloads_folders.setup(
             ("#" + _("_Download directory"), self.OnDownloadDirectory),
             ("#" + _("Download directory _to..."), self.OnDownloadDirectoryTo),
@@ -135,7 +123,7 @@ class UserBrowse:
             ("#" + _("Download r_ecursive to..."), self.OnDownloadDirectoryRecursiveTo)
         )
 
-        self.popup_menu_downloads_files = PopupMenu(self.frame)
+        self.popup_menu_downloads_files = PopupMenu(self.frame, False)
         self.popup_menu_downloads_files.setup(
             ("#" + _("_Download file(s)"), self.OnDownloadFiles),
             ("#" + _("Download _to..."), self.OnDownloadFilesTo),
@@ -146,13 +134,13 @@ class UserBrowse:
             ("#" + _("Download r_ecursive to..."), self.OnDownloadDirectoryRecursiveTo)
         )
 
-        self.popup_menu_uploads_folders = PopupMenu(self.frame)
+        self.popup_menu_uploads_folders = PopupMenu(self.frame, False)
         self.popup_menu_uploads_folders.setup(
             ("#" + _("Upload Directory to..."), self.OnUploadDirectoryTo),
             ("#" + _("Upload Directory recursive to..."), self.OnUploadDirectoryRecursiveTo)
         )
 
-        self.popup_menu_uploads_files = PopupMenu(self.frame)
+        self.popup_menu_uploads_files = PopupMenu(self.frame, False)
         self.popup_menu_uploads_files.setup(
             ("#" + _("Upload Directory to..."), self.OnUploadDirectoryTo),
             ("#" + _("Upload Directory recursive to..."), self.OnUploadDirectoryRecursiveTo),
@@ -184,32 +172,40 @@ class UserBrowse:
         self.FolderTreeView.connect("button_press_event", self.OnFolderClicked)
         self.FolderTreeView.get_selection().connect("changed", self.OnSelectDir)
 
-        # DecodedFilename, HSize, Bitrate, HLength, Size, Length, RawFilename
+        # Filename, HSize, Bitrate, HLength, Size, Length, RawFilename
         self.FileStore = gtk.ListStore(str, str, str, str, gobject.TYPE_INT64, int, str)
 
         self.FileTreeView.set_model(self.FileStore)
+        widths = self.frame.np.config.sections["columns"]["userbrowse_widths"]
         cols = InitialiseColumns(
             self.FileTreeView,
-            [_("Filename"), 250, "text", self.CellDataFunc],
-            [_("Size"), 100, "text", self.CellDataFunc],
-            [_("Bitrate"), 70, "text", self.CellDataFunc],
-            [_("Length"), 50, "text", self.CellDataFunc]
+            [_("Filename"), widths[0], "text", self.CellDataFunc],
+            [_("Size"), widths[1], "text", self.CellDataFunc],
+            [_("Bitrate"), widths[2], "text", self.CellDataFunc],
+            [_("Length"), widths[3], "text", self.CellDataFunc]
         )
         cols[0].set_sort_column_id(0)
         cols[1].set_sort_column_id(4)
         cols[2].set_sort_column_id(2)
         cols[3].set_sort_column_id(5)
-        self.FileStore.set_sort_column_id(0, gtk.SORT_ASCENDING)
+        self.FileStore.set_sort_column_id(0, gtk.SortType.ASCENDING)
 
-        for i in range(4):
-            parent = cols[i].get_widget().get_ancestor(gtk.Button)
-            if parent:
-                parent.connect('button_press_event', PressHeader)
+        try:
+            for i in range(len(cols)):
 
-        # Read Show / Hide column settings from last session
-        self.FileTreeView.get_selection().set_mode(gtk.SELECTION_MULTIPLE)
+                parent = cols[i].get_widget().get_ancestor(gtk.Button)
+                if parent:
+                    parent.connect("button_press_event", PressHeader)
+
+                # Read Show / Hide column settings from last session
+                cols[i].set_visible(self.frame.np.config.sections["columns"]["userbrowse"][i])
+        except IndexError:
+            # Column count in config is probably incorrect (outdated?), don't crash
+            pass
+
+        self.FileTreeView.get_selection().set_mode(gtk.SelectionMode.MULTIPLE)
         self.FileTreeView.set_headers_clickable(True)
-        self.FileTreeView.set_property("rules-hint", True)
+        self.FileTreeView.set_rubber_banding(True)
 
         self.file_popup_menu = PopupMenu(self.frame)
 
@@ -237,7 +233,7 @@ class UserBrowse:
 
         self.ChangeColours()
 
-        for name, object in self.__dict__.items():
+        for name, object in list(self.__dict__.items()):
             if type(object) is PopupMenu:
                 object.set_user(self.user)
 
@@ -291,15 +287,12 @@ class UserBrowse:
         self.frame.ChangeListFont(self.FolderTreeView, self.frame.np.config.sections["ui"]["browserfont"])
         self.frame.ChangeListFont(self.FileTreeView, self.frame.np.config.sections["ui"]["browserfont"])
 
-    def CellDataFunc(self, column, cellrenderer, model, iter):
+    def CellDataFunc(self, column, cellrenderer, model, iter, dummy="dummy"):
         colour = self.frame.np.config.sections["ui"]["search"]
         if colour == "":
             colour = None
         cellrenderer.set_property("foreground", colour)
 
-    def decode(self, str):
-        return self.frame.np.decode(str, self.encoding)
-
     def OnExpand(self, widget):
 
         if self.ExpandButton.get_active():
@@ -319,7 +312,7 @@ class UserBrowse:
 
     def OnFolderClicked(self, widget, event):
 
-        if event.button == 1 and event.type == gtk.gdk._2BUTTON_PRESS:
+        if event.button == 1 and event.type == Gdk.EventType._2BUTTON_PRESS:
             self.OnDownloadDirectory(widget)
             return True
         elif event.button == 3:
@@ -337,7 +330,7 @@ class UserBrowse:
         for item in items[1:]:
             item.set_sensitive(act)
 
-        self.folder_popup_menu.popup(None, None, None, event.button, event.time)
+        self.folder_popup_menu.popup(None, None, None, None, event.button, event.time)
 
     def SelectedFilesCallback(self, model, path, iter):
         rawfilename = self.FileStore.get_value(iter, 6)
@@ -345,7 +338,7 @@ class UserBrowse:
 
     def OnFileClicked(self, widget, event):
 
-        if event.button == 1 and event.type == gtk.gdk._2BUTTON_PRESS:
+        if event.button == 1 and event.type == Gdk.EventType._2BUTTON_PRESS:
             self.selected_files = []
             self.FileTreeView.get_selection().selected_foreach(self.SelectedFilesCallback)
             self.OnDownloadFiles(widget)
@@ -358,6 +351,8 @@ class UserBrowse:
 
     def OnFilePopupMenu(self, widget, event):
 
+        SetTreeviewSelectedRow(widget, event)
+
         self.selected_files = []
         self.FileTreeView.get_selection().selected_foreach(self.SelectedFilesCallback)
 
@@ -383,8 +378,8 @@ class UserBrowse:
             items[2].set_sensitive(files)  # Downloads
             items[4].set_sensitive(not multiple and files)  # Copy URL
 
-        self.FileTreeView.emit_stop_by_name("button_press_event")
-        self.file_popup_menu.popup(None, None, None, event.button, event.time)
+        self.FileTreeView.stop_emission_by_name("button_press_event")
+        self.file_popup_menu.popup(None, None, None, None, event.button, event.time)
 
         return True
 
@@ -404,9 +399,9 @@ class UserBrowse:
                 if filedata[2] < 18446744000000000000:
                     self.totalsize += filedata[2]
                 else:
-                    print "Unbelievable filesize: %s, %s" % (HumanizeBytes(filedata[2]), repr(filedata))
+                    print("Unbelievable filesize: %s, %s" % (HumanSize(filedata[2]), repr(filedata)))
 
-        self.AmountShared.set_text(_("Shared: %s") % HumanizeBytes(self.totalsize))
+        self.AmountShared.set_text(_("Shared: %s") % HumanSize(self.totalsize))
         self.NumDirectories.set_text(_("Dirs: %s") % len(self.shares))
 
         # Generate the directory tree and select first directory
@@ -440,7 +435,7 @@ class UserBrowse:
             self.FolderTreeView.set_model(self.DirStore)
 
             # Sort the DirStore
-            self.DirStore.set_sort_column_id(0, gtk.SORT_ASCENDING)
+            self.DirStore.set_sort_column_id(0, gtk.SortType.ASCENDING)
 
             return directory
 
@@ -470,7 +465,7 @@ class UserBrowse:
             """
 
             # Foreach subdir
-            for subdir in dictdir.keys():
+            for subdir in list(dictdir.keys()):
 
                 if parent is None:
                     # The first sudirs are attached to the root (None)
@@ -479,7 +474,7 @@ class UserBrowse:
                     # Other sudirs futher down the path are attached to their parent
                     current_path = dirseparator.join([path, subdir])
 
-                self.directories[current_path] = self.DirStore.append(parent, [self.decode(subdir), current_path])
+                self.directories[current_path] = self.DirStore.append(parent, [subdir, current_path])
 
                 # If there are subdirs futher down the path: recurse
                 if len(dictdir[subdir]):
@@ -507,7 +502,7 @@ class UserBrowse:
         directory = sortlist[0]
 
         # Sort the DirStore
-        self.DirStore.set_sort_column_id(0, gtk.SORT_ASCENDING)
+        self.DirStore.set_sort_column_id(0, gtk.SortType.ASCENDING)
 
         # Set the model of the treeviex
         self.FolderTreeView.set_model(self.DirStore)
@@ -532,69 +527,55 @@ class UserBrowse:
             return
 
         for file in files:
-            # DecodedFilename, HSize, Bitrate, HLength, Size, Length, RawFilename
+            # Filename, HSize, Bitrate, HLength, Size, Length, RawFilename
             rl = 0
             try:
                 size = int(file[2])
             except ValueError:
                 size = 0
 
-            f = [self.decode(file[1]), Humanize(size)]
+            f = [file[1], HumanSize(size)]
 
-            if file[3] == "":
-                f += ["", ""]
-            else:
-                # file[4] is for file types such as 'mp3'
-                attrs = file[4]
-                if attrs != [] and type(attrs) is list:
+            h_bitrate, bitrate, h_length = GetResultBitrateLength(size, file[4])
+            f += [h_bitrate, h_length]
 
-                    if len(attrs) >= 3:
-
-                        br = str(attrs[0])
-                        if attrs[2]:
-                            br = br + " (vbr)"
-
-                        try:
-                            rl = int(attrs[1])
-                        except ValueError:
-                            rl = 0
-
-                        l = "%i:%02i" % (rl / 60, rl % 60)
-                        f += [br, l]
-                    else:
-                        f += ["", ""]
-                else:
-                    f += ["", ""]
-
-            f += [long(size), rl, file[1]]
+            f += [int(size), rl, file[1]]
 
             try:
                 self.files[f[0]] = self.FileStore.append(f)
-            except Exception, error:
+            except Exception:
                 displayTraceback()
 
     def OnSave(self, widget):
-
-        configdir, config = os.path.split(self.frame.np.config.filename)
-        sharesdir = os.path.abspath(configdir + os.sep + "usershares" + os.sep)
+        sharesdir = os.path.join(self.data_dir, "usershares")
 
         try:
             if not os.path.exists(sharesdir):
                 os.mkdir(sharesdir)
-        except Exception, msg:
+        except Exception as msg:
             error = _("Can't create directory '%(folder)s', reported error: %(error)s" % {'folder': sharesdir, 'error': msg})
             self.frame.logMessage(error)
 
         try:
-            import cPickle as mypickle
+            import pickle as mypickle
             import bz2
             sharesfile = bz2.BZ2File(os.path.join(sharesdir, CleanFile(self.user)), 'w')
-            mypickle.dump(self.shares, sharesfile, mypickle.HIGHEST_PROTOCOL)
+            mypickle.dump(self.shares, sharesfile, protocol=mypickle.HIGHEST_PROTOCOL)
             sharesfile.close()
-        except Exception, msg:
+        except Exception as msg:
             error = _("Can't save shares, '%(user)s', reported error: %(error)s" % {'user': self.user, 'error': msg})
             self.frame.logMessage(error)
 
+    def saveColumns(self):
+
+        columns = []
+        widths = []
+        for column in self.FileTreeView.get_columns():
+            columns.append(column.get_visible())
+            widths.append(column.get_width())
+        self.frame.np.config.sections["columns"]["userbrowse"] = columns
+        self.frame.np.config.sections["columns"]["userbrowse_widths"] = widths
+
     def ShowInfo(self, msg):
         self.conn = None
         self.MakeNewModel(msg.list)
@@ -634,10 +615,10 @@ class UserBrowse:
         if model.sort_col == column_id:
             order = model.sort_order
 
-            if order == gtk.SORT_ASCENDING:
-                order = gtk.SORT_DESCENDING
+            if order == gtk.SortType.ASCENDING:
+                order = gtk.SortType.DESCENDING
             else:
-                order = gtk.SORT_ASCENDING
+                order = gtk.SortType.ASCENDING
 
             column.set_sort_order(order)
             model.sort_order = order
@@ -659,11 +640,12 @@ class UserBrowse:
             self.DownloadDirectory(self.selected_folder)
 
     def OnDownloadDirectoryRecursive(self, widget):
+
         self.DownloadDirectory(self.selected_folder, "", 1)
 
     def OnDownloadDirectoryTo(self, widget):
 
-        dir = ChooseDir(self.frame.MainWindow, self.frame.np.config.sections["transfers"]["downloaddir"])
+        dir = ChooseDir(self.frame.MainWindow, self.frame.np.config.sections["transfers"]["downloaddir"], multichoice=False)
 
         if dir is None:
             return
@@ -675,7 +657,7 @@ class UserBrowse:
 
     def OnDownloadDirectoryRecursiveTo(self, widget):
 
-        dir = ChooseDir(self.frame.MainWindow, self.frame.np.config.sections["transfers"]["downloaddir"])
+        dir = ChooseDir(self.frame.MainWindow, self.frame.np.config.sections["transfers"]["downloaddir"], multichoice=False)
 
         if dir is None:
             return
@@ -690,14 +672,14 @@ class UserBrowse:
         if dir is None:
             return
 
+        ldir = prefix + dir.split("\\")[-1]
+
         for d, f in self.shares:
 
             # Find the wanted directory
             if d != dir:
                 continue
 
-            ldir = prefix + dir.split("\\")[-1]
-
             priorityfiles = []
             normalfiles = []
 
@@ -723,30 +705,16 @@ class UserBrowse:
 
                 path = "\\".join([dir, file[1]])
                 size = file[2]
-                length = bitrate = None
-                attrs = file[4]
+                h_bitrate, bitrate, h_length = GetResultBitrateLength(size, file[4])
 
-                if attrs != []:
+                self.frame.np.transfers.getFile(self.user, path, ldir, size=size, bitrate=h_bitrate, length=h_length, checkduplicate=True)
 
-                    bitrate = str(attrs[0])
-                    if len(attrs) > 2 and attrs[2]:
-                        bitrate += " (vbr)"
+        if not recurse:
+            return
 
-                    try:
-                        rl = int(attrs[1])
-                    except ValueError:
-                        rl = 0
-
-                    length = "%i:%02i" % (int(rl // 60), rl % 60)
-
-                self.frame.np.transfers.getFile(self.user, path, ldir, size=size, bitrate=bitrate, length=length)
-
-            if not recurse:
-                break
-
-            for subdir, subf in self.shares:
-                if dir in subdir and dir != subdir:
-                    self.DownloadDirectory(subdir, os.path.join(ldir, ""), recurse)
+        for subdir, subf in self.shares:
+            if dir in subdir and dir != subdir:
+                self.DownloadDirectory(subdir, os.path.join(ldir, ""))
 
     def OnDownloadFiles(self, widget, prefix=""):
 
@@ -766,37 +734,27 @@ class UserBrowse:
 
                 path = "\\".join([dir, file[1]])
                 size = file[2]
-                length = bitrate = None
-                attrs = file[4]
-
-                if attrs != []:
-
-                    bitrate = str(attrs[0])
-                    if len(attrs) > 2 and attrs[2]:
-                        bitrate += " (vbr)"
-
-                    try:
-                        rl = int(attrs[1])
-                    except ValueError:
-                        rl = 0
-
-                    length = "%i:%02i" % (int(rl // 60), rl % 60)
+                h_bitrate, bitrate, h_length = GetResultBitrateLength(size, file[4])
 
                 # Get the file
-                self.frame.np.transfers.getFile(self.user, path, prefix, size=size, bitrate=bitrate, length=length)
+                self.frame.np.transfers.getFile(self.user, path, prefix, size=size, bitrate=h_bitrate, length=h_length, checkduplicate=True)
 
             # We have found the wanted directory: we can break out of the loop
             break
 
     def OnDownloadFilesTo(self, widget):
 
-        basedir, subdir = self.selected_folder.rsplit("\\", 1)
-        path = os.path.join(self.frame.np.config.sections["transfers"]["downloaddir"], subdir)
+        try:
+            _, dir = self.selected_folder.rsplit("\\", 1)
+        except ValueError:
+            dir = self.selected_folder
+
+        path = os.path.join(self.frame.np.config.sections["transfers"]["downloaddir"], dir)
 
         if os.path.exists(path) and os.path.isdir(path):
-            ldir = ChooseDir(self.frame.MainWindow, path)
+            ldir = ChooseDir(self.frame.MainWindow, path, multichoice=False)
         else:
-            ldir = ChooseDir(self.frame.MainWindow, self.frame.np.config.sections["transfers"]["downloaddir"])
+            ldir = ChooseDir(self.frame.MainWindow, self.frame.np.config.sections["transfers"]["downloaddir"], multichoice=False)
 
         if ldir is None:
             return
@@ -809,6 +767,7 @@ class UserBrowse:
     def OnUploadDirectoryTo(self, widget, recurse=0):
 
         dir = self.selected_folder
+
         if dir is None:
             return
 
@@ -839,6 +798,7 @@ class UserBrowse:
         if dir == "" or dir is None or user is None or user == "":
             return
 
+        realpath = self.frame.np.shares.virtual2real(dir)
         ldir = dir.split("\\")[-1]
 
         for d, f in self.shares:
@@ -848,9 +808,10 @@ class UserBrowse:
                 continue
 
             for file in f:
-                path = "\\".join([dir, file[1]])
+                filename = "\\".join([dir, file[1]])
+                realfilename = "\\".join([realpath, file[1]])
                 size = file[2]
-                self.frame.np.transfers.pushFile(user, path, ldir, size=size)
+                self.frame.np.transfers.pushFile(user, filename, realfilename, ldir, size=size)
                 self.frame.np.transfers.checkUploadQueue()
 
         if not recurse:
@@ -863,6 +824,8 @@ class UserBrowse:
     def OnUploadFiles(self, widget, prefix=""):
 
         dir = self.selected_folder
+        realpath = self.frame.np.shares.virtual2real(dir)
+
         users = []
 
         for entry in self.frame.np.config.sections["server"]["userlist"]:
@@ -882,7 +845,7 @@ class UserBrowse:
         self.frame.np.ProcessRequestToPeer(user, slskmessages.UploadQueueNotification(None))
 
         for fn in self.selected_files:
-            self.frame.np.transfers.pushFile(user, "\\".join([dir, fn]), prefix)
+            self.frame.np.transfers.pushFile(user, "\\".join([dir, fn]), "\\".join([realpath, fn]), prefix)
             self.frame.np.transfers.checkUploadQueue()
 
     def OnPlayFiles(self, widget, prefix=""):
@@ -931,7 +894,7 @@ class UserBrowse:
 
         if self.search_list != []:
 
-            if self.search_position not in range(len(self.search_list)):
+            if self.search_position not in list(range(len(self.search_list))):
                 self.search_position = 0
 
             self.search_list.sort()
@@ -943,13 +906,13 @@ class UserBrowse:
 
             # Get matching files in the current directory
             resultfiles = []
-            for file in self.files.keys():
+            for file in list(self.files.keys()):
                 if query in file.lower():
                     resultfiles.append(file)
 
             sel = self.FileTreeView.get_selection()
             sel.unselect_all()
-            l = 1
+            l = 1  # noqa: E741
             resultfiles.sort()
 
             for fn in resultfiles:
@@ -961,7 +924,7 @@ class UserBrowse:
                 if l:
                     # Position cursor at first match
                     self.FileTreeView.scroll_to_cell(path, None, True, 0.5, 0.5)
-                    l = 0
+                    l = 0  # noqa: E741
         else:
             self.search_position = 0
 
@@ -1009,12 +972,3 @@ class UserBrowse:
 
         if "$" in executable:
             executeCommand(executable, path)
-
-    def OnEncodingChanged(self, widget):
-
-        encoding = self.Encoding.get_model().get(self.Encoding.get_active_iter(), 0)[0]
-
-        if encoding != self.encoding:
-            self.encoding = encoding
-            self.MakeNewModel(self.shares)
-            SaveEncoding(self.frame.np, "userencoding", self.user, self.encoding)
