@@ -1,5 +1,4 @@
-# -*- coding: utf-8 -*-
-#
+# COPYRIGHT (C) 2020 Nicotine+ Team
 # COPYRIGHT (C) 2016-2017 Michael Labouebe <gfarmerfr@free.fr>
 # COPYRIGHT (C) 2008-2010 Quinox <quinox@users.sf.net>
 # COPYRIGHT (C) 2006-2009 Daelstorm <daelstorm@gmail.com>
@@ -21,20 +20,25 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-# Python core
-import tempfile
 import os
+import tempfile
 import time
+from gettext import gettext as _
 
-# Python modules
-import gtk
-import gobject
+from gi.repository import Gdk
+from gi.repository import GdkPixbuf
+from gi.repository import GObject as gobject
+from gi.repository import Gtk as gtk
 
-# Application specific
-from utils import IconNotebook, PopupMenu, EncodingsMenu, SaveEncoding,  Humanize, InitialiseColumns, AppendLine
 from pynicotine import slskmessages
-from pynicotine.utils import CleanFile
+from pynicotine.gtkgui.utils import AppendLine
+from pynicotine.gtkgui.utils import Humanize
+from pynicotine.gtkgui.utils import HumanSpeed
+from pynicotine.gtkgui.utils import IconNotebook
+from pynicotine.gtkgui.utils import InitialiseColumns
+from pynicotine.gtkgui.utils import PopupMenu
 from pynicotine.logfacility import log
+from pynicotine.utils import CleanFile
 
 
 # User Info and User Browse Notebooks
@@ -71,7 +75,7 @@ class UserTabs(IconNotebook):
 
         if msg.user in self.users:
             tab = self.users[msg.user]
-            tab.speed.set_text(_("Speed: %s") % Humanize(msg.avgspeed))
+            tab.speed.set_text(_("Speed: %s") % HumanSpeed(msg.avgspeed))
             tab.filesshared.set_text(_("Files: %s") % Humanize(msg.files))
             tab.dirsshared.set_text(_("Directories: %s") % Humanize(msg.dirs))
 
@@ -94,13 +98,17 @@ class UserTabs(IconNotebook):
 
         if user in self.users:
             self.users[user].conn = conn
-            self.frame.np.queue.put(slskmessages.GetUserStats(user))
         else:
             w = self.subwindow(self, user, conn)
             self.append_page(w.Main, user[:15], w.OnClose)
             self.users[user] = w
             self.frame.np.queue.put(slskmessages.AddUser(user))
 
+    def saveColumns(self):
+
+        for user in self.users:
+            self.users[user].saveColumns()
+
     def ShowLocalInfo(self, user, descr, has_pic, pic, totalupl, queuesize, slotsavail, uploadallowed):
 
         self.InitWindow(user, None)
@@ -128,13 +136,13 @@ class UserTabs(IconNotebook):
 
     def UpdateGauge(self, msg):
 
-        for i in self.users.values():
+        for i in list(self.users.values()):
             if i.conn == msg.conn.conn:
                 i.UpdateGauge(msg)
 
     def UpdateColours(self):
 
-        for i in self.users.values():
+        for i in list(self.users.values()):
             i.ChangeColours()
 
     def TabPopup(self, user):
@@ -155,6 +163,7 @@ class UserTabs(IconNotebook):
             ("#" + _("Detach this tab"), self.users[user].Detach),
             ("#" + _("Close this tab"), self.users[user].OnClose)
         )
+
         popup.set_user(user)
 
         items = popup.get_children()
@@ -167,11 +176,11 @@ class UserTabs(IconNotebook):
 
     def on_tab_click(self, widget, event, child):
 
-        if event.type == gtk.gdk.BUTTON_PRESS:
+        if event.type == Gdk.EventType.BUTTON_PRESS:
 
             n = self.page_num(child)
             page = self.get_nth_page(n)
-            username = [user for user, tab in self.users.items() if tab.Main is page][0]
+            username = [user for user, tab in list(self.users.items()) if tab.Main is page][0]
 
             if event.button == 2:
                 self.users[username].OnClose(widget)
@@ -179,7 +188,7 @@ class UserTabs(IconNotebook):
 
             if event.button == 3:
                 menu = self.TabPopup(username)
-                menu.popup(None, None, None, event.button, event.time)
+                menu.popup(None, None, None, None, event.button, event.time)
                 return True
 
             return False
@@ -236,40 +245,17 @@ class UserInfo:
         self.Hates.set_model(self.hatesStore)
         cols = InitialiseColumns(self.Hates, [_("Hates"), 0, "text", self.CellDataFunc])
         cols[0].set_sort_column_id(0)
-        self.hatesStore.set_sort_column_id(0, gtk.SORT_ASCENDING)
+        self.hatesStore.set_sort_column_id(0, gtk.SortType.ASCENDING)
 
         self.likesStore = gtk.ListStore(gobject.TYPE_STRING)
         self.Likes.set_model(self.likesStore)
         cols = InitialiseColumns(self.Likes, [_("Likes"), 0, "text", self.CellDataFunc])
         cols[0].set_sort_column_id(0)
-        self.likesStore.set_sort_column_id(0, gtk.SORT_ASCENDING)
+        self.likesStore.set_sort_column_id(0, gtk.SortType.ASCENDING)
 
-        # Encoding Combobox
-        self.Elist = {}
-        self.encoding, m = EncodingsMenu(self.frame.np, "userencoding", user)
-        self.EncodingStore = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_STRING)
-        self.Encoding.set_model(self.EncodingStore)
-
-        cell = gtk.CellRendererText()
-        self.Encoding.pack_start(cell, True)
-        self.Encoding.add_attribute(cell, 'text', 0)
-
-        cell2 = gtk.CellRendererText()
-        self.Encoding.pack_start(cell2, False)
-        self.Encoding.add_attribute(cell2, 'text', 1)
-
-        for item in m:
-            self.Elist[item[1]] = self.EncodingStore.append([item[1], item[0]])
-            if self.encoding == item[1]:
-                self.Encoding.set_active_iter(self.Elist[self.encoding])
-
-        self.tag_local = self.makecolour(buffer, "chatremote")
+        self.tag_local = self.makecolour("chatremote")  # noqa: F821
         self.ChangeColours()
 
-        self.InterestsExpander.connect("activate", self.ExpanderStatus)
-        self.InformationExpander.connect("activate", self.ExpanderStatus)
-        self.DescriptionExpander.connect("activate", self.ExpanderStatus)
-
         self.likes_popup_menu = popup = PopupMenu(self)
         popup.setup(
             ("$" + _("I _like this"), self.frame.OnLikeRecommendation),
@@ -277,6 +263,7 @@ class UserInfo:
             ("", None),
             ("#" + _("_Search for this item"), self.frame.OnRecommendSearch)
         )
+
         self.Likes.connect("button_press_event", self.OnPopupLikesMenu)
 
         self.hates_popup_menu = popup = PopupMenu(self)
@@ -286,6 +273,7 @@ class UserInfo:
             ("", None),
             ("#" + _("_Search for this item"), self.frame.OnRecommendSearch)
         )
+
         self.Hates.connect("button_press_event", self.OnPopupHatesMenu)
 
         self.image_menu = popup = PopupMenu(self)
@@ -317,7 +305,7 @@ class UserInfo:
         items[0].set_active(thing in self.frame.np.config.sections["interests"]["likes"])
         items[1].set_active(thing in self.frame.np.config.sections["interests"]["dislikes"])
 
-        self.likes_popup_menu.popup(None, None, None, event.button, event.time)
+        self.likes_popup_menu.popup(None, None, None, None, event.button, event.time)
 
     def OnPopupHatesMenu(self, widget, event):
 
@@ -339,7 +327,7 @@ class UserInfo:
         items[0].set_active(thing in self.frame.np.config.sections["interests"]["likes"])
         items[1].set_active(thing in self.frame.np.config.sections["interests"]["dislikes"])
 
-        self.hates_popup_menu.popup(None, None, None, event.button, event.time)
+        self.hates_popup_menu.popup(None, None, None, None, event.button, event.time)
 
     def ConnClose(self):
         pass
@@ -356,7 +344,7 @@ class UserInfo:
             }
         )
 
-    def CellDataFunc(self, column, cellrenderer, model, iter):
+    def CellDataFunc(self, column, cellrenderer, model, iter, dummy="dummy"):
 
         colour = self.frame.np.config.sections["ui"]["search"]
         if colour == "":
@@ -371,34 +359,30 @@ class UserInfo:
         else:
             self.InfoVbox.set_child_packing(widget, True, True, 0, 0)
 
-    def makecolour(self, buffer, colour):
+    def makecolour(self, colour):
 
         buffer = self.descr.get_buffer()
         colour = self.frame.np.config.sections["ui"][colour]
         font = self.frame.np.config.sections["ui"]["chatfont"]
 
+        tag = buffer.create_tag(font=font)
+
         if colour:
-            return buffer.create_tag(foreground=colour, font=font)
-        else:
-            return buffer.create_tag(font=font)
+            tag.set_property("foreground", colour)
+
+        return tag
 
     def changecolour(self, tag, colour):
 
-        if colour in self.frame.np.config.sections["ui"]:
-            color = self.frame.np.config.sections["ui"][colour]
-        else:
+        color = self.frame.np.config.sections["ui"][colour]
+
+        if color == "":
             color = None
 
+        tag.set_property("foreground", color)
+
         font = self.frame.np.config.sections["ui"]["chatfont"]
-
-        if color:
-            if color == "":
-                color = None
-
-            tag.set_property("foreground", color)
-            tag.set_property("font", font)
-        else:
-            tag.set_property("font", font)
+        tag.set_property("font", font)
 
     def ChangeColours(self):
 
@@ -429,7 +413,7 @@ class UserInfo:
         self.image_pixbuf = None
         self.descr.get_buffer().set_text("")
 
-        AppendLine(self.descr, self.frame.np.decode(descr, self.encoding), self.tag_local, showstamp=False, scroll=False)
+        AppendLine(self.descr, descr, self.tag_local, showstamp=False, scroll=False)
 
         self.uploads.set_text(_("Total uploads allowed: %i") % totalupl)
         self.queuesize.set_text(_("Queue size: %i") % queuesize)
@@ -457,7 +441,7 @@ class UserInfo:
         if has_pic and pic is not None:
             try:
                 import gc
-                loader = gtk.gdk.PixbufLoader()
+                loader = GdkPixbuf.PixbufLoader()
                 loader.write(pic)
                 loader.close()
                 self.image_pixbuf = loader.get_pixbuf()
@@ -466,19 +450,13 @@ class UserInfo:
                 gc.collect()
                 self.actual_zoom = 0
                 self.SavePicture.set_sensitive(True)
-            except TypeError, e:
+            except TypeError as e:  # noqa: F841
                 name = tempfile.mktemp()
                 f = open(name, "w")
                 f.write(pic)
                 f.close()
                 self.image.set_from_file(name)
                 os.remove(name)
-            except Exception, e:
-                self.image.set_from_pixbuf(None)
-                self.SavePicture.set_sensitive(False)
-        else:
-            self.image.set_from_pixbuf(None)
-            self.SavePicture.set_sensitive(False)
 
     def ShowInfo(self, msg):
         self.ShowUserInfo(msg.descr, msg.has_pic, msg.pic, msg.totalupl, msg.queuesize, msg.slotsavail, msg.uploadallowed)
@@ -539,24 +517,14 @@ class UserInfo:
         pathname = os.path.join(self.frame.np.config.sections["transfers"]["downloaddir"], CleanFile(filename))
 
         if not os.path.exists(pathname):
-            self.image_pixbuf.save(pathname, "jpeg", {"quality": "100"})
+            self.image_pixbuf.savev(pathname, "jpeg", ["quality"], ["100"])
             log.add(_("Picture saved to %s") % pathname)
         else:
             log.add(_("Picture not saved, %s already exists.") % pathname)
 
-    def OnEncodingChanged(self, widget):
-
-        encoding = self.Encoding.get_model().get(self.Encoding.get_active_iter(), 0)[0]
-
-        if encoding != self.encoding:
-            self.encoding = encoding
-            buffer = self.descr.get_buffer()
-            buffer.set_text(self.frame.np.decode(self._descr, self.encoding))
-            SaveEncoding(self.frame.np, "userencoding", self.user, self.encoding)
-
     def OnImageClick(self, widget, event):
 
-        if event.type != gtk.gdk.BUTTON_PRESS or event.button != 3:
+        if event.type != Gdk.EventType.BUTTON_PRESS or event.button != 3:
             return False
 
         act = True
@@ -568,13 +536,13 @@ class UserInfo:
         for item in items:
             item.set_sensitive(act)
 
-        self.image_menu.popup(None, None, None, event.button, event.time)
+        self.image_menu.popup(None, None, None, None, event.button, event.time)
 
         return True  # Don't scroll the gtk.ScrolledWindow
 
     def OnScrollEvent(self, widget, event):
 
-        if event.direction == gtk.gdk.SCROLL_UP:
+        if event.get_scroll_deltas().delta_y < 0:
             self.MakeZoomIn()
         else:
             self.MakeZoomOut()
@@ -602,7 +570,7 @@ class UserInfo:
         else:
             self.actual_zoom += self.zoom_factor
 
-        pixbuf_zoomed = self.image_pixbuf.scale_simple(CalcZoomIn(x), CalcZoomIn(y), gtk.gdk.INTERP_TILES)
+        pixbuf_zoomed = self.image_pixbuf.scale_simple(CalcZoomIn(x), CalcZoomIn(y), GdkPixbuf.InterpType.TILES)
         self.image.set_from_pixbuf(pixbuf_zoomed)
 
         del pixbuf_zoomed
@@ -628,7 +596,7 @@ class UserInfo:
             self.actual_zoom += self.zoom_factor
             return
 
-        pixbuf_zoomed = self.image_pixbuf.scale_simple(CalcZoomOut(x), CalcZoomOut(y), gtk.gdk.INTERP_TILES)
+        pixbuf_zoomed = self.image_pixbuf.scale_simple(CalcZoomOut(x), CalcZoomOut(y), GdkPixbuf.InterpType.TILES)
         self.image.set_from_pixbuf(pixbuf_zoomed)
 
         del pixbuf_zoomed
