@@ -1,5 +1,5 @@
-# -*- coding: utf-8 -*-
-#
+# COPYRIGHT (C) 2020 Nicotine+ Team
+# COPYRIGHT (C) 2018 Mutnick <mutnick@techie.com>
 # COPYRIGHT (C) 2016-2017 Michael Labouebe <gfarmerfr@free.fr>
 # COPYRIGHT (C) 2009 Quinox <quinox@users.sf.net>
 # COPYRIGHT (C) 2006-2009 Daelstorm <daelstorm@gmail.com>
@@ -21,14 +21,23 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-import gtk
-import gobject
-import time
-import sys
 import os
+import time
+from gettext import gettext as _
+
+from gi.repository import Gdk
+from gi.repository import GLib
+from gi.repository import GObject as gobject
+from gi.repository import Gtk as gtk
 
 from pynicotine import slskmessages
-from utils import InitialiseColumns, PopupMenu, InputDialog, Humanize, HumanSpeed, PressHeader, showCountryTooltip
+from pynicotine.gtkgui.utils import Humanize
+from pynicotine.gtkgui.utils import HumanSpeed
+from pynicotine.gtkgui.utils import InitialiseColumns
+from pynicotine.gtkgui.utils import InputDialog
+from pynicotine.gtkgui.utils import PopupMenu
+from pynicotine.gtkgui.utils import PressHeader
+from pynicotine.gtkgui.utils import showCountryTooltip
 
 
 class UserList:
@@ -57,32 +66,34 @@ class UserList:
         builder.connect_signals(self)
 
         TARGETS = [('text/plain', 0, 1)]
-        self.UserList.enable_model_drag_source(gtk.gdk.BUTTON1_MASK, TARGETS, gtk.gdk.ACTION_COPY)
-        self.UserList.enable_model_drag_dest(TARGETS, gtk.gdk.ACTION_COPY)
+        self.UserList.enable_model_drag_source(Gdk.ModifierType.BUTTON1_MASK, TARGETS, Gdk.DragAction.COPY)
+        self.UserList.enable_model_drag_dest(TARGETS, Gdk.DragAction.COPY)
         self.UserList.connect("drag_data_get", self.buddylist_drag_data_get_data)
         self.UserList.connect("drag_data_received", self.DragUserToBuddylist)
 
         self.userlist = []
 
         self.usersmodel = gtk.ListStore(
-            gtk.gdk.Pixbuf, gtk.gdk.Pixbuf, gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_BOOLEAN,
-            gobject.TYPE_BOOLEAN, gobject.TYPE_BOOLEAN, gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_INT, gobject.TYPE_INT,
-            gobject.TYPE_INT, gobject.TYPE_INT, gobject.TYPE_STRING
+            gobject.TYPE_OBJECT, gobject.TYPE_OBJECT,
+            gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING,
+            gobject.TYPE_BOOLEAN, gobject.TYPE_BOOLEAN, gobject.TYPE_BOOLEAN,
+            gobject.TYPE_STRING, gobject.TYPE_STRING,
+            gobject.TYPE_INT, gobject.TYPE_INT, gobject.TYPE_INT, gobject.TYPE_INT,
+            gobject.TYPE_STRING
         )
-        statusiconwidth = self.frame.images["offline"].get_width() + 4
-
+        widths = self.frame.np.config.sections["columns"]["userlist_widths"]
         self.cols = cols = InitialiseColumns(
             self.UserList,
-            [_("Status"), statusiconwidth, "pixbuf"],
-            [_("Country"), 25, "pixbuf"],
-            [_("User"), 120, "text", self.CellDataFunc],
-            [_("Speed"), 0, "number", self.CellDataFunc],
-            [_("Files"), 0, "number", self.CellDataFunc],
-            [_("Trusted"), 0, "toggle"],
-            [_("Notify"), 0, "toggle"],
-            [_("Privileged"), 0, "toggle"],
-            [_("Last seen"), 160, "text", self.CellDataFunc],
-            [_("Comments"), -1, "edit", self.CellDataFunc]
+            [_("Status"), widths[0], "pixbuf"],
+            [_("Country"), widths[1], "pixbuf"],
+            [_("User"), widths[2], "text", self.CellDataFunc],
+            [_("Speed"), widths[3], "number", self.CellDataFunc],
+            [_("Files"), widths[4], "number", self.CellDataFunc],
+            [_("Trusted"), widths[5], "toggle"],
+            [_("Notify"), widths[6], "toggle"],
+            [_("Privileged"), widths[7], "toggle"],
+            [_("Last seen"), widths[8], "text", self.CellDataFunc],
+            [_("Comments"), widths[9], "edit", self.CellDataFunc]
         )
 
         self.col_status, self.col_country, self.col_user, self.col_speed, self.col_files, self.col_trusted, self.col_notify, self.col_privileged, self.col_last, self.col_comments = cols
@@ -101,92 +112,74 @@ class UserList:
 
         config = self.frame.np.config.sections
 
-        for i in range(10):
+        try:
+            for i in range(len(cols)):
 
-            parent = cols[i].get_widget().get_ancestor(gtk.Button)
-            if parent:
-                parent.connect('button_press_event', PressHeader)
+                parent = cols[i].get_widget().get_ancestor(gtk.Button)
+                if parent:
+                    parent.connect("button_press_event", PressHeader)
 
-            # Read Show / Hide column settings from last session
-            cols[i].set_visible(config["columns"]["userlist"][i])
+                # Read Show / Hide column settings from last session
+                cols[i].set_visible(self.frame.np.config.sections["columns"]["userlist"][i])
+        except IndexError:
+            # Column count in config is probably incorrect (outdated?), don't crash
+            pass
 
         if config["columns"]["hideflags"]:
             cols[1].set_visible(0)
             config["columns"]["userlist"][1] = 0
 
-        for render in self.col_trusted.get_cell_renderers():
+        for render in self.col_trusted.get_cells():
             render.connect('toggled', self.cell_toggle_callback, self.UserList, 5)
 
-        for render in self.col_notify.get_cell_renderers():
+        for render in self.col_notify.get_cells():
             render.connect('toggled', self.cell_toggle_callback, self.UserList, 6)
 
-        for render in self.col_privileged.get_cell_renderers():
+        for render in self.col_privileged.get_cells():
             render.connect('toggled', self.cell_toggle_callback, self.UserList, 7)
 
-        renderers = self.col_comments.get_cell_renderers()
+        renderers = self.col_comments.get_cells()
 
         for render in renderers:
             render.connect('edited', self.cell_edited_callback, self.UserList, 9)
 
         self.UserList.set_model(self.usersmodel)
-        self.UserList.set_property("rules-hint", True)
         self.privileged = []
         self.notify = []
         self.trusted = []
 
         for user in self.frame.np.config.sections["server"]["userlist"]:
+            username, comment, notify, privileged, trusted, last_seen, flag = user
 
-            notify = user[2]
-            privileged = user[3]
-
-            if len(user) > 4:
-                trusted = user[4]
-            else:
-                trusted = 0
-
-            if len(user) > 5:
-                last_seen = user[5]
-                try:
-                    time_from_epoch = time.mktime(time.strptime(last_seen, "%m/%d/%Y %H:%M:%S"))
-                except:
-                    if last_seen == '':
-                        time_from_epoch = sys.maxint
-                    else:
-                        time_from_epoch = 0
-            else:
+            try:
+                time_from_epoch = time.mktime(time.strptime(last_seen, "%m/%d/%Y %H:%M:%S"))
+            except ValueError:
                 last_seen = _("Never seen")
-                user += [last_seen]
                 time_from_epoch = 0
 
-            if len(user) > 6:
-                flag = user[6]
-            else:
-                user += [None]
-                flag = None
-
             row = [
                 self.frame.GetStatusImage(0),
                 self.frame.GetFlagImage(flag),
-                user[0], "0", "0",
+                username, "0", "0",
                 trusted, notify, privileged, last_seen,
-                user[1], 0, 0, 0,
-                int(time_from_epoch),
+                comment, 0, 0, 0,
+                time_from_epoch,
                 flag
             ]
 
             if len(user) > 2:
-                if user[2]:
-                    self.notify.append(user[0])
-                if user[3]:
-                    self.privileged.append(user[0])
+                if notify:
+                    self.notify.append(username)
+                if privileged:
+                    self.privileged.append(username)
                 if trusted:
-                    self.trusted.append(user[0])
+                    self.trusted.append(username)
 
-            iter = self.usersmodel.append(row)
-            self.userlist.append([user[0], user[1], last_seen, iter, flag])
+            iter_ = self.usersmodel.insert(0, row)
+            self.userlist.append([user[0], user[1], last_seen, iter_, flag])
 
-        self.usersmodel.set_sort_column_id(2, gtk.SORT_ASCENDING)
-        self.Popup_Menu_PrivateRooms = PopupMenu(self.frame)
+        self.usersmodel.set_sort_column_id(2, gtk.SortType.ASCENDING)
+        self.Popup_Menu_PrivateRooms = PopupMenu(self.frame, False)
         self.popup_menu = popup = PopupMenu(frame)
 
         popup.setup(
@@ -228,32 +221,13 @@ class UserList:
     def OnTooltip(self, widget, x, y, keyboard_mode, tooltip):
         return showCountryTooltip(widget, x, y, tooltip, 14, 'flag_')
 
-    def OnMoveList(self, widget):
-
-        tab = always = chatrooms = False
-
-        if self.frame.buddylist_in_tab.get_active():
-            tab = True
-        if self.frame.buddylist_always_visible.get_active():
-            always = True
-        if self.frame.buddylist_in_chatrooms1.get_active():
-            chatrooms = True
-
-        if tab:
-            self.frame.buddylist_in_chatrooms1.set_active(True)
-            self.frame.OnChatRooms(None)
-        if always:
-            self.frame.buddylist_in_tab.set_active(True)
-        if chatrooms:
-            self.frame.buddylist_always_visible.set_active(True)
-
     def OnAddUser(self, widget):
 
-        text = widget.get_text()
+        text = self.AddUserEntry.get_text()
         if not text:
             return
 
-        widget.set_text("")
+        self.AddUserEntry.set_text("")
         self.AddToList(text)
 
     def UpdateColours(self):
@@ -269,7 +243,7 @@ class UserList:
 
     def DragUserToBuddylist(self, treeview, context, x, y, selection, info, etime):
 
-        model = treeview.get_model()
+        model = treeview.get_model()  # noqa: F841
         user = selection.data
 
         if user:
@@ -278,7 +252,7 @@ class UserList:
     def OnSettingsBanIgnore(self, widget):
         self.frame.OnSettingsBanIgnore(widget)
 
-    def CellDataFunc(self, column, cellrenderer, model, iter):
+    def CellDataFunc(self, column, cellrenderer, model, iter, dummy="dummy"):
 
         colour = self.frame.np.config.sections["ui"]["search"]
 
@@ -331,7 +305,7 @@ class UserList:
     def SetLastSeen(self, user, online=False):
 
         last_seen = ""
-        time_from_epoch = sys.maxint
+        time_from_epoch = 2147483647  # Gtk only allows range -2147483648 to 2147483647 in set()
 
         if not online:
             last_seen = time.strftime("%m/%d/%Y %H:%M:%S")
@@ -367,7 +341,7 @@ class UserList:
             self.usersmodel.set(user[3], 0, self.frame.GetStatusImage(0), 3, "0", 4, "0", 10, 0, 11, 0, 12, 0)
 
         for user in self.userlist:
-            if self.usersmodel.get(user[3], 8)[0] is "":
+            if self.usersmodel.get(user[3], 8)[0] == "":
                 self.SetLastSeen(user[0])
 
     def OnPopupMenu(self, widget, event):
@@ -380,15 +354,15 @@ class UserList:
             user = self.UserList.get_model().get_value(self.UserList.get_model().get_iter(path), 2)
 
             if event.button != 3:
-                if event.type == gtk.gdk._2BUTTON_PRESS:
+                if event.type == Gdk.EventType._2BUTTON_PRESS:
                     self.frame.privatechats.SendMessage(user, None, 1)
                     self.frame.ChangeMainPage(None, "private")
                 return
 
             self.popup_menu.set_user(user)
 
-            items = self.popup_menu.get_children()
-            me = (self.popup_menu.user == None or self.popup_menu.user == self.frame.np.config.sections["server"]["login"])
+            items = self.popup_menu.get_children()  # noqa: F841
+            me = (self.popup_menu.user is None or self.popup_menu.user == self.frame.np.config.sections["server"]["login"])
 
             self.Menu_BanUser.set_active(user in self.frame.np.config.sections["server"]["banlist"])
             self.Menu_IgnoreUser.set_active(user in self.frame.np.config.sections["server"]["ignorelist"])
@@ -397,7 +371,7 @@ class UserList:
             self.Menu_OnTrusted.set_active(user in self.trusted)
             self.Menu_PrivateRooms.set_sensitive(not me)  # Private rooms
 
-            self.popup_menu.popup(None, None, None, event.button, event.time)
+            self.popup_menu.popup(None, None, None, None, event.button, event.time)
 
     def GetIter(self, user):
 
@@ -427,7 +401,7 @@ class UserList:
 
         if msg.status:  # online
             self.SetLastSeen(msg.user, online=True)
-        elif self.usersmodel.get(iter, 8)[0] is "":  # disconnected
+        elif self.usersmodel.get(iter, 8)[0] == "":  # disconnected
             self.SetLastSeen(msg.user)
 
     def GetUserStats(self, msg):
@@ -440,15 +414,10 @@ class UserList:
         hfiles = Humanize(msg.files)
         self.usersmodel.set(iter, 3, hspeed, 4, hfiles, 11, msg.avgspeed, 12, msg.files)
 
-        if msg.country is not None:
+        if msg.country is not None and msg.country != "":
 
             flag = "flag_" + msg.country
-            self.usersmodel.set(iter, 1, self.frame.GetFlagImage(flag), 14, flag)
-
-            for i in self.userlist:
-                if i[0] == msg.user:
-                    i[4] = flag
-                    break
+            self.SetUserFlag(msg.user, flag)
 
     def SetUserFlag(self, user, flag):
 
@@ -478,11 +447,11 @@ class UserList:
         self.frame.np.queue.put(slskmessages.GetPeerAddress(user))
 
         for widget in self.frame.BuddiesComboEntries:
-            gobject.idle_add(widget.Append, user)
+            GLib.idle_add(widget.Append, user)
 
         if self.frame.np.config.sections["words"]["buddies"]:
-            gobject.idle_add(self.frame.chatrooms.roomsctrl.UpdateCompletions)
-            gobject.idle_add(self.frame.privatechats.UpdateCompletions)
+            GLib.idle_add(self.frame.chatrooms.roomsctrl.UpdateCompletions)
+            GLib.idle_add(self.frame.privatechats.UpdateCompletions)
 
     def OnEditComments(self, widget):
 
@@ -495,7 +464,7 @@ class UserList:
         else:
             comments = ""
 
-        comments = InputDialog(self.frame.MainWindow, _("Edit comments")+"...", _("Comments")+":", comments)
+        comments = InputDialog(self.frame.MainWindow, _("Edit comments") + "...", _("Comments") + ":", comments)
 
         if comments is not None:
             for i in self.userlist:
@@ -507,7 +476,7 @@ class UserList:
 
     def SaveUserList(self):
 
-        l = []
+        l = []  # noqa: E741
 
         for i in self.userlist:
             user, comment, seen, iter, flag = i
@@ -519,11 +488,12 @@ class UserList:
     def saveColumns(self):
 
         columns = []
-
+        widths = []
         for column in self.UserList.get_columns():
             columns.append(column.get_visible())
-
+            widths.append(column.get_width())
         self.frame.np.config.sections["columns"]["userlist"] = columns
+        self.frame.np.config.sections["columns"]["userlist_widths"] = widths
 
     def RemoveFromList(self, user):
 
@@ -543,11 +513,11 @@ class UserList:
         self.SaveUserList()
 
         for widget in self.frame.BuddiesComboEntries:
-            gobject.idle_add(widget.Remove, user)
+            GLib.idle_add(widget.Remove, user)
 
         if self.frame.np.config.sections["words"]["buddies"]:
-            gobject.idle_add(self.frame.chatrooms.roomsctrl.UpdateCompletions)
-            gobject.idle_add(self.frame.privatechats.UpdateCompletions)
+            GLib.idle_add(self.frame.chatrooms.roomsctrl.UpdateCompletions)
+            GLib.idle_add(self.frame.privatechats.UpdateCompletions)
 
     def OnRemoveUser(self, widget):
         self.RemoveFromList(self.popup_menu.get_user())
@@ -560,7 +530,7 @@ class UserList:
             if user in self.trusted:
                 self.trusted.remove(user)
         else:
-            if not user in self.trusted:
+            if user not in self.trusted:
                 self.trusted.append(user)
 
         for i in self.userlist:
@@ -577,7 +547,7 @@ class UserList:
             if user in self.notify:
                 self.notify.remove(user)
         else:
-            if not user in self.notify:
+            if user not in self.notify:
                 self.notify.append(user)
 
         for i in self.userlist:
@@ -594,7 +564,7 @@ class UserList:
             if user in self.privileged:
                 self.privileged.remove(user)
         else:
-            if not user in self.privileged:
+            if user not in self.privileged:
                 self.privileged.append(user)
 
         for i in self.userlist:
