@@ -1,5 +1,5 @@
-# -*- coding: utf-8 -*-
-#
+# COPYRIGHT (C) 2020 Nicotine+ Team
+# COPYRIGHT (C) 2020 Lene Preuss <lene.preuss@gmail.com>
 # COPYRIGHT (C) 2016-2017 Michael Labouebe <gfarmerfr@free.fr>
 # COPYRIGHT (C) 2016 Mutnick <muhing@yahoo.com>
 # COPYRIGHT (C) 2008-2011 Quinox <quinox@users.sf.net>
@@ -21,25 +21,29 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
-
-from __future__ import division
-
-import gtk
-import gobject
-import pango
-import time
 import locale
+import math
 import os
-import sys
-import string
 import re
+import sys
+import time
 import types
-import urllib
+import urllib.error
+import urllib.parse
+import urllib.request
 import webbrowser
+from gettext import gettext as _
+
+from gi.repository import Gdk
+from gi.repository import GLib
+from gi.repository import GObject as gobject
+from gi.repository import Gtk as gtk
+from gi.repository import Pango as pango
 
 from pynicotine import slskmessages
-from pynicotine.utils import executeCommand, findBestEncoding
-from countrycodes import code2name
+from pynicotine.gtkgui.countrycodes import code2name
+from pynicotine.utils import executeCommand
+
 
 DECIMALSEP = ""
 
@@ -52,15 +56,15 @@ NICOTINE = None
 
 
 def popupWarning(parent, title, warning, icon=None):
-    dlg = gtk.Dialog(title=title, parent=parent, buttons=(gtk.STOCK_OK, gtk.RESPONSE_OK))
-    dlg.set_default_response(gtk.RESPONSE_OK)
+    dlg = gtk.Dialog(title=title, parent=parent, buttons=(gtk.STOCK_OK, gtk.ResponseType.OK))
+    dlg.set_default_response(gtk.ResponseType.OK)
     dlg.set_icon(icon)
     dlg.set_border_width(10)
     dlg.vbox.set_spacing(10)
     hbox = gtk.HBox(spacing=5)
     hbox.set_border_width(5)
     hbox.show()
-    dlg.vbox.pack_start(hbox)
+    dlg.vbox.pack_start(hbox, True, True, 0)
 
     image = gtk.Image()
     image.set_padding(0, 0)
@@ -68,20 +72,21 @@ def popupWarning(parent, title, warning, icon=None):
     image.set_from_stock(icon, 4)
     image.show()
 
-    hbox.pack_start(image)
+    hbox.pack_start(image, True, True, 0)
     label = gtk.Label()
     label.set_markup(warning)
     label.set_line_wrap(True)
-    hbox.pack_start(label, True, True)
+    hbox.pack_start(label, True, True, 0)
 
     dlg.vbox.show_all()
 
-    result = None
-    if dlg.run() == gtk.RESPONSE_OK:
+    result = None  # noqa: F841
+    if dlg.run() == gtk.ResponseType.OK:
         dlg.destroy()
 
     return 0
 
+
 # we could move this into a new class
 previouscountrypath = None
 
@@ -110,7 +115,7 @@ def showCountryTooltip(widget, x, y, tooltip, sourcecolumn, stripprefix='flag_')
 
     title = column.get_title()
 
-    if (title != _("Country")):
+    if title != _("Country"):
         return False
 
     model = widget.get_model()
@@ -141,42 +146,66 @@ def showCountryTooltip(widget, x, y, tooltip, sourcecolumn, stripprefix='flag_')
     return True
 
 
-def HumanizeBytes(size):
-
-    if size is None:
-        return None
-
-    try:
-        s = int(size)
-        if s >= 1000*1024*1024:
-            r = _("%.2f GB") % (float(s) / (1024.0*1024.0*1024.0))
-        elif s >= 1000*1024:
-            r = _("%.2f MB") % (float(s) / (1024.0*1024.0))
-        elif s >= 1000:
-            r = _("%.2f KB") % (float(s) / 1024.0)
-        else:
-            r = _("%.2f  B") % (float(s))
-        return r
-    except Exception as e:
-        return size
-
-    return str(size)
-
-
 def recode(s):
     try:
         return s.decode(locale.nl_langinfo(locale.CODESET), "replace").encode("utf-8", "replace")
-    except:
+    except Exception:
         return s
 
 
 def recode2(s):
     try:
         return s.decode("utf-8", "replace").encode(locale.nl_langinfo(locale.CODESET), "replace")
-    except:
+    except Exception:
         return s
 
 
+def FillFileGroupingCombobox(combobox):
+    grouplist = gtk.ListStore(str)
+    groups = [
+        "No grouping",
+        "Group by folder",
+        "Group by user",
+    ]
+
+    for group in groups:
+        grouplist.append([group])
+
+    combobox.set_model(grouplist)
+    renderer_text = gtk.CellRendererText()
+    combobox.pack_start(renderer_text, True)
+    combobox.add_attribute(renderer_text, "text", 0)
+
+
+def SelectUserRowIter(fmodel, sel, user_index, selected_user, iter):
+    while iter is not None:
+        user = fmodel.get_value(iter, user_index)
+
+        if selected_user == user:
+            sel.select_path(fmodel.get_path(iter),)
+
+        child = fmodel.iter_children(iter)
+
+        SelectUserRowIter(fmodel, sel, user_index, selected_user, child)
+
+        iter = fmodel.iter_next(iter)
+
+
+def CollapseTreeview(treeview, groupingmode):
+    treeview.collapse_all()
+
+    if groupingmode == 1:
+        # Group by folder
+
+        model = treeview.get_model()
+        iter = model.get_iter_first()
+
+        while iter is not None:
+            path = model.get_path(iter)
+            treeview.expand_to_path(path)
+            iter = model.iter_next(iter)
+
+
 def InitialiseColumns(treeview, *args):
 
     i = 0
@@ -216,21 +245,21 @@ def InitialiseColumns(treeview, *args):
 
         if c[1] == -1:
             column.set_resizable(False)
-            column.set_sizing(gtk.TREE_VIEW_COLUMN_AUTOSIZE)
+            column.set_sizing(gtk.TreeViewColumnSizing.AUTOSIZE)
         else:
             column.set_resizable(True)
             if c[1] == 0:
-                column.set_sizing(gtk.TREE_VIEW_COLUMN_GROW_ONLY)
+                column.set_sizing(gtk.TreeViewColumnSizing.GROW_ONLY)
             else:
-                column.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
+                column.set_sizing(gtk.TreeViewColumnSizing.FIXED)
                 column.set_fixed_width(c[1])
             column.set_min_width(0)
 
         if len(c) > 3 and type(c[3]) is not list:
             column.set_cell_data_func(renderer, c[3])
 
-        column.set_reorderable(True)
-        column.set_widget(gtk.Label(c[0]))
+        column.set_reorderable(False)
+        column.set_widget(gtk.Label.new(c[0]))
         column.get_widget().show()
 
         treeview.append_column(column)
@@ -266,7 +295,8 @@ def PressHeader(widget, event):
         pos += 1
 
     menu.show_all()
-    menu.popup(None, None, None, event.button, event.time)
+    menu.attach_to_widget(widget.get_toplevel(), None)
+    menu.popup(None, None, None, None, event.button, event.time)
 
     return True
 
@@ -276,62 +306,82 @@ def header_toggle(menuitem, column):
     NICOTINE.SaveColumns()
 
 
+def SetTreeviewSelectedRow(treeview, event):
+    """Handles row selection when right-clicking in a treeview"""
+
+    pathinfo = treeview.get_path_at_pos(event.x, event.y)
+
+    if pathinfo is not None:
+        path, col, cell_x, cell_y = pathinfo
+
+        selection = treeview.get_selection()
+
+        # Make sure we don't attempt to select a single row if the row is already
+        # in a selection of multiple rows, otherwise the other rows will be unselected
+        if selection.count_selected_rows() <= 1 or not selection.path_is_selected(path):
+            treeview.grab_focus()
+            treeview.set_cursor(path, col, False)
+
+
 def ScrollBottom(widget):
     va = widget.get_vadjustment()
-    if va is None:
-        return False
-    va.set_value(va.upper - va.page_size)
+    try:
+        va.set_value(va.get_upper() - va.get_page_size())
+    except AttributeError:
+        pass
+    widget.set_vadjustment(va)
     return False
 
 
 def UrlEvent(tag, widget, event, iter, url):
-    if tag.last_event_type == gtk.gdk.BUTTON_PRESS and event.type == gtk.gdk.BUTTON_RELEASE and event.button == 1:
+    if tag.last_event_type == Gdk.EventType.BUTTON_PRESS and event.button.type == Gdk.EventType.BUTTON_RELEASE and event.button.button == 1:
         if url[:4] == "www.":
             url = "http://" + url
-        OpenUri(url)
-    tag.last_event_type = event.type
+        OpenUri(url, widget.get_toplevel())
+    tag.last_event_type = event.button.type
 
 
-def OpenUri(uri):
+def OpenUri(uri, window):
     """Open a URI in an external (web) browser. The given argument has
     to be a properly formed URI including the scheme (fe. HTTP).
-
     As of now failures will be silently discarded."""
 
     # Situation 1, user defined a way of handling the protocol
     protocol = uri[:uri.find(":")]
     if protocol in PROTOCOL_HANDLERS:
-        if PROTOCOL_HANDLERS[protocol].__class__ is types.MethodType:
+        if isinstance(PROTOCOL_HANDLERS[protocol], types.MethodType):
             PROTOCOL_HANDLERS[protocol](uri.strip())
             return
         if PROTOCOL_HANDLERS[protocol]:
             executeCommand(PROTOCOL_HANDLERS[protocol], uri)
             return
 
-    # Situation 2, user did not define a way of handling the protocol, we'll leave it up to python
-    if webbrowser:
+    # Situation 2, user did not define a way of handling the protocol
+    if sys.platform == "win32" and webbrowser:
         webbrowser.open(uri)
         return
 
-    # Situation 3, we let Gnome VFS deal with it
     try:
-        import gnomevfs
-        gnomevfs.url_show(uri)
-        return
-    except Exception, e:
-        pass
+        gtk.show_uri_on_window(window, uri, Gdk.CURRENT_TIME)
+    except AttributeError:
+        screen = window.get_screen()
+        gtk.show_uri(screen, uri, Gdk.CURRENT_TIME)
 
 
 def AppendLine(textview, line, tag=None, timestamp=None, showstamp=True, timestamp_format="%H:%M:%S", username=None, usertag=None, scroll=True):
 
-    if type(line) not in (type(""), type(u"")):
+    if type(line) not in (type(""), type("")):
         line = str(line)  # Error messages are sometimes tuples
 
     def _makeurltag(buffer, tag, url):
         props = {}
 
-        props["foreground_gdk"] = gtk.gdk.color_parse(NICOTINE.np.config.sections["ui"]["urlcolor"])
-        props["underline"] = pango.UNDERLINE_SINGLE
+        color = NICOTINE.np.config.sections["ui"]["urlcolor"]
+
+        if color != "":
+            props["foreground"] = color
+
+        props["underline"] = pango.Underline.SINGLE
         tag = buffer.create_tag(**props)
         tag.last_event_type = -1
         tag.connect("event", UrlEvent, url)
@@ -347,10 +397,10 @@ def AppendLine(textview, line, tag=None, timestamp=None, showstamp=True, timesta
 
     def _usertag(buffer, section):
         # Tag usernames with popup menu creating tag, and away/online/offline colors
-        if USERNAMEHOTSPOTS and username != None and usertag != None:
-            np = re.compile(re.escape(username))
+        if USERNAMEHOTSPOTS and username is not None and usertag is not None:
+            np = re.compile(re.escape(str(username)))
             match = np.search(section)
-            if match != None:
+            if match is not None:
                 start2 = section[:match.start()]
                 name = match.group()[:]
                 start = section[match.end():]
@@ -364,14 +414,17 @@ def AppendLine(textview, line, tag=None, timestamp=None, showstamp=True, timesta
 
     scrolledwindow = textview.get_parent()
     va = scrolledwindow.get_vadjustment()
-    bottom = va.value >= (va.upper - int(va.page_size*1.5))
+    try:
+        bottom = va.value >= (va.upper - int(va.page_size * 1.5))
+    except AttributeError:
+        bottom = True
 
     buffer = textview.get_buffer()
     linenr = buffer.get_line_count()
     ME = 0
 
     if line.startswith("* "):
-        ME = 1
+        ME = 1  # noqa: F841
 
     TIMESTAMP = None
     TS = 0
@@ -400,10 +453,10 @@ def AppendLine(textview, line, tag=None, timestamp=None, showstamp=True, timesta
         _usertag(buffer, start)
         url = match.group()[:-1]
         urltag = _makeurltag(buffer, tag, url)
-        line = line[match.end()-1:]
+        line = line[match.end() - 1:]
 
         if url.startswith("slsk://") and HUMANIZE_URLS:
-            url = urllib.url2pathname(url)
+            url = urllib.request.url2pathname(url)
 
         _append(buffer, url, urltag)
         # Match remaining url
@@ -413,11 +466,56 @@ def AppendLine(textview, line, tag=None, timestamp=None, showstamp=True, timesta
         _usertag(buffer, line)
 
     if scroll and bottom:
-        gobject.idle_add(ScrollBottom, scrolledwindow)
+        GLib.idle_add(ScrollBottom, scrolledwindow)
 
     return linenr
 
 
+class BuddiesComboBox:
+
+    def __init__(self, frame, ComboBox):
+
+        self.frame = frame
+
+        self.items = {}
+
+        self.combobox = ComboBox
+
+        self.store = gtk.ListStore(gobject.TYPE_STRING)
+        self.combobox.set_model(self.store)
+        self.combobox.set_entry_text_column(0)
+
+        self.store.set_default_sort_func(lambda *args: -1)
+        self.store.set_sort_column_id(-1, gtk.SortType.ASCENDING)
+
+        self.combobox.show()
+
+    def Fill(self):
+
+        self.items.clear()
+        self.store.clear()
+
+        self.items[""] = self.store.append([""])
+
+        for user in self.frame.np.config.sections["server"]["userlist"]:
+            self.items[user[0]] = self.store.append([user[0]])
+
+        self.store.set_sort_column_id(0, gtk.SortType.ASCENDING)
+
+    def Append(self, item):
+
+        if item in self.items:
+            return
+
+        self.items[item] = self.combobox.get_model().append([item])
+
+    def Remove(self, item):
+
+        if item in self.items:
+            self.combobox.get_model().remove(self.items[item])
+            del self.items[item]
+
+
 class ImageLabel(gtk.HBox):
 
     def __init__(self, label="", image=None, onclose=None, closebutton=False, angle=0, show_image=True, statusimage=None, show_status_image=False):
@@ -483,9 +581,9 @@ class ImageLabel(gtk.HBox):
         self.add(self.Box)
         self.Box.show()
 
-        self.Box.pack_start(self.statusimage, False, False)
-        self.Box.pack_start(self.label, True, True)
-        self.Box.pack_start(self.image, False, False)
+        self.Box.pack_start(self.statusimage, False, False, 0)
+        self.Box.pack_start(self.label, True, True, 0)
+        self.Box.pack_start(self.image, False, False, 0)
 
         if self.closebutton and self.onclose is not None:
             self._add_close_button()
@@ -539,14 +637,14 @@ class ImageLabel(gtk.HBox):
             return
         self.button = gtk.Button()
         img = gtk.Image()
-        img.set_from_stock(gtk.STOCK_CLOSE, gtk.ICON_SIZE_MENU)
+        img.set_from_stock(gtk.STOCK_CLOSE, gtk.IconSize.MENU)
         self.button.add(img)
         if self.onclose is not None:
             self.button.connect("clicked", self.onclose)
-        self.button.set_relief(gtk.RELIEF_NONE)
+        self.button.set_relief(gtk.ReliefStyle.NONE)
 
         self.button.show_all()
-        self.Box.pack_start(self.button, False, False)
+        self.Box.pack_start(self.button, False, False, 0)
 
     def _remove_close_button(self):
         if "button" not in self.__dict__:
@@ -570,8 +668,8 @@ class ImageLabel(gtk.HBox):
                 color = NICOTINE.np.config.sections["ui"]["tab_default"]
 
             try:
-                gtk.gdk.color_parse(color)
-            except:
+                Gdk.color_parse(color)
+            except Exception:
                 color = ""
         else:
             color = ""
@@ -618,7 +716,6 @@ class ImageLabel(gtk.HBox):
 
 class IconNotebook:
     """ This class implements a pseudo gtk.Notebook
-
     On top of what a gtk.Notebook provides:
     - You can have icons on the notebook tab.
     - You can choose the label orientation (angle).
@@ -629,6 +726,7 @@ class IconNotebook:
 
         # We store the real gtk.Notebook object
         self.Notebook = notebookraw
+        self.Notebook.set_show_border(True)
 
         self.tabclosers = tabclosers
         self.reorderable = reorderable
@@ -653,7 +751,7 @@ class IconNotebook:
             page, label_tab, status, label_tab_menu = data
             try:
                 self.Notebook.set_tab_reorderable(page, self.reorderable)
-            except:
+            except Exception:
                 pass
 
     def set_tab_closers(self, closers):
@@ -688,21 +786,21 @@ class IconNotebook:
 
     def OnKeyPress(self, widget, event):
 
-        if event.state & (gtk.gdk.MOD1_MASK | gtk.gdk.CONTROL_MASK) != gtk.gdk.MOD1_MASK:
+        if event.state & (Gdk.ModifierType.MOD1_MASK | Gdk.ModifierType.CONTROL_MASK) != Gdk.ModifierType.MOD1_MASK:
             return False
 
-        if event.keyval in [gtk.gdk.keyval_from_name("Up"), gtk.gdk.keyval_from_name("Left")]:
+        if event.keyval in [Gdk.keyval_from_name("Up"), Gdk.keyval_from_name("Left")]:
             self.prev_page()
-        elif event.keyval in [gtk.gdk.keyval_from_name("Down"), gtk.gdk.keyval_from_name("Right")]:
+        elif event.keyval in [Gdk.keyval_from_name("Down"), Gdk.keyval_from_name("Right")]:
             self.next_page()
         else:
             return False
 
-        widget.emit_stop_by_name("key_press_event")
+        widget.stop_emission_by_name("key_press_event")
 
         return True
 
-    def append_page(self, page, label, onclose=None, angle=0):
+    def append_page(self, page, label, onclose=None, angle=0, fulltext=None):
 
         self.set_tab_angle(angle)
         closebutton = self.tabclosers
@@ -713,6 +811,11 @@ class IconNotebook:
             show_status_image=self._show_status_image
         )
 
+        if fulltext is None:
+            fulltext = label
+
+        label_tab.set_tooltip_text(fulltext)
+
         # menu for all tabs
         label_tab_menu = ImageLabel(label, self.images["empty"])
 
@@ -725,12 +828,13 @@ class IconNotebook:
 
         eventbox.add(label_tab)
         eventbox.show()
-        eventbox.set_events(gtk.gdk.BUTTON_PRESS_MASK)
+        eventbox.set_events(Gdk.EventMask.BUTTON_PRESS_MASK)
         eventbox.connect('button_press_event', self.on_tab_click, page)
 
         gtk.Notebook.append_page_menu(self.Notebook, page, eventbox, label_tab_menu)
 
         self.Notebook.set_tab_reorderable(page, self.reorderable)
+        self.Notebook.set_show_tabs(True)
 
     def remove_page(self, page):
 
@@ -741,7 +845,10 @@ class IconNotebook:
                 i[3].destroy()
                 self.pages.remove(i)
 
-                return
+                break
+
+        if self.Notebook.get_n_pages() == 0:
+            self.Notebook.set_show_tabs(False)
 
     def OnTabWindowDestroy(self, widget, page):
 
@@ -796,7 +903,7 @@ class IconNotebook:
 
         eventbox.add(label_tab)
         eventbox.show()
-        eventbox.set_events(gtk.gdk.BUTTON_PRESS_MASK)
+        eventbox.set_events(Gdk.EventType.BUTTON_PRESS_MASK)
         eventbox.connect('button_press_event', self.on_tab_click, page)
 
         gtk.Notebook.append_page_menu(self.Notebook, pagewidget, eventbox, label_tab_menu)
@@ -836,7 +943,7 @@ class IconNotebook:
 
         vbox = gtk.VBox(False, spacing=5)
         vbox.set_border_width(5)
-        vbox.pack_start(page)
+        vbox.pack_start(page, True, True, 0)
         vbox.show()
 
         window.add(vbox)
@@ -990,7 +1097,7 @@ class IconNotebook:
 
 class PopupMenu(gtk.Menu):
 
-    def __init__(self, frame=None):
+    def __init__(self, frame=None, shouldattach=True):
 
         gtk.Menu.__init__(self)
 
@@ -1000,16 +1107,21 @@ class PopupMenu(gtk.Menu):
         self.handlers = {}
         self.editing = False
 
+        # If the menu is not a submenu, it needs to be attached
+        # to the main window, otherwise it has no parent
+        if shouldattach and hasattr(self.frame, 'MainWindow'):
+            self.attach_to_widget(self.frame.MainWindow, None)
+
     def setup(self, *items):
 
         for item in items:
 
             if item[0] == "":
-                menuitem = gtk.MenuItem()
+                menuitem = gtk.SeparatorMenuItem()
 
             elif item[0] == "USER":
 
-                menuitem = gtk.MenuItem(item[1])
+                menuitem = gtk.MenuItem.new_with_label(item[1])
                 self.useritem = menuitem
 
                 if len(item) >= 3:
@@ -1019,7 +1131,7 @@ class PopupMenu(gtk.Menu):
 
             elif item[0] == 1:
 
-                menuitem = gtk.MenuItem(item[1])
+                menuitem = gtk.MenuItem.new_with_label(item[1])
                 menuitem.set_submenu(item[2])
 
                 if len(item) == 5 and item[4] is not None and item[3] is not None:
@@ -1029,7 +1141,7 @@ class PopupMenu(gtk.Menu):
 
             elif item[0] == "USERMENU":
 
-                menuitem = gtk.MenuItem(item[1])
+                menuitem = gtk.MenuItem.new_with_label(item[1])
                 menuitem.set_submenu(item[2])
 
                 if item[3] is not None:
@@ -1040,11 +1152,11 @@ class PopupMenu(gtk.Menu):
             else:
 
                 if item[0][0] == "$":
-                    menuitem = gtk.CheckMenuItem(item[0][1:])
+                    menuitem = gtk.CheckMenuItem.new_with_label(item[0][1:])
                 elif item[0][0] == "#":
-                    menuitem = gtk.MenuItem(item[0][1:])
+                    menuitem = gtk.MenuItem.new_with_label(item[0][1:])
                 else:
-                    menuitem = gtk.MenuItem(item[0])
+                    menuitem = gtk.MenuItem.new_with_label(item[0])
 
                 if len(item) >= 3 and item[2] is not None and item[1] is not None:
                     self.handlers[menuitem] = menuitem.connect("activate", item[1], item[2])
@@ -1053,13 +1165,16 @@ class PopupMenu(gtk.Menu):
 
             self.append(menuitem)
 
+            if item[0] != "":
+                menuitem.set_use_underline(True)
+
             menuitem.show()
 
         return self
 
     def clear(self):
 
-        for (w, widget) in self.handlers.iteritems():
+        for (w, widget) in self.handlers.items():
             w.disconnect(widget)
 
         self.handlers.clear()
@@ -1082,7 +1197,7 @@ class PopupMenu(gtk.Menu):
 
     def OnSearchUser(self, widget):
         self.frame.SearchMethod.set_active_iter(self.frame.searchmethods[_("User")])
-        self.frame.UserSearchCombo.child.set_text(self.user)
+        self.frame.UserSearchCombo.get_child().set_text(self.user)
         self.frame.ChangeMainPage(None, "search")
 
     def OnSendMessage(self, widget):
@@ -1168,7 +1283,7 @@ class PopupMenu(gtk.Menu):
         self.frame.privatechats.SendMessage(self.user, "\x01VERSION\x01", bytestring=True)
 
     def OnCopyUser(self, widget):
-        self.frame.clip.set_text(self.user)
+        self.frame.clip.set_text(self.user, -1)
 
     def OnGivePrivileges(self, widget):
 
@@ -1189,15 +1304,15 @@ class PopupMenu(gtk.Menu):
             try:
                 days = int(text)
                 self.frame.GivePrivileges(self.user, days)
-            except Exception, e:
-                print e
+            except Exception as e:
+                print(e)
 
     def OnPrivateRooms(self, widget):
 
         if self.user is None or self.user == self.frame.np.config.sections["server"]["login"]:
             return False
 
-        user = self.user
+        user = self.user  # noqa: F841
         items = []
         popup = self.frame.userlist.Popup_Menu_PrivateRooms
         popup.clear()
@@ -1227,25 +1342,25 @@ class PopupMenu(gtk.Menu):
 
 def InputDialog(parent, title, message, default=""):
 
-    dlg = gtk.Dialog(title=title, parent=parent, buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, gtk.STOCK_OK, gtk.RESPONSE_OK))
-    dlg.set_default_response(gtk.RESPONSE_OK)
+    dlg = gtk.Dialog(title=title, parent=parent, buttons=(gtk.STOCK_CANCEL, gtk.ResponseType.CANCEL, gtk.STOCK_OK, gtk.ResponseType.OK))
+    dlg.set_default_response(gtk.ResponseType.OK)
 
     dlg.set_border_width(10)
     dlg.vbox.set_spacing(10)
 
-    l = gtk.Label(message)
+    l = gtk.Label(message)  # noqa: E741
     l.set_alignment(0, 0.5)
-    dlg.vbox.pack_start(l, False, False)
+    dlg.vbox.pack_start(l, False, False, 0)
 
     entry = gtk.Entry()
     entry.set_activates_default(True)
     entry.set_text(default)
-    dlg.vbox.pack_start(entry, True, True)
+    dlg.vbox.pack_start(entry, True, True, 0)
 
     dlg.vbox.show_all()
 
     result = None
-    if dlg.run() == gtk.RESPONSE_OK:
+    if dlg.run() == gtk.ResponseType.OK:
         result = entry.get_text()
 
     dlg.destroy()
@@ -1253,37 +1368,13 @@ def InputDialog(parent, title, message, default=""):
     return result
 
 
-def int_sort_func(model, iter1, iter2, column):
-    try:
-        val1 = int(model.get_value(iter1, column))
-    except:
-        val1 = 0
-    try:
-        val2 = int(model.get_value(iter2, column))
-    except:
-        val2 = 0
-    return cmp(val1, val2)
-
-
-def float_sort_func(model, iter1, iter2, column):
-    try:
-        val1 = float(model.get_value(iter1, column))
-    except:
-        val1 = 0.0
-    try:
-        val2 = float(model.get_value(iter2, column))
-    except:
-        val2 = 0.0
-    return cmp(val1, val2)
-
-
 def WriteLog(logsdir, fn, msg):
 
-    oldumask = os.umask(0077)
+    oldumask = os.umask(0o077)
     if not os.path.exists(logsdir):
         os.makedirs(logsdir)
 
-    logfile = open(os.path.join(logsdir, fixpath(fn.replace(os.sep, "-")) + ".log"), 'a', 0)
+    logfile = open(os.path.join(logsdir, fixpath(fn.replace(os.sep, "-")) + ".log"), 'ab', 0)
 
     os.umask(oldumask)
 
@@ -1301,40 +1392,32 @@ def fixpath(path):
             for char in chars:
                 path = path.replace(char, "_")
         return path
-    except:
+    except Exception:
         return path
 
 
-def HumanSize(number):
+def HumanSize(filesize):
     try:
-        s = float(int(number))
-        if s >= 1000*1024*1024:
-            r = _("%.2f GiB") % (s / (1024.0*1024.0*1024.0))
-        elif s >= 1000*1024:
-            r = _("%.2f MiB") % (s / (1024.0*1024.0))
-        elif s >= 1000:
-            r = _("%.2f KiB") % (s / 1024.0)
-        else:
-            r = _("%d Bytes") % s
-        return r
-    except Exception, e:
-        return number
+        if (filesize <= 0):
+            return "0 Bytes"
+
+        p = int(math.floor(math.log(filesize, 2) / 10))
+
+        return "%0.2f %s" % (filesize / math.pow(1024, p), ['Bytes', 'KB', 'MB', 'GB', 'TB', 'PB', 'EB', 'ZB', 'YB'][p])
+    except TypeError:
+        return filesize
 
 
-def HumanSpeed(number):
+def HumanSpeed(filesize):
     try:
-        s = float(int(number))
-        if s >= 1000*1024*1024:
-            r = _("%.2f GiB/s") % (s / (1024.0*1024.0*1024.0))
-        elif s >= 1000*1024:
-            r = _("%.2f MiB/s") % (s / (1024.0*1024.0))
-        elif s >= 1000:
-            r = _("%.2f KiB/s") % (s / 1024.0)
-        else:
-            r = _("%d B/s") % (number)
-        return r
-    except Exception, e:
-        return number
+        if (filesize <= 0):
+            return "0 B/s"
+
+        p = int(math.floor(math.log(filesize, 2) / 10))
+
+        return "%0.2f %s" % (filesize / math.pow(1024, p), ['B/s', 'KB/s', 'MB/s', 'GB/s', 'TB/s', 'PB/s', 'EB/s', 'ZB/s', 'YB/s'][p])
+    except TypeError:
+        return filesize
 
 
 def Humanize(number):
@@ -1369,7 +1452,7 @@ def is_alias(aliases, cmd):
 
 def expand_alias(aliases, cmd):
     output = _expand_alias(aliases, cmd)
-    return findBestEncoding(output, ['UTF-8', 'ASCII'])
+    return output
 
 
 def _expand_alias(aliases, cmd):
@@ -1399,8 +1482,8 @@ def _expand_alias(aliases, cmd):
         ret = ""
         i = 0
         while i < len(alias):
-            if alias[i:i+2] == "$(":
-                arg = getpart(alias[i+1:])
+            if alias[i:i + 2] == "$(":
+                arg = getpart(alias[i + 1:])
                 if not arg:
                     ret = ret + "$"
                     i = i + 1
@@ -1423,62 +1506,15 @@ def _expand_alias(aliases, cmd):
                         last = int(args[1])
                     else:
                         last = len(cmd)
-                v = string.join(cmd[first:last+1])
+                v = " ".join(cmd[first:last + 1])
                 if not v:
                     v = default
                 ret = ret + v
-            elif alias[i:i+2] == "|(":
-                arg = getpart(alias[i+1:])
-                if not arg:
-                    ret = ret + "|"
-                    i = i + 1
-                    continue
-                i = i + len(arg) + 3
-                for j in range(len(cmd)-1, -1, -1):
-                    arg = arg.replace("$%i" % j, cmd[j])
-                arg = arg.replace("$@", string.join(cmd[1:], " "))
-
-                import subprocess
-
-                p = subprocess.Popen(arg, shell=True, stdin=subprocess.PIPE, stdout=subprocess.PIPE, close_fds=(not sys.platform.startswith("win")))
-                exit = p.wait()
-
-                (stdout, stdin) = (p.stdout, p.stdin)
-                v = stdout.read().split("\n")
-                r = ""
-                for l in v:
-                    l = l.strip()
-                    if l:
-                        r = r + l + "\n"
-                ret = ret + r.strip()
-                stdin.close()
-                stdout.close()
             else:
                 ret = ret + alias[i]
                 i = i + 1
         return ret
-    except Exception, error:
-        print error
+    except Exception as error:
+        print(error)
         pass
     return ""
-
-
-def EncodingsMenu(np, section=None, entry=None):
-    if section and entry and entry in np.config.sections["server"][section]:
-        encoding = np.config.sections["server"][section][entry]
-    else:
-        encoding = np.config.sections["server"]["enc"]
-
-    z = []
-    for enc in np.getencodings():
-        z.append(enc)
-
-    return encoding, z
-
-
-def SaveEncoding(np, section, entry, encoding):
-    if encoding != np.config.sections["server"]["enc"]:
-        np.config.sections["server"][section][entry] = encoding
-    elif entry in np.config.sections["server"][section]:
-        del np.config.sections["server"][section][entry]
-    np.config.writeConfiguration()
