@@ -1,5 +1,4 @@
-# -*- coding: utf-8 -*-
-#
+# COPYRIGHT (C) 2020 Nicotine+ Team
 # COPYRIGHT (C) 2016-2017 Michael Labouebe <gfarmerfr@free.fr>
 # COPYRIGHT (C) 2016 Mutnick <muhing@yahoo.com>
 # COPYRIGHT (C) 2008-2011 Quinox <quinox@users.sf.net>
@@ -23,19 +22,26 @@
 
 import os
 import sys
-import gtk
-import gobject
-import zipimport
-import imp
-from thread import start_new_thread
-from traceback import extract_stack, extract_tb, format_list, print_exc
+from ast import literal_eval
+from gettext import gettext as _
+from importlib.machinery import SourceFileLoader
 from time import time
-from pynicotine import slskmessages
-from slskmessages import ToBeEncoded
-from logfacility import log
+from traceback import extract_stack
+from traceback import extract_tb
+from traceback import format_list
+from traceback import print_exc
+
+from gi.repository import GLib
+
+from _thread import start_new_thread
+from pynicotine.logfacility import log
+from pynicotine.pynicotine import slskmessages
+from pynicotine.utils import installPrefix
+
 
 WIN32 = sys.platform.startswith("win")
 
+
 returncode = {
     'break': 0,  # don't give other plugins the event, do let n+ process it
     'zap': 1,    # don't give other plugins the event, don't let n+ process it
@@ -46,7 +52,7 @@ tupletype = type(('', ''))
 
 
 def cast_to_unicode_if_needed(text, logfunc):
-    if isinstance(text, unicode):
+    if isinstance(text, str):
         return text
     try:
         better = str.decode(text, 'utf8')
@@ -56,17 +62,21 @@ def cast_to_unicode_if_needed(text, logfunc):
         better = str.decode(text, 'utf8', 'replace')
         logfunc("Plugin problem: casting '%s' to unicode, losing characters in the process." % repr(text))
         return better
-    except:
+    except Exception:
         logfunc("Plugin problem: failed to completely cast '%s', you're on your own from here on." % repr(text))
         return text
 
 
+class InvalidPluginError(Exception):
+    pass
+
+
 class PluginHandler(object):
 
     frame = None  # static variable... but should it be?
     guiqueue = []  # fifo isn't supported by older python
 
-    def __init__(self, frame, plugindir=None):
+    def __init__(self, frame, plugindir):
         self.frame = frame
         log.add(_("Loading plugin handler"))
         self.myUsername = self.frame.np.config.sections["server"]["login"]
@@ -81,28 +91,23 @@ class PluginHandler(object):
             'str': str
         }
 
-        if not plugindir:
-            if WIN32:
-                try:
-                    mydir = os.path.join(os.environ['APPDATA'], 'nicotine')
-                except KeyError:
-                    # windows 9x?
-                    mydir, x = os.path.split(sys.argv[0])
-                self.plugindir = os.path.join(mydir, "plugins")
-            else:
-                self.plugindir = os.path.join(os.path.expanduser("~"), '.nicotine', 'plugins')
-        else:
-            self.plugindir = plugindir
         try:
-            os.makedirs(self.plugindir)
-        except:
+            os.makedirs(plugindir)
+        except Exception:
             pass
-        self.plugindirs.append(self.plugindir)
-        if os.path.isdir(self.plugindir):
+
+        # Load system-wide plugins
+        prefix = installPrefix()
+        self.plugindirs.append(os.path.join(prefix, "share/nicotine/plugins"))
+
+        # Load home directory plugins
+        self.plugindirs.append(plugindir)
+
+        if os.path.isdir(plugindir):
             # self.load_directory(self.plugindir)
             self.load_enabled()
         else:
-            log.add(_("It appears '%s' is not a directory, not loading plugins.") % self.plugindir)
+            log.add(_("It appears '%s' is not a directory, not loading plugins.") % plugindir)
 
     def __findplugin(self, pluginname):
         for directory in self.plugindirs:
@@ -124,7 +129,7 @@ class PluginHandler(object):
             log.add(_("Failed to load plugin '%s', could not find it.") % pluginname)
             return False
         sys.path.insert(0, path)
-        plugin = imp.load_source(pluginname, os.path.join(path, '__init__.py'))
+        plugin = SourceFileLoader(pluginname, os.path.join(path, '__init__.py')).load_module()
         instance = plugin.Plugin(self)
         self.plugin_settings(instance)
         instance.LoadNotification()
@@ -134,35 +139,6 @@ class PluginHandler(object):
         self.loaded_plugins[pluginname] = plugin
         return plugin
 
-    def install_plugin(self, path):
-        try:
-            tar = tarfile.open(path, "r:*")  # transparently supports gz, bz2
-        except (tarfile.ReadError, OSError):
-            raise InvalidPluginError(_('Plugin archive is not in the correct format'))
-
-        # ensure the paths in the archive are sane
-        mems = tar.getmembers()
-        base = os.path.basename(path)[:-4]
-        if os.path.isdir(os.path.join(self.plugindirs[0], base)):
-            raise InvalidPluginError(_('A plugin with the name "%s" is '
-                                       'already installed') % base)
-
-        for m in mems:
-            if not m.name.startswith(base):
-                raise InvalidPluginError(_("Plugin archive contains an unsafe path"))
-
-        tar.extractall(self.plugindirs[0])
-
-    def uninstall_plugin(self, pluginname):
-        self.disable_plugin(pluginname)
-        for dir in self.plugindirs:
-            try:
-                shutil.rmtree(self.__findplugin(pluginname))
-                return True
-            except:
-                pass
-        return False
-
     def enable_plugin(self, pluginname):
         if pluginname in self.enabled_plugins:
             return
@@ -173,7 +149,7 @@ class PluginHandler(object):
             plugin.enable(self)
             self.enabled_plugins[pluginname] = plugin
             log.add(_("Enabled plugin %s") % plugin.PLUGIN.__name__)
-        except:
+        except Exception:
             print_exc()
             log.addwarning(_("Unable to enable plugin %s") % pluginname)
             # common.log_exception(logger)
@@ -197,7 +173,7 @@ class PluginHandler(object):
             log.add(_("Disabled plugin {}".format(plugin.PLUGIN.__name__)))
             del self.enabled_plugins[pluginname]
             plugin.disable(self)
-        except:
+        except Exception:
             print_exc()
             log.addwarning(_("Unable to fully disable plugin %s") % pluginname)
             # common.log_exception(logger)
@@ -212,27 +188,19 @@ class PluginHandler(object):
 
     def get_plugin_info(self, pluginname):
         path = os.path.join(self.__findplugin(pluginname), 'PLUGININFO')
-        f = open(path)
-        infodict = {}
-        for line in f:
-            try:
-                key, val = line.split("=", 1)
-                infodict[key] = eval(val)
-            except ValueError:
-                pass  # this happens on blank lines
+        with open(path) as f:
+            infodict = {}
+            for line in f:
+                try:
+                    key, val = line.split("=", 1)
+                    infodict[key] = literal_eval(val)
+                except ValueError:
+                    pass  # this happens on blank lines
         return infodict
 
     def save_enabled(self):
         self.frame.np.config.sections["plugins"]["enabled"] = list(self.enabled_plugins.keys())
 
-    def check_enabled(self):
-        if self.frame.np.config.sections["plugins"]["enable"]:
-            self.load_enabled()
-        else:
-            to_enable = self.frame.np.config.sections["plugins"]["enabled"]
-            for plugin in self.enabled_plugins:
-                self.enabled_plugins[plugin].disable(self)
-
     def load_enabled(self):
         enable = self.frame.np.config.sections["plugins"]["enable"]
         if not enable:
@@ -282,14 +250,14 @@ class PluginHandler(object):
                         pass
                     else:
                         log.add(_("Plugin %(module)s returned something weird, '%(value)s', ignoring") % {'module': module, 'value': str(ret)})
-            except:
+            except Exception:
                 log.add(_("Plugin %(module)s failed with error %(errortype)s: %(error)s.\nTrace: %(trace)s\nProblem area:%(area)s") % {
-                            'module': module,
-                            'errortype': sys.exc_info()[0],
-                            'error': sys.exc_info()[1],
-                            'trace': ''.join(format_list(extract_stack())),
-                            'area': ''.join(format_list(extract_tb(sys.exc_info()[2])))
-                        })
+                    'module': module,
+                    'errortype': sys.exc_info()[0],
+                    'error': sys.exc_info()[1],
+                    'trace': ''.join(format_list(extract_stack())),
+                    'area': ''.join(format_list(extract_tb(sys.exc_info()[2])))
+                })
         return False
 
     def TriggerEvent(self, function, args):
@@ -312,14 +280,14 @@ class PluginHandler(object):
                         log.add(_("Plugin %(module)s returned something weird, '%(value)s', ignoring") % {'module': module, 'value': ret})
                 if ret is not None:
                     hotpotato = ret
-            except:
+            except Exception:
                 log.add(_("Plugin %(module)s failed with error %(errortype)s: %(error)s.\nTrace: %(trace)s\nProblem area:%(area)s") % {
-                            'module': module,
-                            'errortype': sys.exc_info()[0],
-                            'error': sys.exc_info()[1],
-                            'trace': ''.join(format_list(extract_stack())),
-                            'area': ''.join(format_list(extract_tb(sys.exc_info()[2])))
-                        })
+                    'module': module,
+                    'errortype': sys.exc_info()[0],
+                    'error': sys.exc_info()[1],
+                    'trace': ''.join(format_list(extract_stack())),
+                    'area': ''.join(format_list(extract_tb(sys.exc_info()[2])))
+                })
         return hotpotato
 
     def SearchRequestNotification(self, searchterm, user, searchid):
@@ -410,14 +378,14 @@ class PluginHandler(object):
         if len(self.guiqueue) >= 0:
             addidle = True
         if addidle:
-            gobject.idle_add(self.processQueue)
+            GLib.idle_add(self.processQueue)
 
     def log(self, text):
         self.appendqueue({'type': 'logtext', 'text': text})
 
     def saychatroom(self, room, text):
         text = cast_to_unicode_if_needed(text, log.addwarning)
-        self.frame.np.queue.put(slskmessages.SayChatroom(room, ToBeEncoded(text, 'UTF-8')))
+        self.frame.np.queue.put(slskmessages.SayChatroom(room, text))
 
     def sayprivate(self, user, text):
         '''Send user message in private (showing up in GUI)'''
@@ -466,9 +434,9 @@ class BasePlugin(object):
             self.__id__ = self.__name__.lower().replace(' ', '_').replace('%', '_').replace('=', '_')
         self.init()
         for (trigger, func) in self.__publiccommands__:
-            self.frame.chatrooms.roomsctrl.CMDS.add('/'+trigger+' ')
+            self.frame.chatrooms.roomsctrl.CMDS.add('/' + trigger + ' ')
         for (trigger, func) in self.__privatecommands__:
-            self.frame.privatechats.CMDS.add('/'+trigger+' ')
+            self.frame.privatechats.CMDS.add('/' + trigger + ' ')
 
     def init(self):
         pass
@@ -567,7 +535,7 @@ class BasePlugin(object):
         except KeyError:
             return False
         text = cast_to_unicode_if_needed(text, self.log)
-        msg = slskmessages.SayChatroom(room, ToBeEncoded(text, 'UTF-8'))
+        msg = slskmessages.SayChatroom(room, text)
         msg.user = user
         room.SayChatRoom(msg, text)
         return True
@@ -609,7 +577,7 @@ class ResponseThrottle(object):
         self.room = room
         self.nick = nick
         self.request = request
-        
+
         willing_to_respond = True
         current_time = time()
 
@@ -623,7 +591,7 @@ class ResponseThrottle(object):
         port = False
         try:
             ip, port = self.frame.np.users[nick].addr
-        except:
+        except Exception:
             port = True
 
         if nick in self.frame.np.config.sections["server"]["ignorelist"]:
