@@ -1,5 +1,5 @@
-# -*- coding: utf-8 -*-
-#
+# COPYRIGHT (C) 2020 Nicotine+ Team
+# COPYRIGHT (C) 2020 Mathias <mail@mathias.is>
 # COPYRIGHT (C) 2016-2017 Michael Labouebe <gfarmerfr@free.fr>
 # COPYRIGHT (C) 2016 Mutnick <muhing@yahoo.com>
 # COPYRIGHT (C) 2013 eL_vErDe <gandalf@le-vert.net>
@@ -29,29 +29,28 @@
 This is the actual client code. Actual GUI classes are in the separate modules
 """
 
-from __future__ import division
-
-import time
+import configparser
 import datetime
-import shutil
-from urllib import urlencode
-import slskproto
-import slskmessages
-from slskmessages import newId, PopupMessage, ToBeEncoded
-import transfers
-import Queue
-import threading
-from config import *
-import string
-import types
-import locale
-import utils
-from shares import Shares
-from utils import CleanFile, findBestEncoding
-import os
 import logging
+import os
+import queue
+import shutil
+import sys
+import threading
+import time
+from gettext import gettext as _
 
-from ConfigParser import Error as ConfigParserError
+from pynicotine import slskmessages
+from pynicotine import slskproto
+from pynicotine import transfers
+from pynicotine.config import Config
+from pynicotine.geoip import IP2Location
+from pynicotine.shares import Shares
+from pynicotine.slskmessages import PopupMessage
+from pynicotine.slskmessages import newId
+from pynicotine.utils import CleanFile
+from pynicotine.utils import log
+from pynicotine.utils import unescape
 
 
 class PeerConnection:
@@ -82,8 +81,8 @@ class Timeout:
     def timeout(self):
         try:
             self.callback([self])
-        except Exception, e:
-            print("Exception in callback %s: %s" % (self.callback, e))
+        except Exception as e:
+            print(("Exception in callback %s: %s" % (self.callback, e)))
 
 
 class ConnectToPeerTimeout(Timeout):
@@ -101,7 +100,7 @@ class NetworkEventProcessor:
     """ This class contains handlers for various messages from the networking
     thread"""
 
-    def __init__(self, frame, callback, writelog, setstatus, bindip, configfile):
+    def __init__(self, frame, callback, writelog, setstatus, bindip, port, data_dir, config):
 
         self.frame = frame
         self.callback = callback
@@ -109,17 +108,18 @@ class NetworkEventProcessor:
         self.setStatus = setstatus
 
         try:
-            self.config = Config(configfile)
-        except ConfigParserError:
-            corruptfile = ".".join([configfile, CleanFile(datetime.datetime.now().strftime("%Y-%M-%d_%H:%M:%S")), "corrupt"])
-            shutil.move(configfile, corruptfile)
+            self.config = Config(config, data_dir)
+        except configparser.Error:
+            corruptfile = ".".join([config, CleanFile(datetime.datetime.now().strftime("%Y-%M-%d_%H:%M:%S")), "corrupt"])
+            shutil.move(config, corruptfile)
             short = _("Your config file is corrupt")
             long = _("We're sorry, but it seems your configuration file is corrupt. Please reconfigure Nicotine+.\n\nWe renamed your old configuration file to\n%(corrupt)s\nIf you open this file with a text editor you might be able to rescue some of your settings.") % {'corrupt': corruptfile}
-            log.addwarning(long)
-            self.config = Config(configfile)
+            log.addwarning(int)
+            self.config = Config(config, data_dir)
             self.callback([PopupMessage(short, long)])
 
         self.bindip = bindip
+        self.port = port
         self.config.frame = frame
         self.config.readConfig()
         self.peerconns = []
@@ -129,20 +129,14 @@ class NetworkEventProcessor:
         self.ip_requested = []
         self.PrivateMessageQueue = {}
         self.users = {}
-        self.queue = Queue.Queue(0)
+        self.queue = queue.Queue(0)
         self.shares = Shares(self)
 
-        try:
-            import GeoIP
-            self.geoip = GeoIP.new(GeoIP.GEOIP_MEMORY_CACHE)
-        except ImportError:
-            try:
-                import _GeoIP
-                self.geoip = _GeoIP.new(_GeoIP.GEOIP_MEMORY_CACHE)
-            except ImportError:
-                self.geoip = None
+        script_dir = os.path.dirname(__file__)
+        file_path = os.path.join(script_dir, "geoip/ipcountrydb.bin")
+        self.geoip = IP2Location.IP2Location(file_path, "SHARED_MEMORY")
 
-        self.protothread = slskproto.SlskProtoThread(self.frame.networkcallback, self.queue, self.bindip, self.config, self)
+        self.protothread = slskproto.SlskProtoThread(self.frame.networkcallback, self.queue, self.bindip, self.port, self.config, self)
 
         uselimit = self.config.sections["transfers"]["uselimit"]
         uploadlimit = self.config.sections["transfers"]["uploadlimit"]
@@ -185,6 +179,7 @@ class NetworkEventProcessor:
         self.respondDistributed = True
         responddistributedtimeout = RespondToDistributedSearchesTimeout(self.callback)
         self.respondDistributedTimer = threading.Timer(60, responddistributedtimeout.timeout)
+        self.respondDistributedTimer.setDaemon(True)
         self.respondDistributedTimer.start()
 
         # Callback handlers for messages
@@ -198,12 +193,12 @@ class NetworkEventProcessor:
             slskmessages.ChangePassword: self.ChangePassword,
             slskmessages.MessageUser: self.MessageUser,
             slskmessages.PMessageUser: self.PMessageUser,
-            slskmessages.ExactFileSearch: self.ExactFileSearch,
+            slskmessages.ExactFileSearch: self.DummyMessage,
             slskmessages.UserJoinedRoom: self.UserJoinedRoom,
             slskmessages.SayChatroom: self.SayChatRoom,
             slskmessages.JoinRoom: self.JoinRoom,
             slskmessages.UserLeftRoom: self.UserLeftRoom,
-            slskmessages.QueuedDownloads: self.QueuedDownloads,
+            slskmessages.QueuedDownloads: self.DummyMessage,
             slskmessages.GetPeerAddress: self.GetPeerAddress,
             slskmessages.OutConn: self.OutConn,
             slskmessages.UserInfoReply: self.UserInfoReply,
@@ -242,12 +237,9 @@ class NetworkEventProcessor:
             slskmessages.ServerPing: self.DummyMessage,
             slskmessages.ParentMinSpeed: self.DummyMessage,
             slskmessages.ParentSpeedRatio: self.DummyMessage,
-            slskmessages.Msg85: self.DummyMessage,
-            slskmessages.Msg12547: self.Msg12547,
-            slskmessages.ParentInactivityTimeout: self.ParentInactivityTimeout,
-            slskmessages.SearchInactivityTimeout: self.SearchInactivityTimeout,
-            slskmessages.MinParentsInCache: self.MinParentsInCache,
-            slskmessages.Msg89: self.DummyMessage,
+            slskmessages.ParentInactivityTimeout: self.DummyMessage,
+            slskmessages.SearchInactivityTimeout: self.DummyMessage,
+            slskmessages.MinParentsInCache: self.DummyMessage,
             slskmessages.WishlistInterval: self.WishlistInterval,
             slskmessages.DistribAliveInterval: self.DummyMessage,
             slskmessages.ChildDepth: self.ChildDepth,
@@ -256,11 +248,10 @@ class NetworkEventProcessor:
             slskmessages.DistribChildDepth: self.DistribChildDepth,
             slskmessages.DistribBranchLevel: self.DistribBranchLevel,
             slskmessages.DistribBranchRoot: self.DistribBranchRoot,
-            slskmessages.DistribMessage9: self.DistribMessage9,
             slskmessages.AdminMessage: self.AdminMessage,
             slskmessages.TunneledMessage: self.TunneledMessage,
             slskmessages.IncConn: self.IncConn,
-            slskmessages.PlaceholdUpload: self.PlaceholdUpload,
+            slskmessages.PlaceholdUpload: self.DummyMessage,
             slskmessages.PlaceInQueueRequest: self.PlaceInQueueRequest,
             slskmessages.UploadQueueNotification: self.UploadQueueNotification,
             slskmessages.SearchRequest: self.SearchRequest,
@@ -268,8 +259,9 @@ class NetworkEventProcessor:
             slskmessages.RoomSearch: self.RoomSearchRequest,
             slskmessages.UserSearch: self.SearchRequest,
             slskmessages.NetInfo: self.NetInfo,
-            slskmessages.DistribAlive: self.DistribAlive,
+            slskmessages.DistribAlive: self.DummyMessage,
             slskmessages.DistribSearch: self.DistribSearch,
+            slskmessages.DistribServerSearch: self.DistribSearch,
             ConnectToPeerTimeout: self.ConnectToPeerTimeout,
             RespondToDistributedSearchesTimeout: self.ToggleRespondDistributed,
             transfers.TransferTimeout: self.TransferTimeout,
@@ -277,7 +269,7 @@ class NetworkEventProcessor:
             slskmessages.RescanBuddyShares: self.shares.RescanBuddyShares,
             str: self.Notify,
             slskmessages.PopupMessage: self.PopupMessage,
-            slskmessages.InternalData: self.DisplaySockets,
+            slskmessages.SetCurrentConnectionCount: self.SetCurrentConnectionCount,
             slskmessages.DebugMessage: self.DebugMessage,
             slskmessages.GlobalRecommendations: self.GlobalRecommendations,
             slskmessages.Recommendations: self.Recommendations,
@@ -305,6 +297,7 @@ class NetworkEventProcessor:
             slskmessages.PrivateRoomAddOperator: self.PrivateRoomAddOperator,
             slskmessages.PrivateRoomRemoveOperator: self.PrivateRoomRemoveOperator,
             slskmessages.PublicRoomMessage: self.PublicRoomMessage,
+            slskmessages.UnknownPeerMessage: self.DummyMessage,
         }
 
     def ProcessRequestToPeer(self, user, message, window=None, address=None):
@@ -359,6 +352,7 @@ class NetworkEventProcessor:
             if firewalled:
                 if addr is None:
                     self.queue.put(slskmessages.GetPeerAddress(user))
+                    addr = "in_progress"
                 elif behindfw is None:
                     self.queue.put(slskmessages.OutConn(None, addr))
                 else:
@@ -375,6 +369,7 @@ class NetworkEventProcessor:
                 timeout = 120.0
                 conntimeout = ConnectToPeerTimeout(self.peerconns[-1], self.callback)
                 timer = threading.Timer(timeout, conntimeout.timeout)
+                timer.setDaemon(True)
                 self.peerconns[-1].conntimer = timer
                 timer.start()
 
@@ -395,6 +390,7 @@ class NetworkEventProcessor:
             self.servertimeout = self.servertimeout * 2
 
         self.servertimer = threading.Timer(self.servertimeout, self.ServerTimeout)
+        self.servertimer.setDaemon(True)
         self.servertimer.start()
         logging.info(_("The server seems to be down or not responding, retrying in %i seconds") % (self.servertimeout))
 
@@ -420,123 +416,23 @@ class NetworkEventProcessor:
     def ConnectToServer(self, msg):
         self.frame.OnConnect(None)
 
-    def encodeuser(self, string, user=None):
-
-        coding = None
-        config = self.config.sections
-
-        if user and user in config["server"]["userencoding"]:
-            coding = config["server"]["userencoding"][user]
-
-        string = self.decode(string, coding)
-        try:
-            return string.encode(locale.nl_langinfo(locale.CODESET))
-        except:
-            return string
-
-    def encode(self, str, networkenc=None):
-
-        if networkenc is None:
-            networkenc = self.config.sections["server"]["enc"]
-
-        if type(str) is types.UnicodeType:
-            return str.encode(networkenc, 'replace')
-        else:
-            return str.decode("utf-8", 'replace').encode(networkenc, 'replace')
-
-    def decode(self, string, networkenc=None):
-
-        if networkenc is None:
-            networkenc = self.config.sections["server"]["enc"]
-
-        return str(string).decode(networkenc, 'replace').encode("utf-8", "replace")
-
-    def getencodings(self):
-
-        # Encodings and descriptions for ComboBoxes
-        return [
-            ["Latin", 'ascii'],
-            ["US-Canada", 'cp037'],
-            ['Hebrew', 'cp424'],
-            ['US English', 'cp437'],
-            ['International', 'cp500'],
-            ['Greek', 'cp737'],
-            ['Estonian', 'cp775'],
-            ['Western European', 'cp850'],
-            ['Central European', 'cp852'],
-            ['Cyrillic', 'cp855'],
-            ['Cyrillic', 'cp856'],
-            ['Turkish', 'cp857'],
-            ['Portuguese', 'cp860'],
-            ['Icelandic', 'cp861'],
-            ['Hebrew', 'cp862'],
-            ['French Canadian', 'cp863'],
-            ['Arabic', 'cp864'],
-            ['Nordic', 'cp865'],
-            ['Cyrillic', 'cp866'],
-            ['Latin-9', 'cp869'],
-            ['Thai', 'cp874'],
-            ['Greek', 'cp875'],
-            ['Japanese', 'cp932'],
-            ['Chinese Simple', 'cp936'],
-            ['Korean', 'cp949'],
-            ['Chinese Traditional', 'cp950'],
-            ['Urdu',  'cp1006'],
-            ['Turkish',  'cp1026'],
-            ['Latin', 'cp1140'],
-            ['Central European', 'cp1250'],
-            ['Cyrillic', 'cp1251'],
-            ['Latin', 'cp1252'],
-            ['Greek', 'cp1253'],
-            ['Turkish', 'cp1254'],
-            ['Hebrew', 'cp1255'],
-            ['Arabic', 'cp1256'],
-            ['Baltic', 'cp1257'],
-            ['Vietnamese', 'cp1258'],
-            ['Latin', 'iso8859-1'],
-            ['Latin 2', 'iso8859-2'],
-            ['South European', 'iso8859-3'],
-            ['North European', 'iso8859-4'],
-            ['Cyrillic', 'iso8859-5'],
-            ['Arabic', 'iso8859-6'],
-            ['Greek', 'iso8859-7'],
-            ['Hebrew', 'iso8859-8'],
-            ['Turkish', 'iso8859-9'],
-            ['Nordic', 'iso8859-10'],
-            ['Thai', 'iso8859-11'],
-            ['Baltic', 'iso8859-13'],
-            ['Celtic', 'iso8859-14'],
-            ['Western European', 'iso8859-15'],
-            ['South-Eastern European', 'iso8859-16'],
-            ['Cyrillic', 'koi8-r'],
-            ['Latin', 'latin-1'],
-            ['Japanese', 'shift_jis'],
-            ['Korean', 'euc_kr'],
-            ['Cyrillic', 'mac-cyrillic'],
-            ['Greek', 'mac-greek'],
-            ['Icelandic', 'mac-iceland'],
-            ['Latin 2', 'mac-latin2'],
-            ['Latin', 'mac-roman'],
-            ['Turkish', 'mac-turkish'],
-            ['International', 'utf-16'],
-            ['International', 'utf-7'],
-            ['International', 'utf-8']
-        ]
-
     # Notify user of error when recieving or sending a message
     # @param self NetworkEventProcessor (Class)
     # @param string a string containing an error message
     def Notify(self, string):
-        self.logMessage("%s" % self.decode(string))
+        self.logMessage("%s" % string, 4)
 
     def PopupMessage(self, msg):
         self.setStatus(_(msg.title))
         self.frame.PopupMessage(msg)
 
+    def DummyMessage(self, msg):
+        self.logMessage("%s %s" % (msg.__class__, vars(msg)), 4)
+
     def DebugMessage(self, msg):
         self.logMessage(msg.msg, msg.debugLevel)
 
-    def DisplaySockets(self, msg):
+    def SetCurrentConnectionCount(self, msg):
         self.frame.SetSocketStatus(msg.msg)
 
     def ConnectError(self, msg):
@@ -547,7 +443,7 @@ class NetworkEventProcessor:
                 _("Can't connect to server %(host)s:%(port)s: %(error)s") % {
                     'host': msg.connobj.addr[0],
                     'port': msg.connobj.addr[1],
-                    'error': self.decode(msg.err)
+                    'error': msg.err
                 }
             )
 
@@ -578,6 +474,7 @@ class NetworkEventProcessor:
 
                         conntimeout = ConnectToPeerTimeout(i, self.callback)
                         timer = threading.Timer(120.0, conntimeout.timeout)
+                        timer.setDaemon(True)
                         timer.start()
 
                         if i.conntimer is not None:
@@ -631,10 +528,20 @@ class NetworkEventProcessor:
             slskmessages.Login(
                 self.config.sections["server"]["login"],
                 self.config.sections["server"]["passw"],
-                157  # 155, 156, 157, 180
+
+                # Afaik, the client version was set to 157 ns at some point in the past
+                # to support distributed searches properly. Probably no reason to mess
+                # with this (yet)
+
+                # Soulseek client version; 155, 156, 157, 180, 181, 183
+                157,
+
+                # Soulseek client minor version
+                # 17 stands for 157 ns 13c, 19 for 157 ns 13e
+                # For client versions newer than 157, the minor version is probably 1
+                19,
             )
         )
-
         if self.waitport is not None:
             self.queue.put(slskmessages.SetWaitPort(self.waitport))
 
@@ -686,7 +593,7 @@ class NetworkEventProcessor:
         else:
             for i in self.peerconns[:]:
                 if i.conn == conn:
-                    self.logMessage(_("Connection closed by peer: %s") % self.decode(vars(i)), 3)
+                    self.logMessage(_("Connection closed by peer: %s") % vars(i), debugLevel=3)
 
                     if i.conntimer is not None:
                         i.conntimer.cancel()
@@ -726,12 +633,12 @@ class NetworkEventProcessor:
 
             self.transfers.setTransferPanels(downloads, uploads)
             self.shares.sendNumSharedFoldersFiles()
-            self.queue.put(slskmessages.SetStatus((not self.frame.away)+1))
+            self.queue.put(slskmessages.SetStatus((not self.frame.away) + 1))
 
             for thing in self.config.sections["interests"]["likes"]:
-                self.queue.put(slskmessages.AddThingILike(self.encode(thing)))
+                self.queue.put(slskmessages.AddThingILike(thing))
             for thing in self.config.sections["interests"]["dislikes"]:
-                self.queue.put(slskmessages.AddThingIHate(self.encode(thing)))
+                self.queue.put(slskmessages.AddThingIHate(thing))
 
             if not len(self.distribcache):
                 self.queue.put(slskmessages.HaveNoParent(1))
@@ -744,8 +651,10 @@ class NetworkEventProcessor:
             self.frame.manualdisconnect = 1
             self.setStatus(_("Can not log in, reason: %s") % (msg.reason))
             self.logMessage(_("Can not log in, reason: %s") % (msg.reason))
-            self.frame.settingswindow.SetSettings(self.config.sections)
-            self.frame.settingswindow.SwitchToPage("Server")
+
+            if self.frame.settingswindow is not None:
+                self.frame.settingswindow.SetSettings(self.config.sections)
+                self.frame.settingswindow.SwitchToPage("Server")
 
     def ChangePassword(self, msg):
         password = msg.password
@@ -802,13 +711,6 @@ class NetworkEventProcessor:
 
     def MessageUser(self, msg):
 
-        if msg.user in self.config.sections["server"]["userencoding"]:
-            encodings = [self.config.sections["server"]["userencoding"][msg.user]] + self.config.sections["server"]["fallbackencodings"]
-            encodings.append(self.config.sections["server"]["enc"])
-        else:
-            encodings = [self.config.sections["server"]["enc"]] + self.config.sections["server"]["fallbackencodings"]
-
-        msg.msg = findBestEncoding(msg.msg, encodings)
         status = 0
         if self.logintime:
             if time.time() <= self.logintime + 2:
@@ -837,8 +739,6 @@ class NetworkEventProcessor:
 
     def PublicRoomMessage(self, msg):
 
-        msg.msg = findBestEncoding(msg.msg, [self.config.sections["server"]["enc"]] + self.config.sections["server"]["fallbackencodings"])
-
         if self.chatrooms is not None:
             self.chatrooms.roomsctrl.PublicRoomMessage(msg, msg.msg)
             self.frame.pluginhandler.PublicRoomMessageNotification(msg.room, msg.user, msg.msg)
@@ -858,10 +758,7 @@ class NetworkEventProcessor:
                 ticker = self.config.sections["ticker"]["default"]
 
             if ticker:
-                encoding = self.config.sections["server"]["enc"]
-                if msg.room in self.config.sections["server"]["roomencoding"]:
-                    encoding = self.config.sections["server"]["roomencoding"][msg.room]
-                self.queue.put(slskmessages.RoomTickerSet(msg.room, ToBeEncoded(ticker, encoding)))
+                self.queue.put(slskmessages.RoomTickerSet(msg.room, ticker))
 
         self.logMessage("%s %s" % (msg.__class__, vars(msg)), 4)
 
@@ -988,14 +885,6 @@ class NetworkEventProcessor:
 
     def SayChatRoom(self, msg):
 
-        if msg.room in self.config.sections["server"]["roomencoding"]:
-            encodings = [self.config.sections["server"]["roomencoding"][msg.room]] + self.config.sections["server"]["fallbackencodings"]
-            encodings.append(self.config.sections["server"]["enc"])
-        else:
-            encodings = [self.config.sections["server"]["enc"]] + self.config.sections["server"]["fallbackencodings"]
-
-        msg.msg = findBestEncoding(msg.msg, encodings)
-
         if self.chatrooms is not None:
             event = self.frame.pluginhandler.IncomingPublicChatEvent(msg.room, msg.user, msg.msg)
             if event is not None:
@@ -1028,8 +917,6 @@ class NetworkEventProcessor:
 
         if msg.files is not None:
             self.GetUserStats(msg)
-        elif msg.userexists and msg.files is None:
-            self.queue.put(slskmessages.GetUserStats(msg.user))
 
     def PrivilegedUsers(self, msg):
 
@@ -1037,7 +924,7 @@ class NetworkEventProcessor:
             self.transfers.setPrivilegedUsers(msg.users)
             self.logMessage(_("%i privileged users") % (len(msg.users)))
             self.queue.put(slskmessages.HaveNoParent(1))
-            self.queue.put(slskmessages.GetUserStats(self.config.sections["server"]["login"]))
+            self.queue.put(slskmessages.AddUser(self.config.sections["server"]["login"]))
             self.frame.pluginhandler.ServerConnectNotification()
         else:
             self.logMessage("%s %s" % (msg.__class__, vars(msg)), 4)
@@ -1053,15 +940,10 @@ class NetworkEventProcessor:
         mins = msg.seconds // 60
         hours = mins // 60
         days = hours // 24
-        wheretogetthem = " " + _("You can acquire privileges by donating at %(url)s") % {
-            'url': 'http://www.slsknet.org/userlogin.php?' + urlencode({
-                'username': self.config.sections["server"]["login"]
-            })
-        }
 
         if msg.seconds == 0:
             self.logMessage(
-                _("You have no privileges left. They are not needed for Nicotine+ to function properly.") + wheretogetthem
+                _("You have no privileges left. They are not necessary, but allow your downloads to be queued ahead of non-privileged users.")
             )
         else:
             self.logMessage(
@@ -1070,7 +952,7 @@ class NetworkEventProcessor:
                     'hours': hours % 24,
                     'minutes': mins % 60,
                     'seconds': msg.seconds % 60
-                } + wheretogetthem
+                }
             )
 
         self.privileges_left = msg.seconds
@@ -1079,48 +961,29 @@ class NetworkEventProcessor:
         self.logMessage("%s" % (msg.msg))
 
     def ChildDepth(self, msg):
-        # TODO: Distributed search messages need to implemented
+        # TODO: Implement me
         self.logMessage("%s %s" % (msg.__class__, vars(msg)), 4)
 
     def BranchLevel(self, msg):
-        # TODO: Distributed search messages need to implemented
+        # TODO: Implement me
         self.logMessage("%s %s" % (msg.__class__, vars(msg)), 4)
 
     def BranchRoot(self, msg):
-        # TODO: Distributed search messages need to implemented
+        # TODO: Implement me
         self.logMessage("%s %s" % (msg.__class__, vars(msg)), 4)
 
     def DistribChildDepth(self, msg):
-        # TODO: Distributed search messages need to implemented
+        # TODO: Implement me
         self.logMessage("%s %s" % (msg.__class__, vars(msg)), 4)
 
     def DistribBranchLevel(self, msg):
-        # TODO: Distributed search messages need to implemented
+        # TODO: Implement me
         self.logMessage("%s %s" % (msg.__class__, vars(msg)), 4)
 
     def DistribBranchRoot(self, msg):
-        # TODO: Distributed search messages need to implemented
+        # TODO: Implement me
         self.logMessage("%s %s" % (msg.__class__, vars(msg)), 4)
 
-    def DistribMessage9(self, msg):
-        # TODO: Distributed search messages need to implemented
-        self.logMessage("%s %s" % (msg.__class__, vars(msg)), 4)
-
-    def DummyMessage(self, msg):
-        self.logMessage("%s %s" % (msg.__class__, vars(msg)), 4)
-
-    def Msg12547(self, msg):
-        self.logMessage("%s %s" % (msg.__class__, vars(msg)), 4)
-
-    def ParentInactivityTimeout(self, msg):
-        pass
-
-    def SearchInactivityTimeout(self, msg):
-        pass
-
-    def MinParentsInCache(self, msg):
-        pass
-
     def WishlistInterval(self, msg):
         if self.search is not None:
             self.search.SetInterval(msg)
@@ -1211,13 +1074,10 @@ class NetworkEventProcessor:
         else:
             self.logMessage("%s %s" % (msg.__class__, vars(msg)), 4)
 
-    def QueuedDownloads(self, msg):
-        self.logMessage("%s %s" % (msg.__class__, vars(msg)), 4)
-
     def GetPeerAddress(self, msg):
 
         for i in self.peerconns:
-            if i.username == msg.user and i.addr is None:
+            if i.username == msg.user and i.addr == "in_progress":
                 if msg.port != 0 or i.tryaddr == 10:
                     if i.tryaddr == 10:
                         self.logMessage(
@@ -1237,6 +1097,7 @@ class NetworkEventProcessor:
 
                     i.addr = (msg.ip, msg.port)
                     i.tryaddr = None
+
                     self.queue.put(slskmessages.OutConn(None, i.addr))
 
                     for j in i.msgs:
@@ -1283,6 +1144,13 @@ class NetworkEventProcessor:
                 del self.ipignore_requested[msg.user]
                 return
 
+            cc = self.geoip.get_all(msg.ip).country_short
+
+            if cc == "-":
+                cc = ""
+
+            self.frame.HasUserFlag(msg.user, "flag_" + cc)
+
             # From this point on all paths should call
             # self.frame.pluginhandler.UserResolveNotification precisely once
             if msg.user in self.PrivateMessageQueue:
@@ -1293,36 +1161,15 @@ class NetworkEventProcessor:
 
             self.ip_requested.remove(msg.user)
 
-            import socket
-            if self.geoip:
-                cc = self.geoip.country_name_by_addr(msg.ip)
-                cn = self.geoip.country_code_by_addr(msg.ip)
-                if cn is not None:
-                    self.frame.HasUserFlag(msg.user, "flag_"+cn)
-            else:
-                cc = ""
-
-            if cc:
+            if cc != "":
                 cc = " (%s)" % cc
-            else:
-                cc = ""
 
-            try:
-                hostname = socket.gethostbyaddr(msg.ip)[0]
-                message = _("IP address of %(user)s is %(ip)s, name %(host)s, port %(port)i%(country)s") % {
-                    'user': msg.user,
-                    'ip': msg.ip,
-                    'host': hostname,
-                    'port': msg.port,
-                    'country': cc
-                }
-            except:
-                message = _("IP address of %(user)s is %(ip)s, port %(port)i%(country)s") % {
-                 'user': msg.user,
-                 'ip': msg.ip,
-                 'port': msg.port,
-                 'country': cc
-                }
+            message = _("IP address of %(user)s is %(ip)s, port %(port)i%(country)s") % {
+                'user': msg.user,
+                'ip': msg.ip,
+                'port': msg.port,
+                'country': cc
+            }
 
             self.logMessage(message)
             self.frame.pluginhandler.UserResolveNotification(msg.user, msg.ip, msg.port, cc)
@@ -1372,7 +1219,6 @@ class NetworkEventProcessor:
         self.logMessage("%s %s" % (msg.__class__, vars(msg)), 3)
 
     def ConnectToPeer(self, msg):
-
         init = slskmessages.PeerInit(None, msg.user, msg.type, 0)
 
         self.queue.put(slskmessages.OutConn(None, (msg.ip, msg.port), init))
@@ -1385,7 +1231,6 @@ class NetworkEventProcessor:
                 init=init
             )
         )
-
         self.logMessage("%s %s" % (msg.__class__, vars(msg)), 3)
 
     def CheckUser(self, user, addr):
@@ -1410,14 +1255,14 @@ class NetworkEventProcessor:
         if self.config.sections["transfers"]["friendsonly"]:
             return 0, _("Sorry, friends only")
 
-        if not self.geoip or not self.config.sections["transfers"]["geoblock"]:
+        if not self.config.sections["transfers"]["geoblock"]:
             return 1, _("geoip")
 
-        cc = None
+        cc = "-"
         if addr is not None:
-            cc = self.geoip.country_code_by_addr(addr)
+            cc = self.geoip.get_all(addr).country_short
 
-        if not cc:
+        if cc == "-":
             if self.config.sections["transfers"]["geopanic"]:
                 return 0, _("Sorry, geographical paranoia")
             else:
@@ -1446,7 +1291,7 @@ class NetworkEventProcessor:
                             'real_ip': u_ip
                         }
                         self.logMessage(warning, 1)
-                        print warning
+                        print(warning)
                         return 1
         return 0
 
@@ -1455,12 +1300,13 @@ class NetworkEventProcessor:
         if peerconn is None:
             return
 
-        for i in self.peerconns[:]:
-            if i.conn == peerconn:
-                if not self.protothread.socketStillActive(i.conn):
-                    self.queue.put(slskmessages.ConnClose(i.conn))
+        if not self.protothread.socketStillActive(peerconn):
+            self.queue.put(slskmessages.ConnClose(peerconn))
+
+            for i in self.peerconns:
+                if i.conn == peerconn:
                     self.peerconns.remove(i)
-                break
+                    break
 
     def UserInfoReply(self, msg):
         for i in self.peerconns:
@@ -1468,6 +1314,7 @@ class NetworkEventProcessor:
                 # probably impossible to do this
                 if i.username != self.config.sections["server"]["login"]:
                     self.userinfo.ShowInfo(i.username, msg)
+                    break
 
     def UserInfoRequest(self, msg):
 
@@ -1533,7 +1380,7 @@ class NetworkEventProcessor:
 
         try:
             if sys.platform == "win32":
-                userpic = u"%s" % self.config.sections["userinfo"]["pic"]
+                userpic = "%s" % self.config.sections["userinfo"]["pic"]
                 if not os.path.exists(userpic):
                     userpic = self.config.sections["userinfo"]["pic"]
             else:
@@ -1542,10 +1389,10 @@ class NetworkEventProcessor:
             f = open(userpic, 'rb')
             pic = f.read()
             f.close()
-        except:
+        except Exception:
             pic = None
 
-        descr = self.encode(eval(self.config.sections["userinfo"]["descr"], {})).replace("\n", "\r\n")
+        descr = unescape(self.config.sections["userinfo"]["descr"])
 
         if self.transfers is not None:
             totalupl = self.transfers.getTotalUploadsAllowed()
@@ -1572,27 +1419,20 @@ class NetworkEventProcessor:
             if i.conn is msg.conn.conn and self.userbrowse is not None:
                 if i.username != self.config.sections["server"]["login"]:
                     self.userbrowse.ShowInfo(i.username, msg)
+                    break
 
     def FileSearchResult(self, msg):
+        if self.search is not None:
+            if msg.conn.addr:
+                country = self.geoip.get_all(msg.conn.addr[0]).country_short
+            else:
+                country = ""
 
-        for i in self.peerconns:
+            if country == "-":
+                country = ""
 
-            if i.conn is msg.conn.conn and self.search is not None:
-
-                if self.geoip:
-                    if i.addr:
-                        country = self.geoip.country_code_by_addr(i.addr[0])
-                    else:
-                        country = ""
-                else:
-                    country = ""
-
-                if country is None:
-                    country = ""
-
-                self.search.ShowResult(msg, i.username, country)
-
-                self.ClosePeerConnection(i.conn)
+            self.search.ShowResult(msg, msg.user, country)
+            self.ClosePeerConnection(msg.conn.conn)
 
         self.logMessage("%s %s" % (msg.__class__, vars(msg)), 4)
 
@@ -1650,6 +1490,7 @@ class NetworkEventProcessor:
                 for j in i.msgs:
                     if j.__class__ in [slskmessages.TransferRequest, slskmessages.FileRequest] and self.transfers is not None:
                         self.transfers.gotCantConnect(j.req)
+                break
 
     def ConnectToPeerTimeout(self, msg):
 
@@ -1667,6 +1508,7 @@ class NetworkEventProcessor:
                 for j in i.msgs:
                     if j.__class__ in [slskmessages.TransferRequest, slskmessages.FileRequest] and self.transfers is not None:
                         self.transfers.gotCantConnect(j.req)
+                break
 
     def TransferTimeout(self, msg):
         if self.transfers is not None:
@@ -1722,9 +1564,6 @@ class NetworkEventProcessor:
         else:
             self.logMessage("%s %s" % (msg.__class__, vars(msg)), 4)
 
-    def PlaceholdUpload(self, msg):
-        self.logMessage("%s %s" % (msg.__class__, vars(msg)), 4)
-
     def PlaceInQueueRequest(self, msg):
         if self.transfers is not None:
             self.transfers.PlaceInQueueRequest(msg)
@@ -1748,20 +1587,14 @@ class NetworkEventProcessor:
             self.logMessage("%s %s" % (msg.__class__, vars(msg)), 4)
 
     def FolderContentsResponse(self, msg):
-
         if self.transfers is not None:
-
-            for i in self.peerconns:
-                if i.conn is msg.conn.conn:
-                    username = i.username
-
             # Check for a large number of files
             many = False
             folder = ""
             files = []
 
-            for i in msg.list.keys():
-                for j in msg.list[i].keys():
+            for i in list(msg.list):
+                for j in list(msg.list[i]):
                     if os.path.commonprefix([i, j]) == j:
                         files = msg.list[i][j]
                         numfiles = len(files)
@@ -1770,6 +1603,11 @@ class NetworkEventProcessor:
                             folder = j
 
             if many:
+                for i in self.peerconns:
+                    if i.conn is msg.conn.conn:
+                        username = i.username
+                        break
+
                 self.frame.download_large_folder(username, folder, files, numfiles, msg)
             else:
                 self.transfers.FolderContentsResponse(msg)
@@ -1807,21 +1645,13 @@ class NetworkEventProcessor:
         else:
             self.logMessage(_("Unknown tunneled message: %s") % (vars(msg)), 4)
 
-    def ExactFileSearch(self, msg):
-        ''' Depreciated '''
-
-        self.logMessage("%s %s" % (msg.__class__, vars(msg)), 4)
-        for i in self.peerconns:
-            if i.conn == msg.conn.conn:
-                user = i.username
-                self.shares.processExactSearchRequest(msg.searchterm, user, msg.searchid, direct=1, checksum=msg.checksum)
-
     def FileSearchRequest(self, msg):
         self.logMessage("%s %s" % (msg.__class__, vars(msg)), 4)
         for i in self.peerconns:
             if i.conn == msg.conn.conn:
                 user = i.username
                 self.shares.processSearchRequest(msg.searchterm, user, msg.searchid, direct=1)
+                break
 
     def SearchRequest(self, msg):
         self.logMessage("%s %s" % (msg.__class__, vars(msg)), 4)
@@ -1856,6 +1686,7 @@ class NetworkEventProcessor:
 
             responddistributedtimeout = RespondToDistributedSearchesTimeout(self.callback)
             self.respondDistributedTimer = threading.Timer(self.config.sections["searches"]["distrib_ignore"], responddistributedtimeout.timeout)
+            self.respondDistributedTimer.setDaemon(True)
             self.respondDistributedTimer.start()
         else:
             # Always respond
@@ -1876,7 +1707,7 @@ class NetworkEventProcessor:
 
             if not self.GetDistribConn():
 
-                user = self.distribcache.keys()[0]
+                user = list(self.distribcache.keys())[0]
                 addr = self.distribcache[user]
 
                 self.queue.put(slskmessages.SearchParent(addr[0]))
@@ -1885,9 +1716,6 @@ class NetworkEventProcessor:
 
         self.logMessage("%s %s" % (msg.__class__, vars(msg)), 4)
 
-    def DistribAlive(self, msg):
-        self.logMessage("%s %s" % (msg.__class__, vars(msg)), 4)
-
     def GetDistribConn(self):
         for i in self.peerconns:
             if i.init.type == 'D':
@@ -1900,7 +1728,7 @@ class NetworkEventProcessor:
 
         if len(self.distribcache) > 0:
 
-            user = self.distribcache.keys()[0]
+            user = list(self.distribcache.keys())[0]
             addr = self.distribcache[user]
 
             self.queue.put(slskmessages.SearchParent(addr[0]))
@@ -1926,17 +1754,6 @@ class NetworkEventProcessor:
 
     def RoomTickerState(self, msg):
 
-        if msg.room in self.config.sections["server"]["roomencoding"]:
-            encodings = [self.config.sections["server"]["roomencoding"][msg.room]] + self.config.sections["server"]["fallbackencodings"]
-            encodings.append(self.config.sections["server"]["enc"])
-        else:
-            encodings = [self.config.sections["server"]["enc"]] + self.config.sections["server"]["fallbackencodings"]
-
-        unicodes = {}
-        for user, bytes in msg.msgs.iteritems():
-            unicodes[user] = findBestEncoding(bytes, encodings)
-
-        msg.msgs = unicodes
         if self.chatrooms is not None:
             self.chatrooms.roomsctrl.TickerSet(msg)
 
@@ -1944,13 +1761,6 @@ class NetworkEventProcessor:
 
     def RoomTickerAdd(self, msg):
 
-        if msg.room in self.config.sections["server"]["roomencoding"]:
-            encodings = [self.config.sections["server"]["roomencoding"][msg.room]] + self.config.sections["server"]["fallbackencodings"]
-            encodings.append(self.config.sections["server"]["enc"])
-        else:
-            encodings = [self.config.sections["server"]["enc"]] + self.config.sections["server"]["fallbackencodings"]
-
-        msg.msg = findBestEncoding(msg.msg, encodings)
         if self.chatrooms is not None:
             self.chatrooms.roomsctrl.TickerAdd(msg)
 
@@ -1970,7 +1780,7 @@ class NetworkEventProcessor:
                 f.write(time.strftime("%c"))
                 f.write(" %s\n" % message)
                 f.close()
-            except IOError, error:
+            except IOError as error:
                 self.logMessage(_("Couldn't write to transfer log: %s") % error)
 
         if toUI:
